@@ -6,7 +6,7 @@
 (define  (split-divide)
 	(duration::ChangeNoteDurationInTicks (/ (duration::GetBaseDurationInTicks) 2) (d-GetDots)))
 
-(if (and (d-GetNotes) (not (string-ci=?  (d-GetType) "Appending"))) ; DuplicateChord only works ON notes, not in the appending position. 
+(if (and (Note?) (not (Appending?))) ; DuplicateChord only works ON notes, not in the appending position. 
 
 (begin
 
@@ -24,14 +24,22 @@
 		 (d-DuplicateChord)
 		 (d-MoveCursorRight)
 		 #t))
-	((3) 	(begin ;8th-triplet
-		(d-StartTriplet)
-		(split-divide)
-		(d-DuplicateChord)
-		(d-DuplicateChord)
-		(d-MoveCursorRight)
-		(d-EndTuplet)
-		#t))
+	((3) 	(if (= 1 (d-GetDots))
+		    (begin
+		      (d-RemoveDot)
+		      (d-Diminish)
+		      (d-DuplicateChord)
+		      (d-DuplicateChord)
+		      (d-MoveCursorRight)
+		      #t)
+		    (begin ;8th-triplet
+		      (d-StartTriplet)
+		      (split-divide)
+		      (d-DuplicateChord)
+		      (d-DuplicateChord)
+		      (d-MoveCursorRight)
+		      (d-EndTuplet)
+		      #t)))
 	((4) (begin  ;16th
 		 (split-divide)
 		 (split-divide)
