@@ -48,10 +48,18 @@ typedef GList objnode;
 
 typedef enum 
   {
+    NOT_UNDO_REDO = 0,
     UNDO,
     REDO
   }unre_mode;
 
+typedef enum 
+  {
+    SAVE_NORMAL=0,
+    SAVE_TEMPLATE,
+    SAVE_COPY
+  } DenemoSaveType;
+
 
 
 typedef enum DenemoGraphicType
@@ -162,9 +170,16 @@ typedef enum input_mode {
  * 
  */
 typedef enum  action_type {
-  ACTION_INSERT,
-  ACTION_DELETE,
-  ACTION_CHANGE
+  ACTION_INSERT,//0
+  ACTION_DELETE,//1
+  ACTION_CHANGE,//2
+  ACTION_SNAPSHOT,//3
+  ACTION_STAGE_START,//4
+  ACTION_STAGE_END,//5
+  ACTION_SCRIPT_ERROR,//6
+  ACTION_MEASURE_REMOVE,//7
+  ACTION_MEASURE_CREATE,//8
+  ACTION_NOOP = -1//
 }action_type;
 
 /**
@@ -249,11 +264,8 @@ typedef struct
   gint volume;	/**< Volume used for midi/csound playback */
   gboolean mute_volume; /**< mute Volume of voices playback */
   /* Back to Hiller stuff */
-  GString *staff_name;
-  /* RTS: I've introduced the staff name here, the other two are versions
-     of the voice name; however I'm still sticking to the unwritten convention
-     that each staff's voices are contiguous in si->thescore. Without this you
-     can't have same named voices in different staffs. */
+  //  GString *staff_name;
+
   GString *denemo_name; /**< denemo_name gets copied into lily_name */
   GString *lily_name; /**< this is the name of the staff that is export to lilypond */
   GString *midi_instrument; /**< midi instrument name used for the staff when exported via midi */
@@ -277,15 +289,13 @@ typedef struct
   GList *tone_store; /**< list of pitches and durations used a source for
 			the notes in this staff
 			the data are tone* */
-  GString *staff_prolog;/**< Customised version of the LilyPond prolog defining the music of this staff */
+
   GList *staff_directives;/**< List of DenemoDirective for the staff context, (only relevant for primary staff)*/
   GList *voice_directives;/**< List of DenemoDirective for the voice context */
-  GString *lyrics_prolog;/**< (Unused)Customised version of the LilyPond prolog defining the lyrics of this staff */
-  GList *midi_events;/*< midi_events to be output at start of the track, after scorewide and movementwide ones */
 
-  GString *figures_prolog;/**<  (Unused)Customised version of the LilyPond prolog defining the figured bass of this staff */
-  GString *fakechords_prolog;/**<  (Unused)Customised version of the LilyPond prolog defining the chord symbols of this staff */
-}DenemoStaff;
+  GList *midi_events;/*< cache of midi events to be output at start of the track, after scorewide and movementwide ones created from staff_directives*/
+
+} DenemoStaff;
 
 /* The ->data part of each staffnode points to a staff structure */
 
@@ -357,16 +367,17 @@ typedef struct DeviceManagerDevice
  */
 typedef struct DenemoPrefs
 {
+  GString *profile; /**< Which set of commands and shortcuts to load, and which initialization of scheme to run */
   GString *lilypath; /**< This is the executable or full path to the lilypond executable */
   GString *midiplayer; /**< This is the external midifile player */ 
   GString *audioplayer; /**< This is used for playing audio files created from csound or other */
   gboolean playbackoutput; /**< This is a switch to turn on playing the csound output 
 			     wav file after rendering or not */
-  GString *shortcut_filename; /**< file containing shortcuts for actions */
   gboolean immediateplayback; /**< This options sends audio directly to synth as notes 
 				are being entered */
   gboolean startmidiin; /**< try to start midi in on startup */
   gboolean applytoselection; /**< apply commands to selection */
+  gboolean quickshortcuts;/**< TRUE if pressing a key while hovering over a menu item sets a shortcut */
   gboolean overlays; /*< whether overlays or insert should be used with pitch entry */
   gboolean continuous; /*< whether pitch entry overlays should cross barlines */
   gboolean lilyentrystyle;  
@@ -374,7 +385,7 @@ typedef struct DenemoPrefs
   gint mode; /**< mode to start in */
   gboolean modal;/**< whether mode-switching menu is desired */
   gboolean persistence;/**< whether Denemo should re-use settings over sessions */
-  gint zoom; /**< percent zoom  1-100 */
+  gint zoom; /**< percent zoom of score area display  1-100 */
   gint system_height; /**< percent of display height per system  1-100 */
   gboolean cursor_highlight; /** Should the cursor be highlighted */
   
@@ -390,6 +401,7 @@ typedef struct DenemoPrefs
 
   gboolean visible_directive_buttons; /**< This option makes the hbox containing score/movement directives visible */
 
+  gboolean disable_undo; /**< Do not collect undo information */
   gboolean saveparts; /**< Automatically save parts*/
   gboolean autosave; /**< Auto save data */
   gint autosave_timeout;
@@ -397,7 +409,7 @@ typedef struct DenemoPrefs
   gint maxhistory;/**< how long a history of used files to retain */
   GString *browser; /**< Default browser string */
   
-  gchar *midi_audio_output; /**< How the user wants to deal with audio/midi output */
+  const gchar *midi_audio_output; /**< How the user wants to deal with audio/midi output */
   GString *csoundcommand; /**< command used to execute csound */
   GString *csoundorcfile; /**< Path to .orc file used for csound playback */
   gboolean rtcs; /**< Real time csound */
@@ -449,11 +461,15 @@ typedef struct DenemoDirective
   gint gx, gy; /**< x and y offsets in pixels for the graphic */
   GString *graphic_name; /**< name of the graphic to be drawn the suffixes .svg or .png will be searched for */
 
-  /* warning these values cannot be changed without bumping the denemo file format version */
+  /* warnings
+     1) these values cannot be changed without bumping the denemo file format version
+     2) if they are to be available to scheme scripts (normal) you must insert the code in view.c*/
 #define DENEMO_OVERRIDE_LILYPOND (1<<0)
-#define DENEMO_OVERRIDE_UNUSED (1<<1)
+#define DENEMO_ALT_OVERRIDE (1<<1)
 #define DENEMO_OVERRIDE_GRAPHIC (1<<2)
 #define DENEMO_OVERRIDE_EDITOR (1<<3)
+#define DENEMO_OVERRIDE_AFFIX (1<<4)
+#define DENEMO_OVERRIDE_TAGEDIT (1<<5)
 
 
 
@@ -550,6 +566,9 @@ typedef enum MidiDestination {
   MIDITOSCORE = 0,
   MIDITHRU = (1<<1),
   MIDIRECORD = (1<<2),
+  MIDIPLAYALONG = (1<<5),
+  MIDICONDUCT = (1<<6),
+
 } MidiDestination;
 
 typedef enum DenemoViewType {
@@ -583,19 +602,28 @@ typedef struct
 	GList *instruments;  // List to contain a list of Instruments of given type
 }InstrumentList;
 
+typedef struct DenemoPosition { /**<Represents a position in a Score */
+  gint movement;
+  gint staff;
+  gint measure;
+  gint object;/**< 0 means no object */
+  gboolean appending;/**< if cursor is in appending position */
+  gboolean offend;/**< cursor is shown red in this case, obscure connection with appending */
+} DenemoPosition;
+
+
 /**
  * Contains data required for undo/redo operation 
  * Borrowed idea from GScore
  */
-typedef struct unre_data
+typedef struct DenemoUndoData
 {
-  gpointer object;    /* pointer to object to be undone/redone */
-  gint staffnum;      /* staff number */     
-  gint measurenum;    /* measure number */
-  gint position;      /* position in bar */
-  enum action_type action; /*action type */
-
-}unre_data;
+  enum action_type action; /*action type must come first*/
+  
+  DenemoObject* object;    /* pointer to object to be undone/redone */
+  DenemoPosition position; /* position where delete/insert took place */
+ 
+} DenemoUndoData;
  
 
 /**
@@ -634,15 +662,6 @@ typedef struct DenemoScriptParam { /**< commands called by scripts use one of th
 
 
 
-typedef struct DenemoPosition { /**<Represents a position in a Score */
-  gint movement;
-  gint staff;
-  gint measure;
-  gint object;/**< 0 means no object */
-  gboolean appending;/**< if cursor is in appending position */
-  gboolean offend;/**< cursor is shown red in this case, obscure connection with appending */
-} DenemoPosition;
-
 
 
 
@@ -715,6 +734,7 @@ typedef struct DenemoScore
   gdouble rightmost_time; /**< MIDI time of rightmost visible object */
 
   gdouble zoom;/**< zoom setting for this movement */
+  gdouble preview_zoom; /**< zoom  of print preview pane */
   gdouble system_height;/**< fraction of scorearea height allotted to one system (line) in this movement of the score, the same for all lines */
   //Settings for the views on this movement
 
@@ -800,8 +820,8 @@ typedef struct DenemoScore
   /*list of undo data */
   GQueue *undodata;
   GQueue *redodata;
-  gint undo_redo_mode;
-
+  gint undo_guard;
+  gboolean redo_invalid;/*< the re-do queue is awaiting freeing and should not be used */
   
 
  
@@ -820,24 +840,14 @@ typedef struct DenemoScore
 typedef struct DenemoGUI
 {
   gint id; /* A unique id, not repeated for this run of the Denemo program */
-  /* Fields used fairly directly for drawing */
-  GtkWidget *page;
-  GtkWidget *scorearea;
-  GdkPixmap *pixmap;
-  GtkObject *vadjustment;
-  GtkWidget *vscrollbar;
-  GtkObject *hadjustment;
-  GtkWidget *hscrollbar;
+
   DenemoViewType view;/**< The current view */
 #define DENEMO_MAX_SYSTEMS (100) /**< Number of lines of music that can be displayed */
   gint lefts[DENEMO_MAX_SYSTEMS];/**< an array to hold the leftmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
   gint rights[DENEMO_MAX_SYSTEMS];/**< an array to hold the rightmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
   gint scales[DENEMO_MAX_SYSTEMS];/**< an array to hold the percent horizontal scaling of each system in the last-drawn score, used for determining the mouse position on the music */
 
-  GtkWidget *printarea;/**< area holding a print preview */
-  GtkWidget *printvscrollbar;/**< scrollbar widget for printarea */
-  GtkWidget *printhscrollbar;/**< scrollbar widget for printarea */
-  GdkPixbuf *pixbuf;/**< print preview pixbuf */
+
   GtkWidget *buttonboxes;/**< box for boxes showing directives */
   GtkWidget *buttonbox;/**< box for buttons accessing DenemoDirectives attached to the whole score */
 
@@ -873,8 +883,10 @@ typedef struct DenemoGUI
   GList *anchors;/**< anchors in the LilyPond text at points where that can be edited */
 
   GString *filename;/**< the filename to save to */
+  GString *tabname;/**< the name of windows tab */
   GString *autosavename;/**< the filename to autosave to, full path */
 
+  gint undo_level;/**< level of script nesting 0 = staging point for undo to return to */
   gboolean notsaved;/**< edited since last save */
   guint changecount;/**< number of edits since score loaded */
   guint lilysync;/**< value of changecount when the Lily text was last refreshed */
@@ -934,17 +946,30 @@ struct cs_callback
  */
 struct DenemoRoot
 {
+  gchar *schemeinit;/* filename for scheme code to run on startup */
+  /* Fields used fairly directly for drawing */
+  GtkWidget *page;
+  GtkWidget *scorearea;
+  GtkObject *vadjustment;
+  GtkWidget *vscrollbar;
+  GtkObject *hadjustment;
+  GtkWidget *hscrollbar;
+
+  GtkWidget *printarea;/**< area holding a print preview */
+  GtkWidget *printvscrollbar;/**< scrollbar widget for printarea */
+  GtkWidget *printhscrollbar;/**< scrollbar widget for printarea */
+  GdkPixbuf *pixbuf;/**< print preview pixbuf */
   /* window state */
   gint width;
   gint height;
   gboolean maximized;
+  gint color;/**< the color of the background */
   keymap *map; /**< pointer to data describing each of the Denemo commands and their keyboard shortcuts */
   gchar *last_merged_command;/**<filename of last command merged into the menu system */
   gint last_keyval, last_keystate;/**< most recent keypress which successfully invoked a command */
   GList *guis; /**< the list of DenemoGUI objects, representing pieces of music
 		  simultaneously open */
   DenemoPrefs prefs;  /**< Preferences stored on exit and re-loaded on startup */
-  
   gint autosaveid;/**< autosave timer id: only one musical score is being autosaved at present */
   gint accelerator_status; /**< if the accelerators have been saved, or extra ones for special keys defined  */
   GtkUIManager *ui_manager;  /**< UI manager */
@@ -964,10 +989,10 @@ struct DenemoRoot
   GtkWidget *InsertModeMenu;/**< Menu to give the note editing facilities in Insert mode */
   GtkWidget *EditModeMenu;/**< Menu to give the note editing facilities in Edit mode */
   GtkWidget *ModelessMenu;/**< Menu to give the note editing facilities when used without modes */
-  gboolean QuickShortcutEdits;/**< TRUE if pressing a key while hovering over a menu item sets a shortcut */
   GList *continuations;
   struct RhythmPattern *singleton_rhythms[256]; /**< rhythm patterns for the EntryToolbar */
   gboolean ScriptRecording;/**< TRUE when menuitems presses are being recorded as scheme script*/
+  gint keyboard_state;/**< state of last keypress/release */
   GtkWidget *ScriptView; /**< a GtkTextView containing a scheme script */
 }  Denemo; /**< The root object. */
 
