@@ -205,14 +205,14 @@ set_width_to_work_with (DenemoGUI * gui)
 
 #if 0
       si->widthtoworkwith
-	= (double)(gui->scorearea->allocation.width*((int)(1/si->system_height ))/si->zoom
+	= (double)(Denemo.scorearea->allocation.width*((int)(1/si->system_height ))/si->zoom
 		   - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME));
 #else
       //the total line length for drawing DenemoObjects onto the screen
       // this length will be divided amongst the systems (line).
       // This length is in "pixels", the Denemo unit of display, which corresponds to a screen pixel when zoom ==1.0
       si->widthtoworkwith
-	= (gint)((gui->scorearea->allocation.width/si->zoom
+	= (gint)((Denemo.scorearea->allocation.width/si->zoom
 	   - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME))*((int)(1/si->system_height )));
 
 #endif
@@ -273,29 +273,26 @@ reset_cursor_stats (DenemoScore * si)
 /**
  *  General function for inserting a DenemoObject
  *  into the score
+ * the object is inserted at position gui->si->cursor_x in the list of objects (counting from 0)
+ * gui->si->cursor_x is incremented
+ * gui->si->currentobject is set to the object at the new cursor_x position, unless this is too large, or we have cursor_appending in which case it is set to the last object.
  */
 void
 object_insert (DenemoGUI * gui, DenemoObject * mudela_obj_new)
 {
   DenemoScore *si = gui->si;
-  unre_data *undo = (unre_data *) g_malloc (sizeof (unre_data));
 
   declarecurmudelaobj;
 
-#if 0
-  //this makes writing scripts difficult, and anyway assumes Denemo knows where the barlines are
-  /* First, check to see if the operation would add something before an
-   * indicator of a time signature change. This would be bad, so don't
-   * allow it to happen */
-  if (curmudelaobj && curmudelaobj->type == TIMESIG && !si->cursor_appending)
+  /* update undo information */
+  DenemoUndoData *undo;
+  if (!si->undo_guard)
     {
-      si->cursor_x++;
-      if (si->currentobject->next)
-	si->currentobject = si->currentobject->next;
-      else
-	si->cursor_appending = TRUE;
+      undo = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));
+      // should not be needed, we are inserting the object undo->object = dnm_clone_object (mudela_obj_new);
+      //do position after inserting, so we can go back to it to delete
     }
-#endif
+
   si->currentmeasure->data =
     g_list_insert ((objnode *) si->currentmeasure->data,
 		   mudela_obj_new, si->cursor_x);
@@ -307,13 +304,14 @@ object_insert (DenemoGUI * gui, DenemoObject * mudela_obj_new)
 	  find_xes_in_all_measures (si);
   }
 
-  /* update undo information */
-  undo->position = si->cursor_x;
-  undo->measurenum = si->currentmeasurenum;
-  undo->staffnum = si->currentstaffnum;
-  undo->object = dnm_clone_object (mudela_obj_new);
-  undo->action = ACTION_INSERT;
 
+ if (!si->undo_guard)
+    {
+      get_position(si,&undo->position);
+      undo->position.appending = 0;
+      undo->action = ACTION_INSERT;
+      update_undo_info (si, undo);
+    }
 
   si->cursor_x++;
   if (si->cursor_appending)
@@ -321,9 +319,16 @@ object_insert (DenemoGUI * gui, DenemoObject * mudela_obj_new)
   else
     si->currentobject = g_list_nth ((objnode *) si->currentmeasure->data,
 				    si->cursor_x);
-  // g_print("object %p cursorx %d length %d\n", si->currentobject, si->cursor_x, g_list_length(si->currentmeasure->data));
-  update_undo_info (si, undo);
 
+  if(si->currentobject==NULL) {
+    g_warning("problematic parameters on insert %d out of %d objects", si->cursor_x+1, g_list_length((objnode *) si->currentmeasure->data));
+    si->cursor_x--;
+    si->currentobject = g_list_nth ((objnode *) si->currentmeasure->data,
+				    si->cursor_x);
+  }
+
+  //g_print("object insert appending %d cursor_x %d length %d\n", si->cursor_appending, si->cursor_x, g_list_length(si->currentmeasure->data));
+ 
   score_status(gui, TRUE);
   si->markstaffnum = 0;
 }
@@ -357,7 +362,8 @@ gomeasureleft (DenemoScriptParam *param, gboolean extend_selection)
   if (gui->si->currentmeasure->prev)
     {
       gui->si->currentmeasurenum--;
-      isoffleftside (gui);
+      if(!gui->si->playingnow) //during playback cursor moves should not affect viewport
+	isoffleftside (gui);
       param->status = TRUE;
     }
   setcurrents (gui->si);
@@ -391,7 +397,8 @@ gomeasureright (DenemoScriptParam *param, gboolean extend_selection)
   if (gui->si->currentmeasure->next)
     {
       gui->si->currentmeasurenum++;
-      isoffrightside (gui);
+       if(!gui->si->playingnow) //during playback cursor moves should not affect viewport
+	isoffrightside (gui);
       setcurrents (gui->si);
       param->status = TRUE;
       if(extend_selection)
@@ -471,6 +478,7 @@ swapstaffs (GtkAction *action, gpointer param)
 	temp = gui->si->currentstaff->data;
 	if(temp->context==DENEMO_NONE ||
 	   confirm("A context is set on this staff", "You will need to alter the staff->properties->context of this and the previous staff; Proceed?")) {
+	  take_snapshot();
 	  gui->si->currentstaff->data = gui->si->currentstaff->prev->data;
 	  gui->si->currentstaff->prev->data = temp;
 	  gui->si->currentstaffnum--;
@@ -503,6 +511,7 @@ splitstaffs (GtkAction *action, gpointer param)
     return FALSE;
   if (gui->si->currentstaff && gui->si->currentstaff->next)
     {
+      take_snapshot();
       DenemoStaff *thestaff = (DenemoStaff *)gui->si->currentstaff->data;
       DenemoStaff *nextstaff = (DenemoStaff *)gui->si->currentstaff->next->data;
       if((thestaff->voicenumber==1) && (nextstaff->voicenumber!=1))
@@ -534,6 +543,7 @@ joinstaffs (GtkAction *action, gpointer param)
     return FALSE;
   if (gui->si->currentstaff && gui->si->currentstaff->prev)
     {
+      take_snapshot();
       DenemoStaff *thestaff = (DenemoStaff *)gui->si->currentstaff->data;
       DenemoStaff *prevstaff = (DenemoStaff *)gui->si->currentstaff->prev->data;
       thestaff->voicenumber=2;
@@ -611,7 +621,7 @@ gostaffup (DenemoScriptParam *param, gboolean extend_selection)
 	calcmarkboundaries (gui->si);
       show_lyrics();
       find_leftmost_allcontexts (si);
-      gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set I think it just needs si->cursorclef = si->curmeasureclef (? after beamandstemdirhelper(si)??)
+      scorearea_expose_event(NULL, NULL);
       move_viewport_up (gui);
       return param->status = TRUE;
     } else
@@ -714,7 +724,8 @@ gostaffdown (DenemoScriptParam *param, gboolean extend_selection)
 	calcmarkboundaries (gui->si);
       show_lyrics();
       find_leftmost_allcontexts (si);
-      gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
+      
+      scorearea_expose_event(NULL, NULL);
       move_viewport_down (gui);
       return param->status = TRUE;
     } else
@@ -778,7 +789,8 @@ move_left (DenemoScriptParam *param, gboolean extend_selection)
 	  si->cursor_appending = TRUE;
 	  si->currentmeasure = si->currentmeasure->prev;
 	  si->currentmeasurenum--;
-	  isoffleftside (gui);
+	  if(!si->playingnow) //during playback cursor moves should not affect viewport
+	    isoffleftside (gui);
 	  si->currentobject =
 	    g_list_last ((objnode *) si->currentmeasure->data);
 	  /* The preceding statement will set currentobject to
@@ -832,7 +844,8 @@ move_right (DenemoScriptParam *param, gboolean extend_selection)
       /* Go to the next measure */
       si->currentmeasure = si->currentmeasure->next;
       si->currentmeasurenum++;
-      isoffrightside (gui);
+      if(!si->playingnow) //during playback cursor moves should not affect viewport
+	isoffrightside (gui);
       si->currentobject = (objnode *) si->currentmeasure->data;
       si->cursor_x = 0;
       if (si->currentobject)
@@ -891,6 +904,45 @@ movecursorleft (DenemoScriptParam *param)
 return  move_left(param, FALSE);
 }
 
+//next chord that is not a rest
+gboolean cursor_to_next_note(DenemoScriptParam *param) {
+  gboolean success = FALSE;
+  while(movecursorright(param) && Denemo.gui->si->currentobject) {
+    if(Denemo.gui->si->cursor_appending) {
+      (void)cursor_to_next_note(param);
+      gtk_widget_queue_draw(Denemo.scorearea);
+    }
+    if(Denemo.gui->si->currentobject) {
+      DenemoObject *obj = Denemo.gui->si->currentobject->data;
+      if(obj->type==CHORD) {
+	chord *thechord = obj->object;
+	if(thechord->notes) {
+	  success = TRUE;
+	  break;
+	}
+      }
+    }
+  }
+  return success;
+}
+// next chord, ie single or multinote chord or rest
+gboolean cursor_to_next_chord(DenemoScriptParam *param) {
+  gboolean success = FALSE;
+  while(movecursorright(param) && Denemo.gui->si->currentobject) {
+    if(Denemo.gui->si->cursor_appending) {
+      (void) cursor_to_next_chord(param);
+      gtk_widget_queue_draw(Denemo.scorearea);
+    }
+    if(Denemo.gui->si->currentobject) {
+      DenemoObject *obj = Denemo.gui->si->currentobject->data;
+      if(obj->type==CHORD) {  
+	success = TRUE;
+	break;
+      }
+    }
+  }
+  return success;
+}
 
 /**
  * Move the cursor up one diatonic step 
@@ -996,6 +1048,33 @@ void insert_note_following_pattern(DenemoGUI *gui)  {
 #undef g
 }
 
+/* get duration of next element in current rhythm pattern  */
+
+gint get_prevailing_duration(void)  {
+  DenemoGUI *gui = Denemo.gui;
+  gint duration=0;
+  if((gui->mode&(INPUTEDIT|INPUTINSERT)) && gui->rstep) {
+    GList *start = (gui->rstep);
+    if(gui->currhythm && gui->cstep)
+      {
+	GList *objs;
+	for(objs = gui->cstep; objs; objs=objs->next) {
+	  if((((DenemoObject*)objs->data)->type == CHORD)) {
+	    duration = ((chord*)((DenemoObject*)objs->data)->object)->baseduration;
+	    break;
+	  }	 
+	}
+      }
+    else {
+      for(duration=0;duration<7;duration++)
+	if((Denemo.gui->prevailing_rhythm == Denemo.singleton_rhythms['0'+duration]) ||
+	   (Denemo.gui->prevailing_rhythm == Denemo.singleton_rhythms['r'+duration]))
+	  break;
+    }
+  }
+  return duration;
+}
+
 
 /**
  * shiftcursor: FIXME change the name of this function!
@@ -1021,6 +1100,9 @@ shiftcursor (DenemoGUI  *gui, gint note_value)
     DenemoObject *theobj =  (DenemoObject *)(gui->si->currentobject->data);
     chord *thechord;
     if(theobj->type == CHORD && (thechord = (chord*)theobj->object)->notes) {
+      store_for_undo_change(gui->si, theobj);
+      //turn off further storage of UNDO info while this takes place
+      gui->si->undo_guard++;      
       theobj->isinvisible=FALSE;
       if(g_list_length(thechord->notes)>1) {/* multi-note chord - remove and add a note */
 	gui->si->cursor_y = oldcursor_y;
@@ -1033,6 +1115,7 @@ shiftcursor (DenemoGUI  *gui, gint note_value)
       showwhichaccidentals ((objnode *) gui->si->currentmeasure->data,
 			    gui->si->curmeasurekey, gui->si->curmeasureaccs);
       }
+      gui->si->undo_guard--;
       score_status(gui, TRUE);
     }  
   } else
@@ -1072,7 +1155,7 @@ insertion_point_for_type (DenemoScore * si, DenemoObjType type) {
 void
 insertion_point (DenemoScore * si)
 {
-  //gtk_widget_draw(Denemo.gui->scorearea, NULL);//FIXME efficiency????
+  //gtk_widget_draw(Denemo.Denemo.scorearea, NULL);//FIXME efficiency????
 
   scorearea_expose_event(NULL, NULL);
 
@@ -1127,15 +1210,15 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
   DenemoScore *si = gui->si;
   DenemoObject *mudela_obj_new;
   int prognum;
-  
 
   if((mode & INPUTEDIT) && !si->cursor_appending && !(mode & INPUTRHYTHM) ) {
     changeduration(si, duration);
     return;
   }
+
   insertion_point (si);
 
-  /* Now actually create the chord */
+  /* Now actually create the chord as an object (before insertion) */
   mudela_obj_new = newchord (duration, 0, 0);
   if ((mode & INPUTNORMAL) && (rest != TRUE))
     addtone (mudela_obj_new, si->cursor_y, si->cursoraccs[si->staffletter_y],
@@ -1151,15 +1234,16 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
   object_insert (gui, mudela_obj_new);
  
   if (gui->mode&(INPUTRHYTHM)) {
-
+      if(Denemo.prefs.immediateplayback) {
 #ifdef _HAVE_FLUIDSYNTH_
-    fluid_rhythm_feedback(duration, rest, FALSE);
+	fluid_rhythm_feedback(duration, rest, FALSE);
 #else
-    if(rest)
-      playpitch(64.0*(1+duration), 60.0/(4.0*gui->si->tempo*(1<<duration)), 0.1, 2);//FIXME make a distinct noise
-      else
-	playpitch(64.0*(1+duration), 60.0/(gui->si->tempo*(1<<duration)), 0.2, 1);
+	if(rest)
+	  playpitch(64.0*(1+duration), 60.0/(4.0*gui->si->tempo*(1<<duration)), 0.1, 2);//FIXME make a distinct noise
+	else
+	  playpitch(64.0*(1+duration), 60.0/(gui->si->tempo*(1<<duration)), 0.2, 1);
 #endif
+      }
     if(!was_appending)
       movecursorleft(NULL);
 
@@ -1168,7 +1252,7 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
       DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
       playnotes (Denemo.prefs.immediateplayback, (chord *) mudela_obj_new->object,  curstaffstruct->midi_channel);
     } else {
-      Denemo.gui->last_source = INPUTKEYBOARD;
+      //Denemo.gui->last_source = INPUTKEYBOARD;
     }
   }
 }
@@ -1259,6 +1343,7 @@ changeduration (DenemoScore * si, gint duration)
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
       changedur (curmudelaobj, duration, 0);
     }
 }
@@ -1280,6 +1365,7 @@ notechange (DenemoScore * si, gboolean remove)
   
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
       if (remove == TRUE)
 	ret = removetone (curmudelaobj, si->cursor_y/*mid_c_offset*/, si->cursorclef/*dclef*/);
       else
@@ -1297,7 +1383,7 @@ notechange (DenemoScore * si, gboolean remove)
 	playnotes (Denemo.prefs.immediateplayback,
 	 (chord *) curmudelaobj->object, curstaffstruct->midi_channel);
       } else {
-      Denemo.gui->last_source = INPUTKEYBOARD;
+	// Denemo.gui->last_source = INPUTKEYBOARD;
     }
     }
   return ret;
@@ -1347,10 +1433,12 @@ displayhelper (DenemoGUI * gui)
   if ((gui->mode&(INPUTRHYTHM)) && si->currentobject && (((DenemoObject*)(si->currentobject->data))->type==CHORD)&& ((DenemoObject*)(si->currentobject->data))->starttickofnextnote>= WHOLE_NUMTICKS * si->cursortime1/si->cursortime2)
     gdk_beep(); //Signal new measures in Edit mode to catch out of step entry
 #endif
-  /*gtk_widget_draw (gui->scorearea, NULL);*/
-  gtk_widget_queue_draw (gui->scorearea);
+  /*gtk_widget_draw (Denemo.scorearea, NULL);*/
+  gtk_widget_queue_draw (Denemo.scorearea);
 }
 
+
+
 /**
  * Increment the enharmonic shift of the tone closest to the cursor.
  * @param si pointer to the DenemoScore structure
@@ -1366,6 +1454,8 @@ incrementenshift (DenemoGUI * gui, gint direction)
   
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
+
       shiftpitch (curmudelaobj, si->cursor_y, direction>0);
       showwhichaccidentals ((objnode *) si->currentmeasure->data,
 			    si->curmeasurekey, si->curmeasureaccs);
@@ -1382,15 +1472,9 @@ incrementenshift (DenemoGUI * gui, gint direction)
 	playnotes (Denemo.prefs.immediateplayback,
 		   (chord *) curmudelaobj->object, curstaffstruct->midi_channel);
       } else {
-	Denemo.gui->last_source = INPUTKEYBOARD;
+	//	Denemo.gui->last_source = INPUTKEYBOARD;
       }
-      unre_data *data = (unre_data *) g_malloc (sizeof (unre_data));
-      data->object = curmudelaobj;
-      data->position = si->cursor_x;
-      data->measurenum = si->currentmeasurenum;
-      data->staffnum = si->currentstaffnum;
-      data->action = ACTION_CHANGE;
-      update_undo_info (si, data);
+
       score_status(gui, TRUE);
     }
 }
@@ -1407,6 +1491,8 @@ setenshift (DenemoScore * si, gint enshift)
   int prognum;
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
+
       changeenshift (curmudelaobj, si->cursor_y, enshift);
       showwhichaccidentals ((objnode *) si->currentmeasure->data,
 			    si->curmeasurekey, si->curmeasureaccs);
@@ -1424,13 +1510,7 @@ setenshift (DenemoScore * si, gint enshift)
 		   (chord *) curmudelaobj->object, curstaffstruct->midi_channel);
 
       }
-      unre_data *data = (unre_data *) g_malloc (sizeof (unre_data));
-      data->object = curmudelaobj;
-      data->position = si->cursor_x;
-      data->measurenum = si->currentmeasurenum;
-      data->staffnum = si->currentstaffnum;
-      data->action = ACTION_CHANGE;
-      update_undo_info (si, data);
+
     }
 }
 
@@ -1447,6 +1527,8 @@ change_stem_directive (DenemoScore * si, enum stemdirections amount)
 
   if (curmudelaobj && curmudelaobj->type == STEMDIRECTIVE)
     {
+      store_for_undo_change (si, curmudelaobj);
+
       switch (amount)
 	{
 	case DENEMO_STEMDOWN:
@@ -1469,6 +1551,7 @@ change_stem_directive (DenemoScore * si, enum stemdirections amount)
     }
 }
 
+
 /**
  * Change the number of dots on the current chord
  *
@@ -1482,16 +1565,20 @@ changedots (DenemoScore * si, gint amount)
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
-      if (Denemo.gui->mode&(INPUTRHYTHM))
-#ifdef _HAVE_FLUIDSYNTH_
-	{
+      store_for_undo_change (si, curmudelaobj);
+
+      if (Denemo.gui->mode&(INPUTRHYTHM)) {
+	if(Denemo.prefs.immediateplayback) {
+#ifdef _HAVE_FLUIDSYNTH_	
 	  chord *thechord = (chord *) curmudelaobj->object;
 	  gboolean rest =  (thechord->notes == NULL);
 	  fluid_rhythm_feedback(thechord->baseduration, rest, TRUE);
-	}
+	  
 #else
 	  playpitch(440.0, 0.2, 0.2, 1);
 #endif
+	}
+      }
       changenumdots (curmudelaobj, amount);
 
       if (curmudelaobj->user_string)
@@ -1529,6 +1616,7 @@ void
 insertmeasureafter (DenemoGUI *gui)
 {
   DenemoScore *si = Denemo.gui->si;
+  take_snapshot();
   si->currentmeasure = addmeasures (si, si->currentmeasurenum++, 1, 0);
   si->cursor_x = 0;
   si->cursor_appending = TRUE;
@@ -1547,6 +1635,7 @@ void
 addmeasureafter (DenemoGUI *gui)
 {
   DenemoScore *si = Denemo.gui->si;
+  take_snapshot();
   si->currentmeasure = addmeasures (si, si->currentmeasurenum++, 1, 1);
   si->cursor_x = 0;
   si->cursor_appending = TRUE;
@@ -1693,6 +1782,7 @@ void deletemeasure(DenemoGUI * gui)
 void deletemeasureallstaffs(DenemoGUI * gui)
 {
   DenemoScore *si=gui->si;
+  //take_snapshot(); this does not prevent the multiple undo steps needed
   si->currentmeasure =
     removemeasures (si, si->currentmeasurenum - 1, 1, TRUE);
   setcurrents (si);
@@ -1727,7 +1817,7 @@ dnm_deletemeasure (DenemoScore * si)
   if(si->markstaffnum)
     calcmarkboundaries (si);
   si->markstaffnum = 0;
-
+  // g_print("Removed current measure now %p number %d\n", si->currentmeasure, si->currentmeasurenum);
 
 }
 
@@ -1800,17 +1890,13 @@ dnm_deleteobject (DenemoScore * si)
   if(curmudelaobj->type==LILYDIRECTIVE && ((lilydirective *)curmudelaobj->object)->locked)
     if(!confirm("This LilyPond insert is locked","Really delete it?"))
       return;
+  DenemoUndoData *undo;
 
-
-  if (si->undo_redo_mode == UNDO)
+  if (!si->undo_guard)
     {
-      unre_data *undo = (unre_data *) g_malloc (sizeof (unre_data));
-      undo->staffnum = si->currentstaffnum;
-      undo->measurenum = si->currentmeasurenum;
-      undo->position = si->cursor_x;
+      undo = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));      
       undo->object = dnm_clone_object (curmudelaobj);
-      undo->action = ACTION_DELETE;
-      update_undo_info (si, undo);
+      //get position after delete
     }
 
 
@@ -1850,6 +1936,8 @@ dnm_deleteobject (DenemoScore * si)
 	  break;
 	case TIMESIG:
 	  delete_object_helper (si);
+#if 0
+	  //do not do this, as this is a primitive and should only delete one object
 	  /* For time signature changes remove from all other staffs 
 	   * if in the conventional, first, position */
 	  for (curstaff = si->thescore; curstaff; curstaff = curstaff->next)
@@ -1865,6 +1953,7 @@ dnm_deleteobject (DenemoScore * si)
 		}
 	      }
 	    }
+#endif
 	  reset_cursor_stats (si);
 	  find_xes_in_all_measures (si);
 	  break;
@@ -1903,6 +1992,13 @@ dnm_deleteobject (DenemoScore * si)
 	}
       si->markstaffnum = 0;
     }
+  if (!si->undo_guard)
+    {
+      get_position(si, &undo->position);
+      undo->action = ACTION_DELETE;
+      update_undo_info (si, undo);
+    }
+
 }
 
 /**
@@ -1937,13 +2033,17 @@ void
 toggle_tie (GtkAction *action, gpointer param)
 {
   DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
   DenemoObject *curmudelaobj = (DenemoObject *)
     (gui->si->currentobject ? gui->si->currentobject->data : NULL);
   if (curmudelaobj && curmudelaobj->type == CHORD &&
       ((chord *) curmudelaobj->object)->notes)
     {
+      store_for_undo_change (si, curmudelaobj);
+
 
       ((chord *) curmudelaobj->object)->is_tied ^= 1;
+      gtk_widget_queue_draw (Denemo.scorearea);
     }
 }
 
@@ -1993,9 +2093,9 @@ gotohome (gpointer param, gboolean extend_selection)
     calcmarkboundaries (gui->si);
   find_leftmost_allcontexts (gui->si);
   update_hscrollbar (gui);
-  /*gtk_widget_draw (gui->scorearea, NULL);*/
+  /*gtk_widget_draw (Denemo.scorearea, NULL);*/
   scorearea_expose_event(NULL, NULL);//refresh cached values, eg current timesig
-  gtk_widget_queue_draw (gui->scorearea);
+  gtk_widget_queue_draw (Denemo.scorearea);
 }
 
 
@@ -2034,9 +2134,11 @@ toend (GtkAction *action, gpointer param)
  * @return none
 */
 void
-movetostart (GtkAction *action, gpointer param)
+movetostart (GtkAction *action, DenemoScriptParam* param)
 {
   gotohome(param, FALSE);
+  if(param)
+    param->status = TRUE;
 }
 
 /**
@@ -2046,9 +2148,11 @@ movetostart (GtkAction *action, gpointer param)
  * @return none
 */
 void
-movetoend (GtkAction *action, gpointer param)
+movetoend (GtkAction *action, DenemoScriptParam* param)
 {
   gotoend(param, FALSE);
+  if(param)
+    param->status = TRUE;
 }
 
 
@@ -2088,6 +2192,9 @@ toggle_begin_slur (DenemoGUI *gui)
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
+
+
       ((chord *) curmudelaobj->object)->slur_begin_p
 	= !((chord *) curmudelaobj->object)->slur_begin_p;
       if (curmudelaobj->user_string)
@@ -2133,6 +2240,9 @@ toggle_end_slur (DenemoGUI *gui)
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      store_for_undo_change (si, curmudelaobj);
+
+
       ((chord *) curmudelaobj->object)->slur_end_p
 	= !((chord *) curmudelaobj->object)->slur_end_p;
       if (curmudelaobj->user_string)
