@@ -40,9 +40,9 @@ movetomeasureleft (DenemoScriptParam *param);
 void
 movetomeasureright (DenemoScriptParam *param);
 void
-movetostart (GtkAction *action, gpointer param);
+movetostart (GtkAction *action,  DenemoScriptParam *param);
 void
-movetoend (GtkAction *action, gpointer param);
+movetoend (GtkAction *action, DenemoScriptParam *param);
 gboolean
 swapstaffs (GtkAction *action, gpointer param);
 gboolean
@@ -77,6 +77,12 @@ movetovoiceup (DenemoScriptParam *param);
 gboolean
 movetovoicedown (DenemoScriptParam *param);
 
+gboolean 
+cursor_to_next_note(DenemoScriptParam *param);
+
+gboolean 
+cursor_to_next_chord(DenemoScriptParam *param);
+
 gboolean
 cursorleft (DenemoScriptParam *param);
 
@@ -201,7 +207,7 @@ void
 insert_rhythm_pattern(DenemoGUI *gui);
 void
 toggle_tie (GtkAction *action, gpointer param);
-
+gint get_prevailing_duration(void);
 void
 dnm_inserttuplet (DenemoGUI * gui, tuplet_type type);
 #endif /*COMMANDFUNCSH */
