@@ -19,9 +19,9 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <signal.h>
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+//#ifdef HAVE_SYS_WAIT_H
+//# include <sys/wait.h>
+//#endif
 #ifdef HAVE_WAIT_H
 #include <wait.h>
 #endif
