@@ -32,16 +32,71 @@
 #define EXCL_WIDTH 3
 #define EXCL_HEIGHT 13
 
-#if 0
-static GdkGC *  bluegc;
-static GdkGC *  redgc;
-static GdkGC *  graygc;
-static GdkGC *  slategraygc;
-static GdkGC *  lightbluegc;
-static GdkGC *  blackgc;
-static GdkGC *greengc = NULL;
-#endif
+static gboolean
+draw_score (cairo_t *cr);
 GdkPixbuf *StaffPixbuf, *StaffPixbufSmall;
+#define MAX_PLAYHEADS (100)
+static GdkRectangle old_playhead_damage[MAX_PLAYHEADS];
+static GdkRectangle new_playhead_damage[MAX_PLAYHEADS];
+
+static gint old_playhead_index=0;
+static gint new_playhead_index=0;
+static gboolean playhead_flip = TRUE;
+
+
+static layout_needed = TRUE; //Set FALSE when further call to draw_score(NULL) is not needed.
+void initialize_playhead(void) {
+  new_playhead_index = 0;
+  old_playhead_index = 0;
+  layout_needed = TRUE;
+}
+
+static void add_playhead_damage(gint x, gint y, gint w, gint h) {
+  if(playhead_flip) {
+  new_playhead_damage[new_playhead_index].x = x;
+  new_playhead_damage[new_playhead_index].y = y;
+  new_playhead_damage[new_playhead_index].width = w;
+  new_playhead_damage[new_playhead_index].height = h;
+  new_playhead_index++;
+  new_playhead_index = new_playhead_index%MAX_PLAYHEADS;
+  //g_print("@");
+  } else {
+  old_playhead_damage[old_playhead_index].x = x;
+  old_playhead_damage[old_playhead_index].y = y;
+  old_playhead_damage[old_playhead_index].width = w;
+  old_playhead_damage[old_playhead_index].height = h;
+  old_playhead_index++;
+  old_playhead_index = old_playhead_index%MAX_PLAYHEADS;
+  //g_print("-");
+  } 
+
+}
+void region_playhead(void) {
+  gint i;
+ 
+  draw_score (NULL);
+  layout_needed = FALSE;
+  {
+    for(i=0;i< old_playhead_index; i++)
+      {
+	//g_print("*");
+	gdk_window_invalidate_rect (Denemo.scorearea->window, old_playhead_damage+i, FALSE); }
+    for(i=0;i< new_playhead_index; i++)
+      {
+	//g_print("~");
+	gdk_window_invalidate_rect (Denemo.scorearea->window, new_playhead_damage+i, FALSE); }   
+    if(playhead_flip) {
+      old_playhead_index = 0;
+    }
+    else {
+      new_playhead_index = 0;
+    }
+    playhead_flip = !playhead_flip;
+    
+  }
+  
+  //g_print("o");Hah! this is needed to allow the re-draw to take place
+}
 
 static void      
 create_tool_pixbuf(void) {
@@ -54,8 +109,7 @@ create_tool_pixbuf(void) {
 /**
  * scorearea_configure_event
  *
- * This function just creates a backing pixmap of the appropriate
- * size, recaculates the number of measures that can be fit into
+ * This function recaculates the number of measures that can be fit into
  * the display, and returns 
  */
 gint
@@ -69,14 +123,8 @@ scorearea_configure_event (GtkWidget * widget, GdkEventConfigure * event)
   static gboolean init = FALSE;
   if(!init) {
     create_tool_pixbuf();
+    init = TRUE;
   }
-  /* Create a new backing pixmap of the appropriate size */
-  if (gui->pixmap)
-    gdk_pixmap_unref (gui->pixmap);
-
-  gui->pixmap = gdk_pixmap_new (widget->window,
-				widget->allocation.width,
-				widget->allocation.height, -1);
 
   set_width_to_work_with (gui);
   nudgerightward (gui);
@@ -177,7 +225,7 @@ static gint
 draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
 	     DenemoGUI * gui, struct infotopass *itp)
 {
-
+  //g_print("draw obj %p\n", cr);
 
   static gboolean init=FALSE;
   static GdkColor white, black, blue, green, yellow;
@@ -199,9 +247,9 @@ draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
   DenemoObject *mudelaitem = (DenemoObject *) curobj->data;
 
 
-  //this is the selection being given a blue background I guess
-  //FIXME too much save and restore for trifling reasons...
-  if(cr) if(Denemo.gui->si->playingnow==NULL && itp->mark) {
+  //this is the selection being given a blue background
+  //FIXME NO! save and restore are *fast*. too much save and restore for trifling reasons...
+  if(cr) if(/* Denemo.gui->si->playingnow==NULL && show selection during playback*/ itp->mark) {
     cairo_save(cr);
     cairo_set_source_rgb( cr, 0.5, 0.5, 1.0 );
     cairo_rectangle (cr, x+mudelaitem->x, y, 20, 80 );
@@ -211,14 +259,18 @@ draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
   //g_print("%x %f %f %f\n", Denemo.gui->si->playingnow, Denemo.gui->si->playhead,  mudelaitem->earliest_time, mudelaitem->latest_time );
   
   // draw playhead as green background
-  if(cr) if( Denemo.gui->si->playingnow && (Denemo.gui->si->playhead > mudelaitem->earliest_time) && (Denemo.gui->si->playhead < mudelaitem->latest_time)) {
-    cairo_save(cr);
-    cairo_set_source_rgb( cr, 0.5, 1.0, 0.5 );
-    cairo_rectangle (cr, x+mudelaitem->x, y, 20, 80 );
-    cairo_fill(cr);
-    cairo_restore(cr);
-  }
-
+   if( Denemo.gui->si->playingnow && (Denemo.gui->si->playhead > mudelaitem->earliest_time-0.0001) && (Denemo.gui->si->playhead < mudelaitem->latest_time+0.0001)) {
+     if(cr) {
+       cairo_save(cr);
+       cairo_set_source_rgb( cr, 0.5, 1.0, 0.5 );
+       cairo_rectangle (cr, x+mudelaitem->x, y, 20, 80 );
+       cairo_fill(cr);
+       cairo_restore(cr);
+     } else {     
+       add_playhead_damage((x+mudelaitem->x)*Denemo.gui->si->zoom*100.0/(*itp->scale), y*Denemo.gui->si->zoom, 20*Denemo.gui->si->zoom*100.0/(*itp->scale), 80*Denemo.gui->si->zoom);   
+     }
+   }
+   
 
 
 
@@ -430,9 +482,7 @@ draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
 	  (mudelaitem->starttickofnextnote > itp->tickspermeasure);
       if (si->cursor_appending)
 	{
-
-	  draw_cursor (cr, si, x + mudelaitem->x + extra, y, ((itp->curmeasure->next!=NULL) && (objnode *) itp->curmeasure->next->data)?-1:1 /*itp->last_gap */,
-		       gui->mode, si->cursorclef);
+	  draw_cursor (cr, si, x + mudelaitem->x + extra, y, ((itp->curmeasure->next!=NULL) && (objnode *) itp->curmeasure->next->data)?-1:0 /*itp->last_gap */, gui->mode, si->cursorclef);
 	  memcpy (si->cursoraccs, itp->curaccs, SEVENGINTS);
 	}
       else
@@ -459,8 +509,8 @@ draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
 
   //gdk_gc_set_foreground (blackgc, &black);
 
-  //  g_print("obj at %d %d\n",  x + mudelaitem->x + mudelaitem->minpixelsalloted, (int)(gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)));
-  //  itp->line_end = itp->markx2 > (int)(gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME));
+  //  g_print("obj at %d %d\n",  x + mudelaitem->x + mudelaitem->minpixelsalloted, (int)(Denemo.scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)));
+  //  itp->line_end = itp->markx2 > (int)(Denemo.scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME));
 
 
   itp->rightmosttime = mudelaitem->latest_time;
@@ -507,11 +557,11 @@ draw_measure (cairo_t *cr, measurenode * curmeasure, gint x, gint y,
     }
 
   /*  paint the measure number at the preceding barline 
-      - do not do measure 1 as it clashes with the name (and is not needed) */
+       */
 
-  if(cr) if(itp->measurenum>1) {
+  if(cr) {
     g_string_sprintf (mstring, "%d", itp->measurenum);
-    drawnormaltext_cr (cr, mstring->str, x - SPACE_FOR_BARLINE, y - 12);
+    drawnormaltext_cr (cr, mstring->str, x - SPACE_FOR_BARLINE - 5, y - 3);
   }
 
 
@@ -594,14 +644,15 @@ draw_measure (cairo_t *cr, measurenode * curmeasure, gint x, gint y,
   } // for each object
   if(cr) {
     cairo_save(cr);
+    //marking the barline if within selection
     if(si->markstaffnum && 
-       (curmeasure->data==NULL) &&
+/*        (curmeasure->data==NULL) && */
        (si->firststaffmarked <= itp->staffnum) &&
        (si->laststaffmarked >= itp->staffnum) &&
        (si->firstmeasuremarked <= itp->measurenum) &&
-       (si->lastmeasuremarked >= itp->measurenum))
+       (si->lastmeasuremarked > itp->measurenum))
       {
-	cairo_set_source_rgb( cr, 0, 0, 1 );
+	cairo_set_source_rgb( cr, 0.5, 0.5, 1.0 );
 	cairo_rectangle (cr, x + GPOINTER_TO_INT (itp->mwidthiterator->data), y, 20, STAFF_HEIGHT+1); 
 	cairo_fill(cr);
       }
@@ -724,7 +775,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
     gint staffname_offset = (thestaff->voicenumber == 1) ? 24 :
       (thestaff->voicenumber == 2
        || thestaff->voicenumber == 3) ? 12 : 0;
-    drawnormaltext_cr( cr, thestaff->denemo_name->str, KEY_MARGIN, y - staffname_offset+10 );
+    drawnormaltext_cr( cr, thestaff->denemo_name->str, 0/*KEY_MARGIN*/, y - staffname_offset+10 );
   } else {
  cairo_save(cr);
    gint staffname_offset = (thestaff->voicenumber == 1) ? 24 :
@@ -780,7 +831,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
 	 && itp->measurenum <= nummeasures)
     {
       if( x + GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE >
-	  (int) (gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)))
+	  (int) (Denemo.scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)))
 	  itp->line_end=TRUE;
 	 
       itp->last_gap = 0;
@@ -791,11 +842,11 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
 #if 0
       if(itp->line_end) {
       *itp->scale = itp->curmeasure->next?
-	(int)(100*x/(gui->scorearea->allocation.width/gui->si->zoom)):100;
+	(int)(100*x/(Denemo.scorearea->allocation.width/gui->si->zoom)):100;
       }
 #else
       if(itp->line_end && itp->curmeasure->next)
-	*itp->scale = (int)(100*x/(gui->scorearea->allocation.width/gui->si->zoom));
+	*itp->scale = (int)(100*x/(Denemo.scorearea->allocation.width/gui->si->zoom));
       if( (Denemo.gui->view!=DENEMO_PAGE_VIEW) && *itp->scale < 100)
 	*itp->scale = 100;
 #endif
@@ -865,11 +916,11 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
 }
 static void
 print_system_separator (cairo_t *cr, gdouble position){
-  //g_print("At %f for %d\n", position, Denemo.gui->scorearea->allocation.height);
+  //g_print("At %f for %d\n", position, Denemo.scorearea->allocation.height);
 #define SYSTEM_SEP (6)
   cairo_save(cr);
   cairo_set_source_rgb( cr, 0.5, 0.0, 0.0 );
-  cairo_rectangle (cr, 0, position-SYSTEM_SEP/2, Denemo.gui->scorearea->allocation.width/Denemo.gui->si->zoom, SYSTEM_SEP);
+  cairo_rectangle (cr, 0, position-SYSTEM_SEP/2, Denemo.scorearea->allocation.width/Denemo.gui->si->zoom, SYSTEM_SEP);
   cairo_set_source_rgb( cr, 0.7, 0.0, 0.0 );
   cairo_fill(cr);
 #undef SYSTEM_SEP
@@ -923,25 +974,26 @@ static gboolean schedule_draw(gint *flip_count) {
   if(*flip_count>MAX_FLIP_STAGES) {
     return FALSE;
   }
-  gtk_widget_queue_draw (Denemo.gui->scorearea);
+  gtk_widget_queue_draw (Denemo.scorearea);
   return TRUE;
 }
 /**
  * This actually draws the score, staff-by-staff 
- * @param widget pointer to the parent widget
+ * @param cr cairo context to draw with.
  * @param gui pointer to the DenemoGUI structure
  * returns whether the height of the drawing area was sufficient to draw everything
  */
 static gboolean
-draw_score (GtkWidget * widget, DenemoGUI * gui)
-{
+draw_score (cairo_t *cr)
+{//g_print("draw_score %p\n", cr);
   staffnode *curstaff;
   gint y=0;
   struct infotopass itp;
   gboolean repeat = FALSE;
   gdouble leftmost = 10000000.0;
+  DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
-  gint line_height = gui->scorearea->allocation.height*gui->si->system_height/gui->si->zoom;
+  gint line_height = Denemo.scorearea->allocation.height*gui->si->system_height/gui->si->zoom;
   static gint flip_count;//passed to a timer to indicate which stage of animation of page turn should be used when re-drawing, -1 means not animating 0+ are the stages
   /* Initialize some fields in itp */
 
@@ -968,7 +1020,9 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       get_obj_for_start_time(gui->si->smf, gui->si->end_time - 0.001);
     //g_print("Start time %p %f end time %p %f\n", itp.startobj, si->start_time, itp.endobj, si->end_time);
   }
-  cairo_t *cr = widget?gdk_cairo_create( gui->pixmap ): NULL;
+
+  /* The colour for staff lines and such is black. */
+  if (cr) cairo_set_source_rgb( cr, 0, 0, 0 );
 
   if(cr) cairo_scale( cr, gui->si->zoom, gui->si->zoom );
   if(cr) cairo_translate( cr, 0.5, 0.5 );
@@ -1016,7 +1070,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 	cairo_rectangle( cr,0,y, width, height );
 	cairo_fill( cr );
 	cairo_restore( cr );
-	//gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
+	//gdk_draw_pixbuf(Denemo.pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
       }
       if(staff->voice_directives) {
 
@@ -1027,7 +1081,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 	cairo_rectangle( cr,0,y+STAFF_HEIGHT/2, width, height );
 	cairo_fill( cr );
 	cairo_restore( cr );
-	//gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y + STAFF_HEIGHT/2, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
+	//gdk_draw_pixbuf(Denemo.pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y + STAFF_HEIGHT/2, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
       }
     
     
@@ -1095,7 +1149,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
     if(Denemo.gui->si->playingnow && itp.measurenum >= si->rightmeasurenum)
       itp.line_end = FALSE;//don't print whole lines of grayed out music during playback
 
-    while(((itp.left-gui->lefts)<DENEMO_MAX_SYSTEMS-1) && itp.line_end && (yy<(gui->scorearea->allocation.height/gui->si->zoom))) {
+    while(((itp.left-gui->lefts)<DENEMO_MAX_SYSTEMS-1) && itp.line_end && (yy<(Denemo.scorearea->allocation.height/gui->si->zoom))) {
       if(cr) if (itp.staffnum==si->top_staff)
 	print_system_separator (cr, line_height*system_num);
       system_num++;
@@ -1115,23 +1169,18 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
     
     si->rightmost_time = itp.rightmosttime;
     
-     if(cr && (system_num>2) && Denemo.gui->si->playingnow && (si->playhead>leftmost) && itp.measurenum <= g_list_length (((DenemoStaff*)curstaff->data)->measures)/*(itp.measurenum > (si->rightmeasurenum+1))*/) {
+     if((system_num>2) && Denemo.gui->si->playingnow && (si->playhead>leftmost) && itp.measurenum <= g_list_length (((DenemoStaff*)curstaff->data)->measures)/*(itp.measurenum > (si->rightmeasurenum+1))*/) {
       //put the next line of music at the top with a break marker
       itp.left = &gui->lefts[0];
       itp.right = &gui->rights[0];     
       itp.scale = &gui->scales[0];     
-
+      if(cr) {
 	cairo_save(cr);
-	if(0)   {
-	  //clear the previously drawn version FIXME? do not draw in first place, will that be faster?
-	  cairo_set_source_rgb( cr, 1.0, 1.0, 1.0 );
-	  cairo_rectangle (cr, 0, y - (si->staffspace / 2) - ((DenemoStaff*)curstaff->data)->space_above, gui->scorearea->allocation.width/Denemo.gui->si->zoom, STAFF_HEIGHT+((DenemoStaff*)curstaff->data)->space_above + ((DenemoStaff*)curstaff->data)->space_below  + (si->staffspace));
-	  cairo_fill(cr);
-      }
 	cairo_set_source_rgb( cr, 0.0, 0.0, 1.0 );//Strong Blue Line to break pages
-	cairo_rectangle (cr, 0, line_height-10, gui->scorearea->allocation.width/Denemo.gui->si->zoom, 10);
+	cairo_rectangle (cr, 0, line_height-10, Denemo.scorearea->allocation.width/Denemo.gui->si->zoom, 10);
 	cairo_fill(cr);
 	cairo_restore(cr);
+      }
      
       itp.line_end = FALSE;//to force print of timesig
       if(itp.measurenum > (si->rightmeasurenum+1))
@@ -1148,12 +1197,15 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       // g_print("drawing %d\n", flip_count);
       if(flip_count>0 && flip_count<MAX_FLIP_STAGES)
 	flip = flip_count/(gdouble)MAX_FLIP_STAGES;
-      cairo_translate( cr, gui->scorearea->allocation.width*(1-flip)*0.5/Denemo.gui->si->zoom, 0.0);	
-      cairo_scale( cr, flip, 1.0);
+      if(cr) {
+	cairo_translate( cr, Denemo.scorearea->allocation.width*(1-flip)*0.5/Denemo.gui->si->zoom, 0.0);	
+	cairo_scale( cr, flip, 1.0);
+     
       if(draw_staff (flip_count>0?cr:NULL, curstaff, y, gui, &itp))
 	repeat = TRUE; 
       cairo_scale( cr, 1/flip, 1.0);
-      cairo_translate( cr, -gui->scorearea->allocation.width*(1-flip)*0.5/Denemo.gui->si->zoom, 0.0);
+      cairo_translate( cr, -Denemo.scorearea->allocation.width*(1-flip)*0.5/Denemo.gui->si->zoom, 0.0);
+      }
       //draw_break_marker();
      } else {
         if(flip_count!=-1)
@@ -1194,52 +1246,53 @@ gint
 scorearea_expose_event (GtkWidget * widget, GdkEventExpose * event)
 {
 DenemoGUI *gui = Denemo.gui;
+ cairo_t *cr;
+ //g_print("expose\n");
  if((!Denemo.gui->si)||(!Denemo.gui->si->currentmeasure)){
    g_warning("Cannot draw!\n");
    return TRUE;
  }
  if(widget==NULL) {
-   draw_score (NULL, gui);
+   draw_score (NULL);
    return TRUE;
  }
+
+  /* Layout the score. */
+ //if(!Denemo.gui->si->playingnow)
+ if(layout_needed)
+   draw_score (NULL);
+ layout_needed = TRUE;
+ // while(draw_score (NULL, gui) && !Denemo.gui->si->playingnow)
+ //   {/*nothing*/}
+
+  /* Setup a cairo context for rendering and clip to the exposed region. */
+  cr = gdk_cairo_create (event->window);
+
+ 
+  gdk_cairo_region (cr, event->region);
   
- do{
-   /* Clear the backing pixmap */
-   if(Denemo.gui->input_source!=INPUTKEYBOARD && Denemo.gui->input_source!=INPUTMIDI &&
-      (Denemo.prefs.overlays || (Denemo.gui->input_source==INPUTAUDIO))
-      && pitch_entry_active(gui)) {
-     gdk_draw_rectangle (gui->pixmap,
-			 gcs_lightbluegc(),
-			TRUE,
-			0, 0,
-			widget->allocation.width, widget->allocation.height);
+  //region_playhead(cr); //!!!!!!! but not both!!!!!!
+  cairo_clip (cr);
 
+  /* Clear with an appropriate background color. */
+  if(Denemo.gui->input_source!=INPUTKEYBOARD && Denemo.gui->input_source!=INPUTMIDI &&
+     (Denemo.prefs.overlays || (Denemo.gui->input_source==INPUTAUDIO))
+     && pitch_entry_active(gui)) {
+    GdkColor col;
+    gdk_color_parse ("lightblue", &col);
+    gdk_cairo_set_source_color (cr, &col);
+  } else if (GTK_WIDGET_IS_SENSITIVE (Denemo.scorearea)) {
+    //cairo_set_source_rgb (cr, 1,1,1);
+    cairo_set_source_rgb (cr, ((0xFF0000&Denemo.color)>>16)/255.0, ((0xFF00&Denemo.color)>>8)/255.0, ((0xFF&Denemo.color))/255.0);
   } else {
-  if (GTK_WIDGET_IS_SENSITIVE (gui->scorearea) )
-    gdk_draw_rectangle (gui->pixmap,
-			widget->style->white_gc,
-			TRUE,
-			0, 0,
-			widget->allocation.width, widget->allocation.height);
-  else
-    gdk_draw_rectangle (gui->pixmap,
-			widget->style->bg_gc[0],
-			TRUE,
-			0, 0,
-			widget->allocation.width, widget->allocation.height);
+    gdk_cairo_set_source_color (cr, widget->style->bg);
+   
   }
+  cairo_paint (cr);
 
-  /* Draw the score */
-  } while(draw_score (widget, gui) && !Denemo.gui->si->playingnow);
-
-  /* Now actually draw the backing pixmap onto the drawing area */
-
-  gdk_draw_pixmap (gui->scorearea->window,
-		   gui->scorearea->style->black_gc,
-		   gui->pixmap,
-		   0, 0, 0, 0,
-		   gui->scorearea->allocation.width,
-		   gui->scorearea->allocation.height);
+  /* Draw the score. */
+  draw_score (cr);
+  cairo_destroy (cr);
 
   return TRUE;
 }
