@@ -134,7 +134,7 @@ draw_notehead (cairo_t *cr,
     else
       xx -= headwidths[noteheadtype];
   }
-  //OVERRIDE_GRAPHIC just affects the head, everything else -  accidentals dots stems beams ...are controlled by baseduration
+  //OVERRIDE_GRAPHIC on the note directive just affects the head, everything else -  accidentals dots stems beams ...are controlled by chord/chord-directives. Even the notehead is set by the chord directives if override_notehead is set
   if(!(get_override(thenote->directives)&DENEMO_OVERRIDE_GRAPHIC)) {
     if(override_notehead) {
       //g_print("drawing a chord override graphic at %d %d\n",  xx+gx-override_notehead->width/2,  y+height+gy-override_notehead->height/2);
@@ -152,26 +152,8 @@ draw_notehead (cairo_t *cr,
   gint maxwidth = headwidths[noteheadtype];
   
   /* any display for note directives */
- { GList *g = thenote->directives;
- gint count=10;
-  for(;g;g=g->next, count+=10) {
-    DenemoDirective *directive = (DenemoDirective *)g->data;
-    if(directive->graphic) {
-      gint gwidth, gheight;
-      gwidth = directive->graphic->width;
-      gheight = directive->graphic->height;
+  maxwidth = MAX(draw_for_directives(cr, thenote->directives, xx, y+thenote->y), maxwidth);
 
-      maxwidth = MAX(gwidth, maxwidth);
-      //g_print("drawing a graphic at %d %d\n", xx+directive->gx+count-gwidth/2,  y+height+directive->gy-gheight/2);
-      drawbitmapinverse_cr ( cr, directive->graphic,
-			       xx+directive->gx+count-gwidth/2,  y+height+directive->gy-gheight/2);
-      
-    }
-    if(directive->display) {
-      drawnormaltext_cr (cr, directive->display->str, xx+directive->tx+count, y+thenote->y+directive->ty ); 
-    }
-  }
- }
  /* Now draw any trailing dots */
  if ((height % LINE_SPACE) == 0)
    draw_dots (cr, xx + maxwidth,
@@ -262,6 +244,18 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
   cairo_save(cr);
   cairo_set_line_width( cr, 1.0 );
 
+
+
+ if (thechord.slur_begin_p)
+   draw_slur_start(cr, xx, y);
+
+ if (thechord.slur_end_p)
+   draw_slur_end(cr, xx, y);
+
+
+
+
+
   if(is_grace) {
     note *thenote = (note *) thechord.notes->data;
     cairo_translate( cr, xx, y+thenote->y);
@@ -275,17 +269,39 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
     else
       cairo_set_source_rgb( cr, 180.0/255, 160.0/255, 32.0/255 );// yellow for non printing
   }
-
+  gboolean override_chord_graphic = FALSE;
   { GList *g = thechord.directives;
       gint count = 0;
       for(;g;g=g->next) {
 	DenemoDirective *directive = (DenemoDirective *)g->data;
+	
 	if(directive->graphic) {
-	  if(directive->override&DENEMO_OVERRIDE_GRAPHIC)
-	    gx=directive->gx, gy=directive->gy, override_notehead = directive->graphic;//will be used to draw all the notes/the rest
-	  else
-	    drawbitmapinverse_cr (cr, directive->graphic,
-				  xx+directive->gx, y+directive->gy);
+	  
+	  if((directive->override&DENEMO_OVERRIDE_GRAPHIC))
+	    override_chord_graphic = TRUE;
+
+	  if( (directive->override&DENEMO_OVERRIDE_GRAPHIC) ) {
+	    if(directive->override&DENEMO_ALT_OVERRIDE) {
+	      gx=directive->gx;
+	      gy=directive->gy;
+	      override_notehead = directive->graphic;//will be used to draw all the notes/the rest
+	    } else {
+	      drawbitmapinverse_cr (cr, directive->graphic, 
+				    xx+directive->gx-directive->graphic->width/2, 
+				    y + STAFF_HEIGHT+40+directive->gy  - directive->graphic->height/2);	      
+	    }
+	  } else { //this directive's graphic does not override entire chord (other ones may)
+	    if(directive->override&DENEMO_ALT_OVERRIDE) {
+	      drawbitmapinverse_cr (cr, directive->graphic, 
+				    xx+directive->gx-directive->graphic->width/2, 
+				    (thechord.is_stemup? (y + STAFF_HEIGHT+40+directive->gy):(y - 40 -directive->gy))  - directive->graphic->height/2 );
+	      
+	    } else {
+	      drawbitmapinverse_cr (cr, directive->graphic, 
+				    xx+directive->gx-directive->graphic->width/2, 
+				    y + STAFF_HEIGHT+40+count+directive->gy  - directive->graphic->height/2);	      
+	    }
+	  }
 	}
 	if(directive->display) {
 	  drawnormaltext_cr (cr, directive->display->str, xx+directive->tx, y+STAFF_HEIGHT+40+count+directive->ty );
@@ -293,20 +309,24 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	}
       } //for each chord directive
   }//block displaying chord directives
-  if (!thechord.notes/* a rest */) {
-    draw_rest (cr, MAX(duration, 0), thechord.numdots, xx, y, override_notehead, gx, gy);
-  }  else {
-    /* Draw the noteheads and accidentals */
-    for (curnode = thechord.notes; curnode; curnode = curnode->next){
-      note *thenote = (note *)curnode->data;
-      draw_notehead (cr, thenote, duration,
-		     thechord.numdots, xx, y, accs, thechord.is_stemup, mudelaitem->isinvisible, override_notehead, gx, gy); 
+  if( (!override_chord_graphic) || (override_chord_graphic && override_notehead)) {
+    if (!thechord.notes/* a rest */) {
+      draw_rest (cr, MAX(duration, 0), thechord.numdots, xx, y, override_notehead, gx, gy);
+    }  else {
+      /* Draw the noteheads and accidentals */
+      for (curnode = thechord.notes; curnode; curnode = curnode->next){
+	note *thenote = (note *)curnode->data;
+	draw_notehead (cr, thenote, duration,
+		       thechord.numdots, xx, y, accs, thechord.is_stemup, mudelaitem->isinvisible, override_notehead, gx, gy); 
+      }
     }
   }
-  /* Now the stem and beams. This is complicated. */
-  if (thechord.notes/* not a rest */) {
-    if (thechord.is_stemup)
-      {
+  
+      /* Now the stem and beams. This is complicated. */
+      if (thechord.notes/* not a rest */) {
+	if (thechord.is_stemup)
+	  {
+if(!override_chord_graphic) {
 	if (mudelaitem->isstart_beamgroup && mudelaitem->isend_beamgroup)
 	  {
 	    if (duration >= 3)
@@ -381,7 +401,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	  cairo_line_to( cr, xx + headwidths[noteheadtype], thechord.lowesty + y - 2 );
 	  cairo_stroke( cr );
 	}
-	
+ } //if graphic not overrriden	
 	
 	/* Now draw the tie, if appropriate */
 	if (thechord.is_tied)
@@ -398,7 +418,11 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	  }
       }			/* End stemup stuff */
     else
-      {			/* chord is stemdown */
+      {
+
+if(!override_chord_graphic) {
+
+			/* chord is stemdown */
 	if (mudelaitem->isstart_beamgroup && mudelaitem->isend_beamgroup)
 	  {
 	    if (duration >= 3)
@@ -471,6 +495,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	  cairo_line_to( cr, xx, thechord.stemy + y );
 	  cairo_stroke( cr );
 	}
+ }
 	/* Now draw the tie, if appropriate */
 	if (thechord.is_tied)
 	  {
