@@ -1141,6 +1141,8 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
   time_t starttime;
   time_t endtime;
 
+  call_out_to_guile("(InitializeMidiGeneration)");
+
   /* get user preferences, if any */
   envp = getenv ("EXP_MIDI_VEL");
   if (envp)
@@ -1897,8 +1899,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		}
  		if(debug){
 		  printf("\nmeasure is empty = %d", measure_is_empty);
-		  printf("\nticks_at_bar(%d) + measurewidth(%d)!= "
-				  "ticks_read(%d)\n",
+		  printf("\nticks_at_bar %ld  + measurewidth %d != ticks_read %ld\n",
 				  ticks_at_bar, measurewidth, ticks_read);
 		  printf("\ninternal ticks = %d\n", internaltoticks(0));
 		}
@@ -1944,11 +1945,19 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
       smf_track_remove_from_smf(Denemo.gui->si->recorded_midi_track);
   }
   /* we are done */
-
-
-  free_midi_data(si);
-  si->smf = smf;
-
+  {gboolean midi_track = FALSE;
+    if(Denemo.gui->si->recorded_midi_track) {
+      if(si->smf && (((smf_track_t*)Denemo.gui->si->recorded_midi_track)->smf==si->smf)) {
+	smf_track_remove_from_smf(Denemo.gui->si->recorded_midi_track);
+	midi_track = TRUE;
+      }
+    }
+    free_midi_data(si);  
+    si->smf = smf;
+    if(midi_track)
+      smf_add_track(smf, Denemo.gui->si->recorded_midi_track);
+  }
+  
   si->smfsync = si->changecount;
 
 
@@ -1962,6 +1971,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
       si->end_time = smf_get_length_seconds(smf);
     g_print("Start time %f end time %f\n", si->start_time, si->end_time);
   }
+  call_out_to_guile("(FinalizeMidiGeneration)");
   return smf_get_length_seconds(smf);
 }
 
