@@ -320,22 +320,18 @@ newVersesElem(xmlNodePtr curElem,  xmlNsPtr ns, GList *verses, gchar *type) {
 }
 static void
 newDirectivesElem(xmlNodePtr objElem, xmlNsPtr ns, GList *g, gchar *type) { 
-  xmlNodePtr directivesElem =  xmlNewChild (objElem, ns, (xmlChar *) type, NULL);
-  //GList *g = ((chord *) curObj->object)->directives;
+  xmlNodePtr directivesElem = NULL;  
   for(;g;g=g->next) {
     DenemoDirective *directive = (DenemoDirective *)g->data;
 
 /*      hhmmmm the set_action_script_for_tag is not modifying the directive - just associating a script with the tag. It is when we try to invoke the directive that we find there is an action script */
 /* We Decline to store any directive that has a tag for which an action script is defined in this run of denemo 
 *  this implies people need to be careful in the tags they use for action scripts*/
-
-/*      We have some serious thought needed here!!!!!!!!!!!!!!!!!!!!!!!! */
-/* could be */
     if(directive->tag && get_action_script (directive->tag->str))
        continue;
-
-
-     xmlNodePtr directiveElem =  xmlNewChild (directivesElem, ns, (xmlChar *) "directive", NULL);
+    if(directivesElem==NULL)
+      directivesElem = xmlNewChild (objElem, ns, (xmlChar *) type, NULL);
+    xmlNodePtr directiveElem =  xmlNewChild (directivesElem, ns, (xmlChar *) "directive", NULL);
 #define DO_DIREC(field)  if (directive->field \
                    && directive->field->len)\
                       xmlNewChild (directiveElem, ns, (xmlChar *) #field,\
@@ -519,12 +515,12 @@ newXMLAccidental (xmlNodePtr parent, xmlNsPtr ns, gint enshift, gboolean show)
  * as a child of the given node.
  */
 static xmlNodePtr
-newXMLStemDirective (xmlNodePtr parent, xmlNsPtr ns, enum stemdirections type)
+newXMLStemDirective (xmlNodePtr parent, xmlNsPtr ns, stemdirective *stem)
 {
   xmlNodePtr stemElem =
     xmlNewChild (parent, ns, (xmlChar *) "stem-directive", NULL);
   gchar *stemName = NULL;
-  switch (type)
+  switch (stem->type)
     {
     case DENEMO_STEMDOWN:
       stemName = "down";
@@ -536,11 +532,13 @@ newXMLStemDirective (xmlNodePtr parent, xmlNsPtr ns, enum stemdirections type)
       stemName = "up";
       break;
     default:
-      g_warning ("Unknown stem directive type %d, using auto", type);
+      g_warning ("Unknown stem directive type %d, using auto", stem->type);
       stemName = "auto";
       break;
     }
   xmlSetProp (stemElem, (xmlChar *) "type", (xmlChar *) stemName);
+ if(stem->directives)
+   newDirectivesElem(stemElem, ns, stem->directives, "directives");
   return stemElem;
 }
 
@@ -700,8 +698,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
   if(si->page_height)
     newXMLIntChild (parentElem, ns, (xmlChar *) "page-height",
 		    si->page_height);
-
-
+  if(si->measurewidth!=DENEMO_INITIAL_MEASURE_WIDTH)
+    newXMLIntChild (parentElem, ns, (xmlChar *) "measure-width",
+		    si->measurewidth);
 
   if(si->header.directives) {
     newDirectivesElem(mvmntElem, ns, si->header.directives, "header-directives");
@@ -717,7 +716,7 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
   curElem = xmlNewChild (parentElem, ns, (xmlChar *) "tempo", NULL);
   newXMLFraction (xmlNewChild (curElem, ns, (xmlChar *) "duration", NULL), ns,
 		  1, 4);
-  newXMLIntChild (curElem, ns, (xmlChar *) "bpm", si->tempo);
+  newXMLIntChild (curElem, ns, (xmlChar *) "bpm", si->tempo*si->master_tempo);
   
 
 
@@ -779,9 +778,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 	  if(curStaffStruct->verses)
 	    newVersesElem(curElem, ns, curStaffStruct->verses, "verses"); 
 	  
-	  if(curStaffStruct->staff_prolog && curStaffStruct->staff_prolog->len)
-	     xmlNewChild (curElem, ns, (xmlChar *) "staff-prolog",
-		       (xmlChar *) curStaffStruct->staff_prolog->str);
+	  //if(curStaffStruct->staff_prolog && curStaffStruct->staff_prolog->len)
+	  // xmlNewChild (curElem, ns, (xmlChar *) "staff-prolog",
+	  //	       (xmlChar *) curStaffStruct->staff_prolog->str);
 	  
 	  if(curStaffStruct->staff_directives) 
 	    newDirectivesElem(curElem, ns,curStaffStruct->staff_directives, "staff-directives");
@@ -792,15 +791,15 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 	    newDirectivesElem(curElem, ns,curStaffStruct->voice_directives, "clef-directives");
 	    
 
-	  if(curStaffStruct->lyrics_prolog && curStaffStruct->lyrics_prolog->len)
-	     xmlNewChild (curElem, ns, (xmlChar *) "lyrics-prolog",
-		       (xmlChar *) curStaffStruct->lyrics_prolog->str);
-	  if(curStaffStruct->figures_prolog && curStaffStruct->figures_prolog->len)
-	     xmlNewChild (curElem, ns, (xmlChar *) "figures-prolog",
-		       (xmlChar *) curStaffStruct->figures_prolog->str);
-	  if(curStaffStruct->fakechords_prolog && curStaffStruct->fakechords_prolog->len)
-	     xmlNewChild (curElem, ns, (xmlChar *) "fakechords-prolog",
-		       (xmlChar *) curStaffStruct->fakechords_prolog->str);
+	  //if(curStaffStruct->lyrics_prolog && curStaffStruct->lyrics_prolog->len)
+	  //  xmlNewChild (curElem, ns, (xmlChar *) "lyrics-prolog",
+	  //	       (xmlChar *) curStaffStruct->lyrics_prolog->str);
+	  // if(curStaffStruct->figures_prolog && curStaffStruct->figures_prolog->len)
+	  //  xmlNewChild (curElem, ns, (xmlChar *) "figures-prolog",
+	  //       (xmlChar *) curStaffStruct->figures_prolog->str);
+	  //if(curStaffStruct->fakechords_prolog && curStaffStruct->fakechords_prolog->len)
+	  //  xmlNewChild (curElem, ns, (xmlChar *) "fakechords-prolog",
+	  //	       (xmlChar *) curStaffStruct->fakechords_prolog->str);
 	}
     }
 
@@ -985,15 +984,10 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		    	
   			   GString *temp = g_string_new("");
 		           temp = g_string_append(temp, ((GString *) (thechord)->fakechord)->str);
-			   if ((thechord)->fakechord_extension != NULL)
-			   	temp = g_string_append(temp, ((GString *) (thechord)->fakechord_extension)->str);
 			   
-
 		      parentElem = xmlNewChild (objElem, ns, (xmlChar *) "fakechord",
 						(xmlChar *) ((GString *) temp)->str);
-
-  
-		      }
+  		    }
 
 
 		  /* Output all the decorations. */
@@ -1286,40 +1280,54 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		  break;
 
 		case TUPOPEN:
-		  objElem =
-		    xmlNewChild (measureElem, ns, (xmlChar *) "tuplet-start",
-				 NULL);
-		  /*
-		   * FIXME: This code does not yet handle nested tuplets.  For
-		   *        that, we'd need a stack of "tuplet-start" IDs
-		   *        instead of just a single "last ID."
-		   */
-
-		  lastTupletStartXMLID = getXMLID (curObj);
-		  xmlSetProp (objElem, (xmlChar *) "id",
-			      (xmlChar *) lastTupletStartXMLID);
-		  newXMLFraction (xmlNewChild
-				  (objElem, ns, (xmlChar *) "multiplier",
-				   NULL), ns,
-				  ((tupopen *) curObj->object)->numerator,
-				  ((tupopen *) curObj->object)->denominator);
+		  {
+		    tuplet *theob = (tuplet *) curObj->object;
+		    objElem =
+		      xmlNewChild (measureElem, ns, (xmlChar *) "tuplet-start",
+				   NULL);
+		    /*
+		     * not true? FIXME: This code does not yet handle nested tuplets.  For
+		     *        that, we'd need a stack of "tuplet-start" IDs
+		     *        instead of just a single "last ID."
+		     */
+		    
+		    lastTupletStartXMLID = getXMLID (curObj);
+		    xmlSetProp (objElem, (xmlChar *) "id",
+				(xmlChar *) lastTupletStartXMLID);
+		    newXMLFraction (xmlNewChild
+				    (objElem, ns, (xmlChar *) "multiplier",
+				     NULL), ns,
+				    ((tupopen *) curObj->object)->numerator,
+				    ((tupopen *) curObj->object)->denominator);
+		    if(theob->directives) {
+		      newDirectivesElem(objElem, ns, theob->directives, "directives");
+		      
+		    }
+		  }
 		  break;
-
+		  
 		case TUPCLOSE:
-		  objElem =
-		    xmlNewChild (measureElem, ns, (xmlChar *) "tuplet-end",
-				 NULL);
-		  if (lastTupletStartXMLID == NULL)
-		    {
-		      g_warning ("Encountered nested tuplets or tuplet end "
-				 "without start");
-		    }
-		  else
-		    {
-		      xmlSetProp (objElem, (xmlChar *) "tuplet",
-				  (xmlChar *) lastTupletStartXMLID);
-		      lastTupletStartXMLID = NULL;
+		  {
+		    tuplet *theob = (tuplet *) curObj->object;
+		    objElem =
+		      xmlNewChild (measureElem, ns, (xmlChar *) "tuplet-end",
+				   NULL);
+		    if (lastTupletStartXMLID == NULL)
+		      {
+			g_warning ("Encountered nested tuplets or tuplet end "
+				   "without start");
+		      }
+		    else
+		      {
+			xmlSetProp (objElem, (xmlChar *) "tuplet",
+				    (xmlChar *) lastTupletStartXMLID);
+			lastTupletStartXMLID = NULL;
+		      }
+		    if(theob->directives) {
+		      newDirectivesElem(objElem, ns, theob->directives, "directives");
+		      
 		    }
+		  }
 		  break;
 
 		case CLEF:
@@ -1339,9 +1347,11 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		  break;
 
 		case STEMDIRECTIVE:
-		  objElem = newXMLStemDirective (measureElem, ns,
-						 ((stemdirective *)
-						  curObj->object)->type);
+		  {
+		    stemdirective *theob = (stemdirective *) curObj->object;
+		    objElem = newXMLStemDirective (measureElem, ns, theob);
+		    
+		  }
 		  break;
 
 		case GRACE_START:
