@@ -8,7 +8,7 @@ gchar* get_temp_filename (const gchar* name);
 GPid spawn_external(const GString* cmdline);
 
 /* ext_pidfiles is a shared pid list used by ext_* functions.*/
-//static gchar* ext_pidfiles[] = {"midiplayer.pid", "csoundplayer.pid", NULL};
+static gchar* ext_pidfiles[] = {"midiplayer.pid", "csoundplayer.pid", NULL};
 
 /* GPID_UNREF contains the default value for an unreferenced GPid*/
 #ifdef G_OS_WIN32
