@@ -1,4 +1,4 @@
-/* figure.cpp
+/* figure.c
  *
  * Functions for the manipulations of figured basses
  *
@@ -43,6 +43,8 @@ insertfigure (gboolean filter, gpointer data)
   GString *current_figure;
   gchar filter_sep = filter?'/':'|';
   gchar filter_spc = filter?'*':' ';
+  if(si->measurewidth==DENEMO_INITIAL_MEASURE_WIDTH)
+    si->measurewidth = DENEMO_INITIAL_MEASURE_WIDTH*2;
 
   if (si->currentobject != NULL) {
     DenemoObject *curObj = (DenemoObject *) si->currentobject ?
@@ -129,12 +131,28 @@ void hide_figured_bass  (GtkAction *action, gpointer param) {
   DenemoStaff* thestaff = (DenemoStaff*)gui->si->currentstaff->data;
    thestaff->hasfigures=FALSE;
 }
+/* turn on figured bass if any figures are present */
+void show_figured_bass  (GtkAction *action, gpointer param) {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoStaff* thestaff = (DenemoStaff*)gui->si->currentstaff->data;
+  measurenode *curmeasure;
+  for(curmeasure = thestaff->measures;curmeasure;curmeasure=curmeasure->next) {
+      objnode *curobj;
+      for(curobj = curmeasure->data;curobj;curobj=curobj->next) {    
+	DenemoObject *curObj=(DenemoObject*)curobj->data;
+	if (curObj && curObj->type == CHORD) {
+	  GString *s= ((chord *) curObj->object)->figure;
+	  if(s) thestaff->hasfigures=TRUE;
+	}
+      }
+    }
+}
 /**
  * Creates figured bass entry dialog
  *
  */
 void
-figure_insert (GtkAction *action, gpointer param)
+figure_insert (GtkAction *action, DenemoScriptParam * param)
 {
   DenemoGUI *gui = Denemo.gui;
   gchar *string;
@@ -142,9 +160,9 @@ figure_insert (GtkAction *action, gpointer param)
   DenemoScore *si = gui->si;
   static struct callbackdata cbdata;
 
-  if(!action && param)
+  if(!action && param && param->string)
     {
-    GString *values = ((DenemoScriptParam *)param)->string;
+    GString *values = param->string;
     gchar *str;
 #define SET_STRING(a, b)     if( (str = g_strstr_len(values->str+i,strlen(values->str+i), a))) {\
       b = g_strdup(str+strlen(a)+1);\
