@@ -6,6 +6,7 @@
 #include "smf.h"
 #include "fluid.h"
 #include "moveviewport.h"
+#include "playback.h"
 
 #define MAX_NUMBER_OF_TRACKS    128
 #define NOTE_OFF                0x80
@@ -202,7 +203,7 @@ void fluid_playpitch(int key, int duration, int channel, int volume)
   if (synth){
     //g_print("Emitting key %d\n", key);
     fluid_synth_noteon(synth, channel, key, (volume?volume:127)*Denemo.gui->si->master_volume);
-    g_timeout_add(duration, noteoff_callback, (gpointer)( (channel<<8) + key)); 
+    g_timeout_add(duration,(GSourceFunc) noteoff_callback, (gpointer)( (channel<<8) + key)); 
   }
 }
 
@@ -310,6 +311,7 @@ static gboolean finish_play(gchar *callback) {
     safely_track_remove_from_smf(Denemo.gui->si->recorded_midi_track);
     finish_recording();
   }
+  set_tempo();
   return FALSE;
 }
 
@@ -317,6 +319,85 @@ static gboolean finish_play(gchar *callback) {
 
 static gdouble last_draw_time;
 static gdouble pause_time = -1.0;
+static  gdouble playalong_time = 0.0;
+static  gdouble early_time = 0.0;
+
+
+static gdouble GET_TIME (void) {
+  DenemoScore *si = Denemo.gui->si;
+  gdouble thetime = get_time();
+  if((Denemo.gui->midi_destination & (MIDIPLAYALONG|MIDICONDUCT)) && (thetime > si->start_player + playalong_time))
+    return playalong_time;
+  return thetime - si->start_player;
+}
+
+//test if the midi event in buf is a note-on for the current note
+//if so set a time delta so that a call to GET_TIME() will return get_time()- lost_time
+//unless this time is greater than the start time of the next note when GET_TIME() will stick
+//advance cursor to next note
+static void advance_clock(gchar *buf) {
+  if(Denemo.gui->si->currentobject) {
+    DenemoScriptParam aparam;
+    DenemoScriptParam *param = &aparam;
+    DenemoObject *obj = Denemo.gui->si->currentobject->data;
+    if(obj->type!=CHORD) 
+      if(cursor_to_next_chord(param)) 
+	obj = Denemo.gui->si->currentobject->data;
+    
+    if(Denemo.gui->si->currentobject && obj->type==CHORD) {
+      chord *thechord = obj->object;
+      if(thechord->notes) {
+	note *thenote = thechord->notes->data;
+	if( ((buf[0]&SYS_EXCLUSIVE_MESSAGE1)==NOTE_ON) && buf[2] && buf[1] == (dia_to_midinote (thenote->mid_c_offset) + thenote->enshift)) {
+	  gdouble thetime = get_time();
+	  Denemo.gui->si->start_player = thetime -  obj->earliest_time;
+	  
+	  if(thechord->is_tied && cursor_to_next_note(param)) {
+	    obj = Denemo.gui->si->currentobject->data;	   
+	  }
+	  //playalong_time = obj->latest_time;
+	  //IF THE NEXT OBJ IS A REST ADVANCE OVER IT/THEM
+	  do {
+	    if(!cursor_to_next_note(param))	//if(!cursor_to_next_chord())	   	      
+	      {
+		playalong_time = Denemo.gui->si->end_time + 1.0;
+		break;
+	      }
+	    else {
+	      obj = Denemo.gui->si->currentobject->data;
+	      thechord = obj->object;
+	      playalong_time = obj->earliest_time;
+	    }
+	  } 
+	  while(!thechord->notes);	    
+	}
+      }
+    } else
+      g_warning("Not on a chord");
+  } else
+    g_warning("Not on an object");
+}
+
+void advance_time(gdouble seconds) {
+  playalong_time += seconds;
+}
+static void initialize_clock(void) {
+  if(Denemo.gui->si->currentobject ) {
+    DenemoObject *obj = Denemo.gui->si->currentobject->data;
+    if(obj->type==CHORD) {
+      chord *thechord = obj->object;
+      if(thechord->notes) {
+	note *thenote = thechord->notes->data;      
+	//gboolean thetime = get_time();
+	//Denemo.gui->si->start_player =  thetime - obj->earliest_time;
+	playalong_time = obj->latest_time; 
+      }
+    }
+  }
+}
+
+
+
 static gboolean fluidsynth_play_smf_event(gchar *callback)
 {
   DenemoScore *si = Denemo.gui->si;
@@ -346,7 +427,7 @@ static gboolean fluidsynth_play_smf_event(gchar *callback)
   // g_print("rightmost %f event %f\n", si->rightmost_time, event->time_seconds);
   if(si->rightmost_time>0.0 && event->time_seconds>si->rightmost_time)
     page_viewport(), si->rightmost_time=-1;
-  gdouble thetime = get_time() - si->start_player;
+  gdouble thetime = GET_TIME();//get_time() - si->start_player;
   pause_time = thetime;
   //g_print("thetime %f\n", thetime);
   thetime -= si->tempo_change_time - si->start_player;
@@ -362,8 +443,8 @@ static gboolean fluidsynth_play_smf_event(gchar *callback)
 	event->time_seconds - last_draw_time>Denemo.prefs.display_refresh) {
        //       g_print("drawing because %f %f\n", event->time_seconds, last_draw_time);
        last_draw_time = event->time_seconds;
-       
-       gtk_widget_queue_draw (Denemo.gui->scorearea);
+       //gtk_widget_queue_draw (Denemo.scorearea);
+       region_playhead();//FIXME - need this to decide if the playhead has moved not just assume it has.!!!!
      }
     gint chan = (event->midi_buffer[0] & 0x0f);
     //g_print("message %x %x\n", event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1, PROGRAM_CHANGE);
@@ -437,6 +518,7 @@ void fluid_midi_play(gchar *callback)
   }
   if(gui->si->recorded_midi_track)
     safely_add_track(gui->si->smf, gui->si->recorded_midi_track);
+ 
   static GString *callback_string;
 
   if(callback_string==NULL)
@@ -461,19 +543,29 @@ void fluid_midi_play(gchar *callback)
     gui->si->start_time =  0.0;
   if(gui->si->start_time<0.0)
     gui->si->start_time = 0.0;
-  if( (pause_time>0.0) && (pause_time<gui->si->end_time))
-    gui->si->start_player = get_time() - pause_time;
-  else
-    pause_time = -1.0, gui->si->start_player = get_time() - gui->si->start_time;
+
+
+
   playing_piece = TRUE;
   toggle_playbutton();
-  gui->si->tempo_change_time = gui->si->start_player;
-
   smf_rewind(Denemo.gui->si->smf);
   last_draw_time = 0.0;
-  g_idle_add(fluidsynth_play_smf_event, callback_string->str);
-  smf_seek_to_seconds(gui->si->smf, pause_time>0.0? pause_time:gui->si->start_time);
+  if(Denemo.gui->midi_destination & (MIDIPLAYALONG|MIDICONDUCT))
+    initialize_clock();
+  initialize_playhead();
 
+  gint error = smf_seek_to_seconds(gui->si->smf, pause_time>0.0? pause_time:gui->si->start_time);
+
+  if( (pause_time>0.0) && (pause_time<gui->si->end_time))
+    gui->si->start_player = get_time() - pause_time;//FIXME ??
+  else
+    pause_time = -1.0, gui->si->start_player = get_time() - gui->si->start_time;//FIXME ??
+  gui->si->tempo_change_time = gui->si->start_player;
+
+  g_idle_add((GSourceFunc)fluidsynth_play_smf_event, callback_string->str);
+  //this is never turned off????
+  
+  // g_timeout_add_full (G_PRIORITY_HIGH, 15,(GSourceFunc)fluidsynth_play_smf_event, callback_string->str, NULL);                                             
 }
 
 void
@@ -513,65 +605,110 @@ fluid_rhythm_feedback(gint duration, gboolean rest, gboolean dot) {
 }
 
 
+
+
 static fluid_midi_driver_t* midi_in;
-//Under Interrupt
+//#define ENTERING_MASK (GDK_MOD1_MASK) //Alt
+#define EDITING_MASK (GDK_SHIFT_MASK)  
 static void handle_midi_event(gchar *buf) {
-  if(Denemo.gui->midi_destination & MIDIRECORD)
-    record_midi(buf,  get_time() - Denemo.gui->si->start_player);
-  if(Denemo.gui->midi_destination & MIDITHRU)
+  //g_print("%x : %x %x %x %x\n", Denemo.keyboard_state, GDK_CONTROL_MASK, GDK_SHIFT_MASK, GDK_MOD1_MASK, GDK_LOCK_MASK);
+
+  if( (Denemo.gui->midi_destination & MIDIRECORD) ||
+      (Denemo.gui->midi_destination & (MIDIPLAYALONG|MIDICONDUCT))) {
+    if(Denemo.gui->midi_destination & MIDIRECORD)
+      record_midi(buf,  get_time() - Denemo.gui->si->start_player);
+    if(Denemo.gui->midi_destination & (MIDIPLAYALONG))
+      advance_clock(buf);
     fluid_output_midi_event(buf);
-  else
-    process_midi_event(buf);
+  } else {
+    if((Denemo.keyboard_state==(GDK_SHIFT_MASK|GDK_LOCK_MASK)) ||
+       Denemo.keyboard_state==(GDK_CONTROL_MASK) ||
+       Denemo.keyboard_state==(ADDING_MASK) ||
+       Denemo.keyboard_state==((ADDING_MASK)|(CHORD_MASK)) ||
+       Denemo.keyboard_state==(GDK_CONTROL_MASK|GDK_LOCK_MASK) ||
+       (Denemo.keyboard_state==0))
+      process_midi_event(buf);
+    else
+      if(Denemo.keyboard_state==(GDK_SHIFT_MASK) ||
+	 Denemo.keyboard_state==(GDK_LOCK_MASK))
+	fluid_output_midi_event(buf);
+  }
 }
 
 
-
-static gchar midi_in_buf[3]; 
-static volatile gboolean midi_in_ready = FALSE;
+#define MAX_MIDI (10)
+static gchar midi_in_buf[MAX_MIDI][3]; 
+static volatile gint midi_in_count = 0;
+static gint midi_in_feed = 0;
 static midi_in_timer_id = 0;
 static gboolean midi_in_timer_callback(void) {
-  if(midi_in_ready)
-    handle_midi_event(midi_in_buf);
-  midi_in_ready = FALSE;
+
+  if(midi_in_count>midi_in_feed) {
+    gint theone;
+    midi_in_feed++;
+    theone = midi_in_feed;
+    if(midi_in_count==midi_in_feed)
+      midi_in_count=midi_in_feed=0;
+    handle_midi_event(midi_in_buf[theone]);
+  }
+  if(midi_in_count==midi_in_feed)
+    midi_in_count=midi_in_feed=0;
   return TRUE;//timer keeps going
 }
 
+
 //Under Interrupt
-static void handle_midi_in(void* data, fluid_midi_event_t* event)
+static void load_midi_buf(gint type, gint key, gint vel) {
+  midi_in_count++;
+  if(midi_in_count<MAX_MIDI) {
+  midi_in_buf[midi_in_count][0] = type;
+  midi_in_buf[midi_in_count][1] = key;
+  midi_in_buf[midi_in_count][2] = vel;
+  } 
+  else midi_in_count--;//over flow
+}
+
+//Under Interrupt
+static handle_midi_event_func_t handle_midi_in(void* data, fluid_midi_event_t* event)
 {
   int type = 
     fluid_midi_event_get_type(event);
   //g_print("event type: %x\n", type);
   switch(type) {
+
+
+
+  case CONTROL_CHANGE:
   case NOTE_ON:
   case NOTE_OFF:
   case KEY_PRESSURE:
-  case CONTROL_CHANGE:
+  
   case 0xF2:    
     {
       int key = fluid_midi_event_get_key(event);
       int velocity = fluid_midi_event_get_velocity(event);
-      midi_in_buf[0] = type;
-      midi_in_buf[1] = key;
-      midi_in_buf[2] = velocity;
       if(type==NOTE_ON && velocity==0) {//Zero velocity NOTEON is used as NOTEOFF by some MIDI controllers
-	midi_in_buf[0]=NOTE_OFF;
-	midi_in_buf[2]=127;
+	type=NOTE_OFF;
+	velocity=127;
       }
-      midi_in_buf[0] |= ((DenemoStaff *)Denemo.gui->si->currentstaff->data)->midi_channel;
-      //g_print("key is %d\n", key);
-      //handle_midi_event(midi_in_buf);
-      midi_in_ready = TRUE;
+#if 0
+      if (type == CONTROL_CHANGE && (key == 0x40)){
+	if (velocity == 0x7F)
+	  //PEDAL DOWN
+	  Denemo.keyboard_state |= ADDING_MASK;
+	else
+	  Denemo.keyboard_state &= ~(CHORD_MASK|ADDING_MASK);
+      }
+#endif  
+      type  |= ((DenemoStaff *)Denemo.gui->si->currentstaff->data)->midi_channel;
+      load_midi_buf(type, key, velocity);
     }
     break;
   case PITCH_BEND:
   
     {
       short key = fluid_midi_event_get_pitch(event);
-      midi_in_buf[0] = type;
-      midi_in_buf[1] = key>>7;
-      midi_in_buf[2] = key&0x7F;
-      midi_in_ready = TRUE;
+      load_midi_buf(type, key>>8, key&0x7F);
     }
     break;
 
@@ -580,15 +717,13 @@ static void handle_midi_in(void* data, fluid_midi_event_t* event)
   case 0xF3: 
     {
       int key = fluid_midi_event_get_key(event);
-      midi_in_buf[0] = type;
-      midi_in_buf[1] = key;
-      //handle_midi_event(midi_in_buf);
-      midi_in_ready = TRUE;
+      load_midi_buf(type, key, 0);
     }
     break;
   default:
     g_warning("not handled type %x\n", type);
   }
+  return 0;
 }
 
 
@@ -605,12 +740,16 @@ fluid_start_midi_in(void)
   success = fluid_settings_setstr(settings, "midi.driver", "alsa_seq");
 #endif
   //g_print("success %d\n", success);
-  midi_in = new_fluid_midi_driver(settings, handle_midi_in, NULL);
+  midi_in = new_fluid_midi_driver(settings, 
+    (handle_midi_event_func_t) handle_midi_in, NULL);
 
 
   //g_print("midi in on %p\n", midi_in);
   if(midi_in) {
-    midi_in_timer_id = g_timeout_add (20, (GSourceFunc) midi_in_timer_callback, NULL);
+      midi_in_timer_id = g_timeout_add (20, (GSourceFunc) midi_in_timer_callback, NULL);
+      //midi_in_timer_id = g_idle_add_full(G_PRIORITY_DEFAULT_IDLE-10, (GSourceFunc)midi_in_timer_callback, NULL, NULL);
+
+
     return 0;
   }
   else
@@ -638,5 +777,6 @@ void fluid_midi_panic(void){}
 int fluid_kill_timer(void){}
 int fluid_stop_midi_in(void){}
 int fluid_start_midi_in(void){}
+void advance_time(void) {}
 #endif 
 
