@@ -3,8 +3,12 @@
  * Grace note function prototypes
  *
  * for Denemo, a gtk+ frontend for GNU Lilypond
- * (c) 2000, 2001 Adam Tee 
+ * (c) 2000, 2001 Adam Tee, 2010 Richard Shann
  */
+
+#ifndef GRACEOPS_H
+#define GRACEOPS_H
+
 #include <glib.h>
 #include <denemo/denemo.h> 
 
@@ -13,3 +17,6 @@ DenemoObject* newgracestart();
 DenemoObject *newgraceend();
 /*void addchord(DenemoObject *thechord, DenemoObject *grace);*/
 
+void toggle_grace  (GtkAction *action, DenemoScriptParam * param);
+
+#endif //GRACEOPS_H
