@@ -695,8 +695,6 @@ importMidi (gchar *filename, DenemoGUI *gui)
   smf_t *smf;
   gint ret = 0;	// (-1 on failure)
 
-  /* delete old data in the score */
-  deletescore (NULL, gui);
   /* load the file */
   smf = cmd_load(filename);
   if(!smf)
