@@ -1,8 +1,9 @@
-/* importxml.cpp
+/* importxml.c
  * Import Denemo's "native" XML file format into a Denemo score structure
  *
  * for Denemo, a gtk+ frontend to GNU Lilypond
  * (c) 2001-2005 Eric Galluzzo, Adam Tee
+ * 2009, 2010, 2011 Richard Shann
  */
 
 #include "chordops.h"
@@ -190,37 +191,54 @@ getXMLChild (xmlNodePtr parentElem, gchar * childElemName, xmlNsPtr childNs)
  * Convert textual context to a denemo context
  *
  */
-static DenemoContext
-lookupContext (gchar * string)
+static void
+add_staff_context(DenemoContext c) {
+  if(c==DENEMO_NONE)
+    return;
+  switch (c) {
+  case DENEMO_PIANO_START:
+    staff_directive_put_prefix("ContextPianoStaff", " \\new PianoStaff <<\n");
+staff_directive_put_override("ContextPianoStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  case DENEMO_GROUP_START:
+    staff_directive_put_prefix("ContextGroupStaff", " \\new StaffGroup <<\n");
+staff_directive_put_override("ContextGroupStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  case DENEMO_CHOIR_START:
+    staff_directive_put_prefix("ContextChoirStaff", " \\new ChoirStaff <<\n");
+staff_directive_put_override("ContextChoirStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  case DENEMO_PIANO_END:
+    staff_directive_put_postfix("ContextPianoStaff", ">>\n");
+staff_directive_put_override("ContextPianoStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  case DENEMO_GROUP_END:
+    staff_directive_put_postfix("ContextGroupStaff", ">>\n");
+staff_directive_put_override("ContextGroupStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  case DENEMO_CHOIR_END:
+    staff_directive_put_postfix("ContextChoirStaff", ">>\n");
+staff_directive_put_override("ContextChoirStaff", DENEMO_OVERRIDE_AFFIX);
+    break;
+  default: g_warning("Unexpected context value\n");
+  }
+ 
+  
+}
+static void
+addContext (gchar * string)
 {
   if(string == NULL)
-    return 0;
+    return;
   static gboolean pianostaff=FALSE, groupstaff=FALSE, choirstaff=FALSE;
-  if (!strcmp (string, "PianoStaff"))//for backwards compatibility only
-    {
-      pianostaff = !pianostaff;
-      if(pianostaff)
-	return DENEMO_PIANO_START;
-      else
-	return DENEMO_PIANO_END;
-    }
-  else if (!strcmp (string, "ChoirStaff"))//for backwards compatibility only
-    {
-      choirstaff = !choirstaff;
-      if(choirstaff)
-	return DENEMO_CHOIR_START;
-      else
-	return DENEMO_CHOIR_END;
-    }
-  else if (!strcmp (string, "GroupStaff"))//for backwards compatibility only
-    {
-      groupstaff = !groupstaff;
-      if(groupstaff)
-	return DENEMO_GROUP_START;
-      else
-	return DENEMO_GROUP_END;
-    }
-#define LOOKUP(A,B)  else if (!strcmp (string, A)) {return B;}
+  if (
+      (!strcmp (string, "PianoStaff")) || 
+      (!strcmp (string, "ChoirStaff")) ||
+      (!strcmp (string, "GroupStaff"))) {
+    g_warning("Old context specs found - no longer supported. You will have to reset the Staff contexts\n");
+    return;
+  }
+#define LOOKUP(A,B)  if (!strcmp (string, A)) {add_staff_context(B); return;}
 LOOKUP(PIANO_START_STRING,DENEMO_PIANO_START)
 LOOKUP(PIANO_END_STRING,DENEMO_PIANO_END)
 LOOKUP(CHOIR_START_STRING,DENEMO_CHOIR_START)
@@ -228,12 +246,12 @@ LOOKUP(CHOIR_END_STRING,DENEMO_CHOIR_END)
 LOOKUP(GROUP_START_STRING,DENEMO_GROUP_START)
 LOOKUP(GROUP_END_STRING,DENEMO_GROUP_END)
 #undef LOOKUP
-  else
-    {
-      return DENEMO_NONE;
-    }
+
 }
 
+#define UPDATE_OVERRIDE(directive)		\
+  if(version_number<4)\
+    directive->override |= DENEMO_OVERRIDE_TAGEDIT;
 
 
 #define DO_DIREC(field) if (ELEM_NAME_EQ (childElem, #field))\
@@ -269,6 +287,8 @@ parseDirective (xmlNodePtr parentElem, xmlNsPtr ns,
       /* FIXME,handle not loaded */
     }      
   }
+  UPDATE_OVERRIDE(directive);
+ 
 }
 
 static gint
@@ -296,6 +316,8 @@ parseWidgetDirective (xmlNodePtr parentElem, xmlNsPtr ns, gpointer fn, DenemoDir
   }
   if(directive->tag==NULL)
     directive->tag = g_string_new("<Unknown Tag>");
+
+  UPDATE_OVERRIDE(directive);
   widget_for_directive(directive, fn);
   return TRUE;
 }
@@ -316,20 +338,19 @@ parseVerse (xmlNodePtr parentElem, xmlNsPtr ns,
   g_free (text);
 }
 
-static GList *
+static void
 parseVerses (DenemoScore *si, DenemoStaff *staff, xmlNodePtr parentElem, xmlNsPtr ns)
 {
-  GList *verses = NULL;
   xmlNodePtr childElem;
   FOREACH_CHILD_ELEM (childElem, parentElem) {
     GtkWidget *verse = add_verse_to_staff(si, staff);
     parseVerse(childElem, ns, verse);
   }
-  return verses;
 }
 
 /* Fix for semantic change to prefix field in the directives */
-static fix_prefix_use(GList *directives) {
+static void
+fix_prefix_use(GList *directives) {
   GList *g;
   for(g=directives;g;g=g->next) {
     DenemoDirective *directive = g->data;
@@ -739,13 +760,10 @@ parseFakechord (xmlNodePtr fakechordElem, DenemoObject * curobj)
 						  fakechordElem->xmlChildrenNode,
 						  1);
   if(fakechord) {
-    ((chord *) curobj->object)->is_fakechord = TRUE; 
-    separate_fakechord_elements (fakechord, curobj); 
+    ((chord *) curobj->object)->is_fakechord = TRUE;
+    ((chord *) curobj->object)->fakechord = g_string_new (fakechord);   
   }
-  //((chord *) curobj->object)->fakechord = g_string_new (fakechord);
-
   g_free (fakechord);
-
 }
 
 
@@ -1011,8 +1029,14 @@ parseRest (xmlNodePtr restElem, xmlNsPtr ns, DenemoScore * si)
   xmlNodePtr childElem;
 
   chordObj = parseBaseChord (restElem, ns, si);
-
-
+  FOREACH_CHILD_ELEM (childElem, restElem)
+    {
+      if (ELEM_NAME_EQ (childElem, "ticks"))
+	{
+	  chordObj->basic_durinticks = getXMLIntChild (childElem);
+	  ((chord *)chordObj->object)->baseduration = -chordObj->basic_durinticks;
+	}
+    }
   return chordObj;
 }
 
@@ -1532,7 +1556,7 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
 #define GET_INT_FIELD(x)\
   gchar *x = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) #x);\
   if(x)\
-    ((lilydirective*)curobj->object)->x = atoi(x);\
+    thedirective->x = atoi(x);\
   g_free(x);
 
   GET_INT_FIELD(x);
@@ -1542,6 +1566,7 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
   GET_INT_FIELD(gx);
   GET_INT_FIELD(gy);
   GET_INT_FIELD(override);
+  UPDATE_OVERRIDE(thedirective);
   GET_INT_FIELD(minpixels);
   // curobj->minpixelsalloted = thedirective->minpixels?thedirective->minpixels:16;//FIXME setpixelmin
   setpixelmin(curobj);
@@ -1562,10 +1587,11 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
  * @return the new DenemoObject
  * 
  */
-static DenemoObject *
+static void
 parseStemDirective (xmlNodePtr stemDirectiveElem,
-		    xmlNsPtr ns, DenemoScore * si)
+		    xmlNsPtr ns, stemdirective *stem)
 {
+  xmlNodePtr childElem;
   gchar *stemDirName =
     (gchar *) xmlGetProp (stemDirectiveElem, (xmlChar *) "type");
   enum stemdirections stemDir = DENEMO_STEMBOTH;
@@ -1583,7 +1609,16 @@ parseStemDirective (xmlNodePtr stemDirectiveElem,
     g_warning ("Invalid stem directive type \"%s\"; defaulting to auto",
 	       stemDirName);
 
-  return dnm_stem_directive_new (stemDir);
+  FOREACH_CHILD_ELEM (childElem, stemDirectiveElem)
+  {
+
+    if (ELEM_NAME_EQ (childElem, "directives"))
+      {
+	stem->directives = parseDirectives(childElem, ns);
+      }
+  }
+  stem->type = stemDir;
+  return;
 }
 
 
@@ -1596,9 +1631,10 @@ parseStemDirective (xmlNodePtr stemDirectiveElem,
  * 
  * @return the new DenemoObject
  */
-static DenemoObject *
-parseTupletStart (xmlNodePtr tupletStartElem, xmlNsPtr ns, DenemoScore * si)
+static void
+parseTupletStart (xmlNodePtr tupletStartElem, xmlNsPtr ns, tuplet *tup)
 {
+  xmlNodePtr childElem;
   xmlNodePtr multiplierElem = getXMLChild (tupletStartElem, "multiplier", ns);
   gboolean successful = FALSE;
   gint numerator, denominator;
@@ -1620,16 +1656,44 @@ parseTupletStart (xmlNodePtr tupletStartElem, xmlNsPtr ns, DenemoScore * si)
 		     "defaulting to 1/1");
 	}
     }
+  //FIXME get mult elem in this loop
+ FOREACH_CHILD_ELEM (childElem, tupletStartElem)
+  {
 
+    if (ELEM_NAME_EQ (childElem, "directives"))
+      {
+	tup->directives = parseDirectives(childElem, ns);
+      }
+  }
   if (!successful)
     {
       numerator = 1;
       denominator = 1;
     }
 
-  return newtupopen (numerator, denominator);
+
+  tup->numerator = numerator;
+  tup->denominator = denominator;
+  return;
 }
 
+/* tupet end  */
+static void
+parseTupletEnd (xmlNodePtr tupletStartElem, xmlNsPtr ns, tuplet *tup)
+{
+  xmlNodePtr childElem;
+
+ FOREACH_CHILD_ELEM (childElem, tupletStartElem)
+  {
+
+    if (ELEM_NAME_EQ (childElem, "directives"))
+      {
+	tup->directives = parseDirectives(childElem, ns);
+      }
+  }
+
+  return;
+}
 
 
 
@@ -1814,7 +1878,11 @@ parseEditInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si)
 	    si->page_height = getXMLIntChild (childElem);
 	    if(si->page_height < 0) si->page_height = 0;
 	  }
-
+	else if (ELEM_NAME_EQ (childElem, "measure-width"))
+	  {
+	    si->measurewidth = getXMLIntChild (childElem);
+	    if(si->measurewidth < 10) si->measurewidth = DENEMO_INITIAL_MEASURE_WIDTH;
+	  }
 
       }
   }
@@ -2107,7 +2175,7 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	    gchar *temp = NULL;
 	    temp = (gchar *) xmlNodeListGetString
 	      (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->context |= lookupContext (temp);
+	    addContext (temp);
 	    g_free (temp);
 	  }
 
@@ -2120,13 +2188,13 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	      g_warning("Ignoring the old-style string %s\nAdd this in LilyPond window if required\n", temp);
 	    g_free(temp);
 	  }
-	else if (ELEM_NAME_EQ (childElem, "staff-prolog"))
-	  {
-	    gchar *temp = 
-	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->staff_prolog = (temp?g_string_new(temp):NULL);
-	    g_free (temp);
-	  }
+/* 	else if (ELEM_NAME_EQ (childElem, "staff-prolog")) */
+/* 	  { */
+/* 	    gchar *temp =  */
+/* 	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1); */
+/* 	    curStaff->staff_prolog = (temp?g_string_new(temp):NULL); */
+/* 	    g_free (temp); */
+/* 	  } */
 	else if (ELEM_NAME_EQ (childElem, "staff-prolog-insert"))//backward compatibility only
 	  {
 	    gchar *temp = 
@@ -2163,27 +2231,27 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	  }
 
 
-	else if (ELEM_NAME_EQ (childElem, "lyrics-prolog"))
-	  {
-	    gchar *temp = 
-	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->lyrics_prolog = (temp?g_string_new(temp):NULL);
-	    g_free (temp);
-	  }
-	else if (ELEM_NAME_EQ (childElem, "figures-prolog"))
-	  {
-	    gchar *temp = 
-	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->figures_prolog = (temp?g_string_new(temp):NULL);
-	    g_free (temp);
-	  }
-	else if (ELEM_NAME_EQ (childElem, "fakechords-prolog"))
-	  {
-	    gchar *temp = 
-	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->fakechords_prolog = (temp?g_string_new(temp):NULL);
-	    g_free (temp);
-	  }
+/* 	else if (ELEM_NAME_EQ (childElem, "lyrics-prolog")) */
+/* 	  { */
+/* 	    gchar *temp =  */
+/* 	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1); */
+/* 	    curStaff->lyrics_prolog = (temp?g_string_new(temp):NULL); */
+/* 	    g_free (temp); */
+/* 	  } */
+/* 	else if (ELEM_NAME_EQ (childElem, "figures-prolog")) */
+/* 	  { */
+/* 	    gchar *temp =  */
+/* 	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1); */
+/* 	    curStaff->figures_prolog = (temp?g_string_new(temp):NULL); */
+/* 	    g_free (temp); */
+/* 	  } */
+/* 	else if (ELEM_NAME_EQ (childElem, "fakechords-prolog")) */
+/* 	  { */
+/* 	    gchar *temp =  */
+/* 	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1); */
+/* 	    curStaff->fakechords_prolog = (temp?g_string_new(temp):NULL); */
+/* 	    g_free (temp); */
+/* 	  } */
 	else
 	  {
 	    ILLEGAL_ELEM ("staff-info", childElem);
@@ -2518,7 +2586,8 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
 		}
 	      else if (ELEM_NAME_EQ (objElem, "stem-directive"))
 		{
-		  curObj = parseStemDirective (objElem, ns, si);
+		  curObj = dnm_stem_directive_new (DENEMO_STEMBOTH);
+		  parseStemDirective (objElem, ns, curObj->object);
 		}
 	      else if (ELEM_NAME_EQ (objElem, "time-signature"))
 		{
@@ -2528,10 +2597,12 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
 	      else if (ELEM_NAME_EQ (objElem, "tuplet-end"))
 		{
 		  curObj = newtupclose ();
+		  parseTupletEnd (objElem, ns, curObj->object);
 		}
 	      else if (ELEM_NAME_EQ (objElem, "tuplet-start"))
 		{
-		  curObj = parseTupletStart (objElem, ns, si);
+		  curObj = newtupopen (4, 4);
+		  parseTupletStart (objElem, ns, curObj->object);
 		}
 	      else
 		{
