@@ -13,6 +13,7 @@
 #include "smf.h"
 #include "device_manager.h"
 #include "moveviewport.h"
+#include "playback.h"
 #define NOTE_OFF                0x80
 #define NOTE_ON                 0x90
 #define SYS_EXCLUSIVE_MESSAGE1  0xF0
@@ -234,6 +235,7 @@ send_midi_event(jack_nframes_t nframes, gint client_number){
 static gboolean finish_play(gchar *callback) {
   if(callback && *callback)
     call_out_to_guile (callback);
+  set_tempo();
   return FALSE;
 }
 
@@ -256,7 +258,7 @@ static gboolean jackmidi_play_smf_event(gchar *callback)
     si->playingnow = NULL;
     playing_piece = FALSE;
     pause_time = -1.0;
-    toggle_playbutton();
+    toggle_playbutton();    
     return  finish_play(callback);
   }
   else 
@@ -291,7 +293,7 @@ static gboolean jackmidi_play_smf_event(gchar *callback)
        //       g_print("drawing because %f %f\n", event->time_seconds, last_draw_time);
        last_draw_time = event->time_seconds;
        
-       gtk_widget_queue_draw (Denemo.gui->scorearea);
+       gtk_widget_queue_draw (Denemo.scorearea);
      }
     gint chan = (event->midi_buffer[0] & 0x0f);
     //g_print("message %x %x\n", event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1, PROGRAM_CHANGE);
@@ -317,6 +319,38 @@ static gboolean jackmidi_play_smf_event(gchar *callback)
   return TRUE;
 }
 
+static void handle_midi_event(gchar *buf) {
+  if(Denemo.gui->midi_destination & MIDIRECORD)
+    record_midi(buf,  get_time() - Denemo.gui->si->start_player);
+    process_midi_event(buf);
+}
+
+#define MAX_MIDI (10)
+static gchar midi_in_buf[MAX_MIDI][3]; 
+static volatile gboolean midi_in_count = 0;
+static midi_in_timer_id = 0;
+static gboolean midi_in_timer_callback(void) {
+  if(midi_in_count) {
+    gint i;
+    for(i=0;i<midi_in_count;i++)
+      handle_midi_event(midi_in_buf[i+1]);
+  }
+  midi_in_count = 0;//if an event has arrived during the last note entry we will lose it here - it is not critical, if it were we would have to turn off interrupts while working on the counter...
+  return TRUE;//timer keeps going
+}
+
+//Under Interrupt
+static void load_midi_buf(gint type, gint key, gint vel) {
+  midi_in_count++;
+  if(midi_in_count<MAX_MIDI) {
+  midi_in_buf[midi_in_count][0] = type;
+  midi_in_buf[midi_in_count][1] = key;
+  midi_in_buf[midi_in_count][2] = vel;
+  } 
+  else midi_in_count--;//over flow
+}
+
+
 static void
 process_midi_input(jack_nframes_t nframes)
 {
@@ -331,7 +365,8 @@ process_midi_input(jack_nframes_t nframes)
   events = jack_midi_get_event_count(port_buffer);
   for (i = 0; i < events; i++) {
     read = jack_midi_event_get(&event, port_buffer, i);
-    process_midi_event(event.buffer);
+    //process_midi_event(event.buffer);
+    load_midi_buf (event.buffer[0], event.buffer[1], event.buffer[2]);
   }
 }
 
@@ -633,7 +668,7 @@ init_jack(void){
       if (i==0 && MD[0].jack_client && input_port==NULL)
         input_port = jack_port_register(MD[0].jack_client, INPUT_PORT_NAME, JACK_DEFAULT_MIDI_TYPE,
 				      JackPortIsInput, 0);
-  
+      midi_in_timer_id = g_timeout_add (20, (GSourceFunc) midi_in_timer_callback, NULL);
       /* activate client */
       jack_activate(MD[i].jack_client);
     }
@@ -666,6 +701,8 @@ void jack_midi_play(gchar *callback)
   if(playing_piece) {
     // pause
     toggle_playbutton();
+    if (jack_transport)
+		jack_transport_stop(MD[0].jack_client);
     playing_piece = FALSE;
     (void)finish_play(callback);
     return;
