@@ -73,6 +73,44 @@ static guint lock_mask(gint keyval) {
 
 }
 
+static guint klock_mask(gint keyval) {
+
+  if((keyval==GDK_Shift_L)||
+     (keyval==GDK_Shift_R))
+    return GDK_SHIFT_MASK;
+  
+  if((keyval==GDK_Control_L)||
+     (keyval==GDK_Control_R))
+    return GDK_CONTROL_MASK;
+
+
+  if(keyval==GDK_Alt_L)
+    return GDK_MOD1_MASK;
+
+
+     /*penguin/windows */
+  if((keyval==GDK_Super_L)||
+     (keyval==GDK_Super_R))
+    return GDK_MOD4_MASK;
+
+  if(keyval==GDK_Alt_R || GDK_ISO_Level3_Shift )
+    return GDK_MOD5_MASK;
+
+  return 0;
+
+}
+
+static guint llock_mask(gint keyval) {
+  if(keyval==GDK_Caps_Lock)
+    return GDK_LOCK_MASK;
+  
+
+  if(keyval==GDK_Num_Lock)
+    return GDK_MOD2_MASK;
+
+  return 0;
+
+}
 
 
 /**
@@ -83,47 +121,39 @@ static guint lock_mask(gint keyval) {
 gint
 scorearea_keyrelease_event (GtkWidget * widget, GdkEventKey * event)
 {
+  Denemo.keyboard_state ^= (0xf & klock_mask(event->keyval));
+  if((event->keyval==GDK_Alt_L)||(event->keyval==GDK_Alt_R))
+    Denemo.keyboard_state &= ~CHORD_MASK;
+  set_midi_in_status();
+  // g_print("release %x state %x\n", Denemo.keyboard_state, event->state);
   // set_cursor_for(keyrelease_modify(event->state), event->keyval);
   gint state;
   if((event->keyval==GDK_Caps_Lock) || (event->keyval==GDK_Num_Lock))
     return TRUE;
   state = (lock_mask(event->keyval)^event->state);
+
   set_cursor_for(state);
   return TRUE;
 }
 
 /* perform the command of the given name and store the event that triggered it */
 static
-gboolean perform_command(const gchar *command_name, GdkEventKey *event) {
+gchar* perform_command(const gchar *command_name, GdkEventKey *event) {
   Denemo.last_keyval = event->keyval;
   Denemo.last_keystate =  dnm_sanitize_key_state(event);
   call_out_to_guile("(define DenemoKeypressActivatedCommand #t)");
   execute_callback_from_name(Denemo.map, command_name);
   call_out_to_guile("(define DenemoKeypressActivatedCommand #f)");
   // note gui = Denemo.gui; may have changed as a result of executing the command
-  displayhelper (Denemo.gui);
-  //gtk_widget_draw (gui->scorearea, NULL);//FIXME what is this doing here?   
-  return TRUE;
+  if(Denemo.gui->si)
+    displayhelper (Denemo.gui);
+  return NULL;
 }
 
-
-/**
- * keypress event callback 
- * looks up the key press and executes the correct function
- */
-
-gint
-scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
-{
+//return the value of perform_command if executed or "" if keypress is part of a two-key shortcut, or NULL toherwise
+gchar *  process_key_event(GdkEventKey * event, gchar* perform_command()) {
   DenemoGUI *gui = Denemo.gui;
   keymap *the_keymap = Denemo.map;
-  if(divert_key_event && !isModifier(event) && divert_key_id==Denemo.gui->id) {
-    dnm_clean_event (event);
-    *divert_key_event = event;
-    //g_object_ref(event); FIXME do we need to keep it around?
-    gtk_main_quit();
-    return TRUE;
-  }
   // g_print("\n********\nCaps Lock %x?\n\n********\nShifted %x?\n", event->state&GDK_LOCK_MASK,	  event->state&GDK_SHIFT_MASK	  );
   {
     gint state;
@@ -135,7 +165,7 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
 
 
   if (isModifier(event))
-    return TRUE;
+    return NULL;
 
 
   /* Look up the keystroke in the keymap and execute the appropriate
@@ -152,13 +182,13 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
       return perform_command(command_name, event);
     } else {
       g_warning("Error: action %i has no name", command_idx);
-      return FALSE;
+      return NULL;
     }
   }
 
     /*  we create a store for the prefix char and look to see if it is populated when a keystroke is received. If it is populated, we try for the two-key combination, {???else we try for the single key, and if that fails populate the store. OR if it fails clear store}. If the two-key combination works we clear the store. If the two-key combination fails we try for the single key, if that succeeds we clear the store if it fails we set the store to the unresolved keystroke.  */
 
-  gboolean ret = FALSE;
+  gchar * ret = NULL;
   if(prefix_store->len) {
     gchar *name = dnm_accelerator_name(event->keyval, event->state);
     //g_print("second key %s\n", name);
@@ -172,7 +202,7 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
 	ret = perform_command(command_name, event);
       }
     } else { //Two key name was not a binding
-      ret = FALSE; //WHY??? FIXME
+      ret = NULL;
       write_status(Denemo.gui);
       toggle_to_drawing_area(TRUE);//restore menus, in case the user is lost and needs to look up a keypress
     }
@@ -185,20 +215,54 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
       GList *g;
       GString *continuations = g_string_new("");
       for(g=Denemo.continuations;g;g=g->next) 
-	g_string_append_printf(continuations, "%s%s", g->data, ", or ");
+	g_string_append_printf(continuations, "%s%s", (gchar *) g->data, ", or ");
       g_string_printf(prefix_store, "Prefix Key %s, waiting for key %stype Esc to abort", name, continuations->str);
       g_string_free(continuations, TRUE);
       gtk_statusbar_pop(GTK_STATUSBAR (Denemo.statusbar), Denemo.status_context_id);
       gtk_statusbar_push(GTK_STATUSBAR (Denemo.statusbar), Denemo.status_context_id,
 			 prefix_store->str);
       g_string_assign(prefix_store, name);
+      return ""; //continuation available
     } else {
       write_status(Denemo.gui);//WHY FIXME
      toggle_to_drawing_area(TRUE);//restore menus, in case the user is lost and needs to look up a keypress
     }
-    return TRUE;
+    return NULL;
   }
-  return TRUE;
+  return NULL;
+}
+
+
+/**
+ * keypress event callback 
+ * looks up the key press and executes the correct function
+ */
+
+gint
+scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
+{
+  DenemoGUI *gui = Denemo.gui;
+  keymap *the_keymap = Denemo.map;
+
+  Denemo.keyboard_state |= (0xf & klock_mask(event->keyval));
+  Denemo.keyboard_state ^= llock_mask(event->keyval);
+  // if((event->keyval==GDK_Alt_L)||(event->keyval==GDK_Alt_R))
+  //  Denemo.keyboard_state |= CHORD_MASK;
+  set_midi_in_status();
+
+  //g_print("press Denemo %x state %x klock %x\n", Denemo.keyboard_state, event->state, klock_mask(event->keyval));
+
+  // g_print("State eored %x\n", (lock_mask(event->keyval)^event->state));
+  if(divert_key_event && !isModifier(event) && divert_key_id==Denemo.gui->id) {
+    dnm_clean_event (event);
+    *divert_key_event = event;
+    //g_object_ref(event); FIXME do we need to keep it around?
+    gtk_main_quit();
+    return TRUE;//*is* reached main loop exits to the caller of the loop when it next gains control
+  }
+
+  (void)process_key_event(event, perform_command);
+  return TRUE;//I think this means do not run any other handlers after this.
 }
 
 /**
@@ -334,7 +398,7 @@ go_to_G_key (DenemoGUI * gui)
 
 
 /**
- * Move cursor by amount or in EDIT mode change the note by one octave
+ * Move cursor by amount or in EDIT mode change the note by the amount
  */
 static void
 octave_shift_key (DenemoGUI * gui, gint amount)
@@ -349,6 +413,8 @@ octave_shift_key (DenemoGUI * gui, gint amount)
 	  if(thenote) {
 	    note copy = *((note *) thenote->data);
 	    GList *direcs = ((note *)thenote->data)->directives;
+	    store_for_undo_change (gui->si, gui->si->currentobject->data);
+	    gui->si->undo_guard++;
 	    delete_chordnote(gui);//does not delete the directives.
 	    gui->si->cursor_y = copy.mid_c_offset + amount;
 	    insert_chordnote(gui);
@@ -356,6 +422,7 @@ octave_shift_key (DenemoGUI * gui, gint amount)
 	    thenote = nearestnote (gui->si->currentobject->data, gui->si->cursor_y);
 	    if(thenote)
 	      ((note *)thenote->data)->directives = direcs;
+	    gui->si->undo_guard--;
 	    score_status(gui, TRUE);
 	  }
 	}
@@ -802,9 +869,14 @@ tie_notes_key (DenemoGUI * gui)
   if (curmudelaobj && curmudelaobj->type == CHORD &&
       ((chord *) curmudelaobj->object)->notes)
     {
+    
       insertion_point (gui->si);
       object_insert (gui, dnm_clone_object (curmudelaobj));
-      ((chord *) curmudelaobj->object)->is_tied = TRUE;
+      movecursorleft(NULL);
+      movecursorleft(NULL);
+      toggle_tie(NULL, NULL);
+      movecursorright(NULL);
+      movecursorright(NULL);
     }
 }
 
