@@ -1,8 +1,8 @@
-/* lilydirectives.cpp 
+/* lilydirectives.c 
  * Implements lilydirectives which are not notes 
  *
  * for Denemo, a gtk+ frontend to GNU Lilypond
- * Richard Shann 2009
+ * Richard Shann 2009, 2010
  * A Tee  (c) 2000-2005
  */
 #include <stdio.h>
@@ -21,9 +21,6 @@
 #include "utils.h"
 #include "prefops.h"
 #include "view.h"
-static void edit_directive_callback(GtkWidget *w, gpointer what);
-
-
 
 static DenemoDirective *get_movementcontrol_directive(gchar *tag);
 static DenemoDirective *get_score_directive(gchar *tag);
@@ -554,6 +551,55 @@ gboolean delete_timesig_directive(gchar *tag) {
   return delete_directive(&curtimesig->directives, tag);
 }
 
+static  tuplet *get_tuplet(void) {
+  tuplet *ret = NULL;
+  DenemoObject *curObj = findobj();  
+  if(curObj && (curObj->type==TUPOPEN || curObj->type==TUPCLOSE)){
+    ret = ((tuplet*)curObj->object);
+  }
+  return ret;
+}
+static
+DenemoDirective *get_tuplet_directive(gchar *tag) {
+  tuplet *curtuplet = get_tuplet();
+  if(curtuplet==NULL || (curtuplet->directives==NULL))
+    return NULL;
+  return find_directive(curtuplet->directives, tag);
+}
+gboolean delete_tuplet_directive(gchar *tag) {
+  tuplet *curtuplet = get_tuplet();
+  if(curtuplet==NULL || (curtuplet->directives==NULL))
+    return FALSE;
+  DenemoDirective *directive = get_tuplet_directive(tag);
+  if(directive==NULL)
+    return FALSE;
+  return delete_directive(&curtuplet->directives, tag);
+}
+
+static  stemdirective *get_stemdirective(void) {
+  stemdirective *ret = NULL;
+  DenemoObject *curObj = findobj();  
+  if(curObj && (curObj->type==STEMDIRECTIVE)){
+    ret = ((stemdirective*)curObj->object);
+  }
+  return ret;
+}
+static
+DenemoDirective *get_stemdirective_directive(gchar *tag) {
+  stemdirective *curstemdirective = get_stemdirective();
+  if(curstemdirective==NULL || (curstemdirective->directives==NULL))
+    return NULL;
+  return find_directive(curstemdirective->directives, tag);
+}
+gboolean delete_stemdirective_directive(gchar *tag) {
+  stemdirective *curstemdirective = get_stemdirective();
+  if(curstemdirective==NULL || (curstemdirective->directives==NULL))
+    return FALSE;
+  DenemoDirective *directive = get_stemdirective_directive(tag);
+  if(directive==NULL)
+    return FALSE;
+  return delete_directive(&curstemdirective->directives, tag);
+}
 
 static  scoreheader *get_scoreheader(void) {
   return &Denemo.gui->scoreheader;
@@ -800,6 +846,8 @@ typedef DenemoLilyControl score;
 what##_directive_put_graphic(gchar *tag, gchar *value) {\
   what *current = get_##what();\
   if(current==NULL) return FALSE;\
+  if(Denemo.gui->si->currentobject)\
+  store_for_undo_change (Denemo.gui->si, Denemo.gui->si->currentobject->data);\
   if(current->directives==NULL)\
        create_directives (&current->directives, tag);\
   DenemoDirective *directive = get_##what##_directive(tag);\
@@ -820,6 +868,8 @@ PUT_GRAPHIC(note);
 
 PUT_GRAPHIC(keysig)
 PUT_GRAPHIC(timesig)
+PUT_GRAPHIC(tuplet)
+PUT_GRAPHIC(stemdirective)
 
 
 
@@ -828,6 +878,8 @@ gboolean \
 what##_directive_put_##field(gchar *tag, gchar *value) {\
   what *current = get_##what();\
   if(current==NULL) return FALSE;\
+  if(Denemo.gui->si->currentobject)\
+  store_for_undo_change (Denemo.gui->si, Denemo.gui->si->currentobject->data);\
   if(current->name==NULL)\
        create_directives (&current->name, tag);\
   DenemoDirective *directive = get_##what##_directive(tag);\
@@ -845,8 +897,8 @@ what##_directive_put_##field(gchar *tag, gchar *value) {\
 }
 
 #define PUT_STR_FIELD_FUNC(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, directives)
-#define PUT_STR_FIELD_FUNCS(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, staff_directives)
-#define PUT_STR_FIELD_FUNCV(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, voice_directives)
+//#define PUT_STR_FIELD_FUNCS(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, staff_directives)
+//#define PUT_STR_FIELD_FUNCV(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, voice_directives)
 
 
 
@@ -859,6 +911,8 @@ GET_TAG_FUNC(voice);
 GET_TAG_FUNC(score);
 GET_TAG_FUNC(clef);
 GET_TAG_FUNC(timesig);
+GET_TAG_FUNC(tuplet);
+GET_TAG_FUNC(stemdirective);
 GET_TAG_FUNC(keysig);
 GET_TAG_FUNC(scoreheader);
 GET_TAG_FUNC(header);
@@ -930,6 +984,8 @@ gboolean \
 what##_directive_put_##field(gchar *tag, gint value) {\
   what *current = get_##what();\
   if(current==NULL) return FALSE;\
+  if(Denemo.gui->si->currentobject)\
+  store_for_undo_change (Denemo.gui->si, Denemo.gui->si->currentobject->data);\
   if(current->name==NULL)\
        create_directives (&current->name, tag);\
   DenemoDirective *directive = get_##what##_directive(tag);\
@@ -943,8 +999,8 @@ what##_directive_put_##field(gchar *tag, gint value) {\
   return TRUE;\
 }
 #define PUT_INT_FIELD_FUNC(what, field)  PUT_INT_FIELD_FUNC_NAME(what, field, directives)
-#define PUT_INT_FIELD_FUNCS(what, field)  PUT_INT_FIELD_FUNC_NAME(what, field, staff_directives)
-#define PUT_INT_FIELD_FUNCV(what, field)  PUT_INT_FIELD_FUNC_NAME(what, field, voice_directives)
+//#define PUT_INT_FIELD_FUNCS(what, field)  PUT_INT_FIELD_FUNC_NAME(what, field, staff_directives)
+//#define PUT_INT_FIELD_FUNCV(what, field)  PUT_INT_FIELD_FUNC_NAME(what, field, voice_directives)
 
 
 #define GET_INT_FIELD_FUNC(what, field)\
@@ -976,6 +1032,8 @@ PUT_INT_FIELD_FUNC(chord, minpixels)
      //PUT_INT_FIELD_FUNC(score, minpixels)
 PUT_INT_FIELD_FUNC(clef, minpixels)
 PUT_INT_FIELD_FUNC(timesig, minpixels)
+PUT_INT_FIELD_FUNC(tuplet, minpixels)
+PUT_INT_FIELD_FUNC(stemdirective, minpixels)
 PUT_INT_FIELD_FUNC(keysig, minpixels)
 
      //PUT_INT_FIELD_FUNC(scoreheader, minpixels)
@@ -993,6 +1051,8 @@ GET_INT_FIELD_FUNC(score, minpixels)
 GET_INT_FIELD_FUNC(clef, minpixels)
 GET_INT_FIELD_FUNC(keysig, minpixels)
 GET_INT_FIELD_FUNC(timesig, minpixels)
+GET_INT_FIELD_FUNC(tuplet, minpixels)
+GET_INT_FIELD_FUNC(stemdirective, minpixels)
 
 GET_INT_FIELD_FUNC(scoreheader, minpixels)
 GET_INT_FIELD_FUNC(header, minpixels)
@@ -1146,8 +1206,8 @@ and allow advanced edit if right button.
 static gboolean
 text_edit_directive_by_fn(DenemoDirective *directive, gpointer fn);
 static void
-button_callback  (GtkWidget *w, GdkEventButton *event, DenemoDirective *directive) {
-  gboolean left = (event->button != 3);                  
+button_callback  (GtkWidget *widget, GdkEventButton *event, DenemoDirective *directive) {
+  gboolean left = event!=NULL && (event->button != 3);                  
   if(directive->override&DENEMO_OVERRIDE_EDITOR)
     {
       GtkWidget *texteditor = (GtkWidget*)g_object_get_data(G_OBJECT(directive->widget), DENEMO_TEXTEDITOR_TAG);
@@ -1158,10 +1218,13 @@ button_callback  (GtkWidget *w, GdkEventButton *event, DenemoDirective *directiv
       }
     } else {
     gchar *script = get_action_script(directive->tag->str);
-    if(script)
+    if(script) {
+      stage_undo(Denemo.gui->si, ACTION_STAGE_END);//undo is a queue so this is the end :)
       call_out_to_guile(script);
+      stage_undo(Denemo.gui->si, ACTION_STAGE_START);
+    }
     else {
-      if(left)
+      if(left  && (directive->override&DENEMO_OVERRIDE_TAGEDIT))
 	script = get_editscript_filename(directive->tag->str);
       else
 	script = NULL;
@@ -1173,15 +1236,15 @@ button_callback  (GtkWidget *w, GdkEventButton *event, DenemoDirective *directiv
 
 	*/
 	GtkAction *action;
-	if(left && ((action = lookup_action_from_name((gchar *)directive->tag->str))!=NULL))
+	if(left && (directive->override&DENEMO_OVERRIDE_TAGEDIT) &&((action = lookup_action_from_name((gchar *)directive->tag->str))!=NULL))
 	  gtk_action_activate(action); else {
-	  gpointer fn = g_object_get_data(G_OBJECT(w), "fn");
+	  gpointer fn = (widget!=NULL)? g_object_get_data(G_OBJECT(widget), "fn"):NULL;
 	  if(fn) {
 	    gboolean delete = !text_edit_directive_by_fn(directive, fn);
 	    if(delete) {
-	      GList *directives = (GList*)g_object_get_data(w, "directives-pointer");
+	      GList **directives = (GList**)g_object_get_data(G_OBJECT(widget), "directives-pointer");
 	      if(directives)
-		delete_directive(directives, directive->tag->str);
+	        delete_directive(directives, directive->tag->str);
 	      else
 		g_warning("Could not get directives list to delete from");
 	    }
@@ -1226,7 +1289,7 @@ eles use tag
 static void
 set_directive_graphic_label(DenemoDirective *directive) {
   gchar *value;
-  if(directive->display && directive->display->len>1)
+  if(directive->display && directive->display->len>0)
     value = g_strdup_printf("%s", directive->display->str);
   else
     value = g_strdup(directive->tag->str);
@@ -1306,7 +1369,7 @@ if directive is non-DenemoObject directive it  places the widget in the appropri
 */
      
 void 
-widget_for_directive(DenemoDirective *directive,  void fn()) {
+widget_for_directive_menu(DenemoDirective *directive,  void fn(), GtkMenu *staffmenu, GtkMenu *voicemenu) {
   GtkWidget *box;
   gchar *value = "";
   //FIXME we don't need value now...
@@ -1334,7 +1397,7 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
       //g_print("Doing the staff case");
       /* g_print("directive-type %s.....", thetype);	*/
       GtkWidget *menu;
-      menu = (GtkWidget *)((DenemoStaff*)Denemo.gui->si->currentstaff->data)->staffmenu;
+      menu = GTK_WIDGET(staffmenu);
       directive->widget = gtk_menu_item_new_with_label(value);
       attach_textedit_widget(directive);
       g_signal_connect(G_OBJECT(directive->widget), "button-release-event",  G_CALLBACK(button_callback), directive);
@@ -1345,12 +1408,12 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
 	directive->widget = GTK_WIDGET(gtk_menu_item_new_with_label(value));//WARNING _with_label is important
 	attach_textedit_widget(directive);
 	GtkWidget *menu;
-	menu = (GtkWidget *)((DenemoStaff*)Denemo.gui->si->currentstaff->data)->voicemenu;  
+	menu = GTK_WIDGET(voicemenu);  
 	g_signal_connect(G_OBJECT(directive->widget), "button-release-event",  G_CALLBACK(button_callback), directive);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), GTK_WIDGET(directive->widget));
       }  else 
 	if(box)  {
-	  //g_print("Doing the score and movement cases");
+	  //g_print("Doing the score and movement cases starting from %p", directive->widget);
 	  directive->widget = GTK_WIDGET(gtk_button_new_with_label(value));
 	  {
 	    GtkWidget *label = gtk_bin_get_child(GTK_BIN(directive->widget));
@@ -1382,15 +1445,86 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
   }
   set_directive_graphic_label(directive);
 }
+void 
+widget_for_directive(DenemoDirective *directive,  void fn()) {
+GtkMenu *staffmenu=NULL, *voicemenu=NULL;
+ if(Denemo.gui->si){
+   if( (fn == (void(*)())staff_directive_put_graphic) || (fn==(void(*)())voice_directive_put_graphic)) {
+   staffmenu =  ((DenemoStaff*)Denemo.gui->si->currentstaff->data)->staffmenu;
+   voicemenu =  ((DenemoStaff*)Denemo.gui->si->currentstaff->data)->voicemenu;
+   }
+ }
+   
+  widget_for_directive_menu(directive, fn, staffmenu, voicemenu);
+}
+void
+widget_for_staff_directive(DenemoDirective *directive, GtkMenu *menu) {
+  return widget_for_directive_menu(directive, (void(*)())staff_directive_put_graphic, menu, NULL);
+}
+void
+widget_for_voice_directive(DenemoDirective *directive, GtkMenu *menu) {
+  return widget_for_directive_menu(directive, (void(*)())voice_directive_put_graphic, NULL, menu);
+}
 
+void
+widget_for_movementcontrol_directive(DenemoDirective *directive) {
+  return widget_for_directive_menu(directive, (void(*)())movementcontrol_directive_put_graphic, NULL, NULL);
+}
+void
+widget_for_header_directive(DenemoDirective *directive) {
+  return widget_for_directive_menu(directive, (void(*)())header_directive_put_graphic, NULL, NULL);
+}
+void
+widget_for_layout_directive(DenemoDirective *directive) {
+  return widget_for_directive_menu(directive, (void(*)())layout_directive_put_graphic, NULL, NULL);
+}
 
 // create a directive for non-DenemoObject directive #what
 // assigning the string VALUE to the field ##field
 // also create a button or menuitem ( if it does not already exist) as the directive->widget, this will be used to edit/action the directive
 // Compare this with the macros above which create the what##_directive_put_##field() without calling widget_for_directive() and so do not create a widget in the graphic field, except via the user setting graphic_name.
 
-#define PUT_GRAPHIC_WIDGET_STR(field, what, directives_name) PUT_STR_FIELD_FUNC_NAME(what, field, directives_name)
-#define PUT_GRAPHIC_WIDGET_INT(field, what, directives_name) PUT_INT_FIELD_FUNC_NAME(what, field, directives_name)
+#define PUT_GRAPHIC_WIDGET_STR(field, what, name) \
+gboolean \
+what##_directive_put_##field(gchar *tag, gchar *value) {\
+  what *current = get_##what();\
+  if(current==NULL) return FALSE;\
+  take_snapshot();		 \
+  if(current->name==NULL)\
+       create_directives (&current->name, tag);\
+  DenemoDirective *directive = get_##what##_directive(tag);\
+  if(directive==NULL){\
+    directive=new_directive(tag);\
+    current->name = g_list_append(current->name, directive);\
+    }\
+  if(directive->field)\
+    g_string_assign(directive->field, value);\
+  else\
+    directive->field = g_string_new(value);\
+  widget_for_directive(directive, (void(*)())what##_directive_put_graphic);\
+  g_object_set_data(G_OBJECT(directive->widget), "directives-pointer", (gpointer)&current->name); \
+  return TRUE;\
+}
+
+
+#define PUT_GRAPHIC_WIDGET_INT(field, what, name)\
+gboolean \
+what##_directive_put_##field(gchar *tag, gint value) {\
+  what *current = get_##what();\
+  if(current==NULL) return FALSE;\
+  take_snapshot();		 \
+  if(current->name==NULL)\
+       create_directives (&current->name, tag);\
+  DenemoDirective *directive = get_##what##_directive(tag);\
+  if(directive==NULL){\
+    directive=new_directive(tag);\
+    current->name = g_list_append(current->name, directive);\
+    }\
+  directive->field = value;\
+  widget_for_directive(directive, (void(*)())what##_directive_put_graphic);\
+  g_object_set_data(G_OBJECT(directive->widget), "directives-pointer", &current->name);\
+  return TRUE;\
+}
 
 
 //As the above (for string and int) but for the graphic name field
@@ -1398,7 +1532,8 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
 #define PUT_GRAPHIC_WIDGET_GRAPHIC(what, name) gboolean \
 what##_directive_put_graphic(gchar *tag, gchar *value) {\
   what *current = get_##what();\
-  if(current==NULL) return NULL;\
+  if(current==NULL) return FALSE;\
+  take_snapshot();		\
   if(current->name==NULL)\
        create_directives (&current->name, tag);\
   DenemoDirective *directive = get_##what##_directive(tag);\
@@ -1557,14 +1692,15 @@ standalone_directive_put_graphic(gchar *tag, gchar *value) {
 	object_insert(Denemo.gui, obj);
 	displayhelper(Denemo.gui);
   }
-  if( loadGraphicItem(value, (GdkBitmap **)&directive->graphic)) {
+  if( loadGraphicItem(value, &directive->graphic)) {
     if(directive->graphic_name)
       g_string_assign(directive->graphic_name, value);
     else
       directive->graphic_name = g_string_new(value);
     return TRUE;
   } else  {
-    directive->graphic = directive->graphic_name = NULL;
+    directive->graphic = NULL;
+    directive->graphic_name = NULL;
     return FALSE;
   }
 }
@@ -1575,8 +1711,9 @@ standalone_directive_put_graphic(gchar *tag, gchar *value) {
 gboolean \
 standalone_directive_put_##field(gchar *tag, gchar *value) {\
   DenemoDirective *directive = get_standalone_directive(tag);\
-  if(directive && directive->field)\
-    g_string_assign(directive->field, value);\
+  if(directive && directive->field){\
+    store_for_undo_change (Denemo.gui->si, Denemo.gui->si->currentobject->data);\
+    g_string_assign(directive->field, value);}				\
   else if(directive)\
     directive->field = g_string_new(value);\
   else {\
@@ -1604,8 +1741,9 @@ STANDALONE_PUT_STR_FIELD_FUNC(midibytes);
 gboolean \
 standalone_directive_put_##field(gchar *tag, gint value) {\
   DenemoDirective *directive = get_standalone_directive(tag);\
-  if(directive)\
-    directive->field = value;\
+  if(directive){\
+    store_for_undo_change (Denemo.gui->si, Denemo.gui->si->currentobject->data);\
+    directive->field = value;}\
   else {\
         DenemoObject *obj = lily_directive_new (" ");\
         directive = (DenemoDirective*)obj->object;\
@@ -1633,6 +1771,7 @@ standalone_directive_put_minpixels(gchar *tag, gint value) {
   if(directive){
     directive->minpixels = value;//This field is not actually useful for standalone directives.
     DenemoObject *obj = findobj();
+    store_for_undo_change (Denemo.gui->si, obj);
     obj->minpixelsalloted = value;
   }
   else {
@@ -1725,8 +1864,31 @@ static
 void user_select_directive_at_cursor(gchar **what, GList ***pdirectives, DenemoDirective **pdirective) {
   *pdirectives = NULL;
   *pdirective = get_standalone_directive(NULL);
+
   if(*pdirective)
-    return;
+    return;//FIXME is this needed???? a return will be done anyway
+
+  {
+    gchar *instr = "Select a directive attached to the tuplet marker";
+    tuplet *curtuplet = get_tuplet();
+    if(curtuplet && curtuplet->directives) {
+      *pdirectives = &curtuplet->directives;
+      *what = "tuplet";
+      *pdirective = select_directive(instr, **pdirectives);
+    } 
+
+  }
+  {
+    gchar *instr = "Select a directive attached to the stemdir marker";
+    stemdirective *curstemdir = get_stemdirective();
+    if(curstemdir && curstemdir->directives) {
+      *pdirectives = &curstemdir->directives;
+      *what = "stemdir";
+      *pdirective = select_directive(instr, **pdirectives);
+    } 
+
+  }
+
   gchar *name=NULL;
   note *curnote = get_note();
   if(curnote!=NULL) {
@@ -1852,6 +2014,9 @@ void edit_object(GtkAction *action,  DenemoScriptParam *param) {
 
   case STEMDIRECTIVE:
      {
+       GList *directives =  ((stemdirective *)obj->object)->directives;
+      GtkWidget *menu = gtk_ui_manager_get_widget (Denemo.ui_manager, "/StemControlPopup"); 
+      populate_menu_for_directives(menu, directives);
       popup_menu( "/StemControlPopup");
     }  
      return;
@@ -2129,6 +2294,8 @@ TEXT_EDIT_IF(voice);
 TEXT_EDIT_IF(score);
 TEXT_EDIT_IF(clef);
 TEXT_EDIT_IF(timesig);
+TEXT_EDIT_IF(tuplet);
+TEXT_EDIT_IF(stemdirective);
 TEXT_EDIT_IF(keysig);
 TEXT_EDIT_IF(scoreheader);
 TEXT_EDIT_IF(header);
@@ -2159,28 +2326,6 @@ edit_directive(DenemoDirective *directive, gchar *what) {
 }
 
 
-static void edit_directive_callback(GtkWidget *w, gpointer what) {
-  DenemoDirective *directive = g_object_get_data(G_OBJECT(w), "directive");
-#define DO_TYPE(score)\
-    if(what == score##_directive_put_graphic){\
-      if(!edit_directive(directive, #score))\
-	delete_##score##_directive(directive->tag->str);\
-    }
-
-    DO_TYPE(score);
-    DO_TYPE(staff);
-    DO_TYPE(voice);
-    DO_TYPE(scoreheader);
-    DO_TYPE(header);
-    DO_TYPE(paper);
-    DO_TYPE(layout);
-    DO_TYPE(movementcontrol);
-    DO_TYPE(chord);
-    DO_TYPE(note);
-#undef DO_TYPE 
-}
-
-
 /**
  * callback for EditDirective on directive attached to an object.
  */
@@ -2302,6 +2447,22 @@ DenemoDirective *select_timesig_directive(void) {
   return select_directive("Select a time signature directive", curtimesig->directives);
 }
 
+static
+DenemoDirective *select_tuplet_directive(void) {
+  tuplet *curtuplet = get_tuplet();
+  if(curtuplet==NULL || curtuplet->directives==NULL)
+    return NULL;
+  return select_directive("Select a time signature directive", curtuplet->directives);
+}
+
+static
+DenemoDirective *select_stemdirective_directive(void) {
+  stemdirective *curstemdirective = get_stemdirective();
+  if(curstemdirective==NULL || curstemdirective->directives==NULL)
+    return NULL;
+  return select_directive("Select a time signature directive", curstemdirective->directives);
+}
+
 static
 DenemoDirective *select_staff_directive(void) {
   if(Denemo.gui->si->currentstaff==NULL)
@@ -2403,6 +2564,38 @@ void edit_timesig_directive(GtkAction *action,  DenemoScriptParam *param) {
   score_status (Denemo.gui, TRUE);
 }
 
+/**
+ * callback for EditTupletDirective 
+ */
+void edit_tuplet_directive(GtkAction *action,  DenemoScriptParam *param) {
+  //g_print("Edit directive called\n");
+  DenemoDirective *directive = select_tuplet_directive();
+  //g_print("Got directive %p\n", directive);
+  if(directive==NULL)
+    return;
+  if(directive->tag == NULL)
+    directive->tag = g_string_new(UNKNOWN_TAG);
+  if(!edit_directive(directive, "tuplet"))
+    delete_tuplet_directive(directive->tag->str);
+  score_status (Denemo.gui, TRUE);
+}
+
+/**
+ * callback for EditStemdirectiveDirective 
+ */
+void edit_stemdirective_directive(GtkAction *action,  DenemoScriptParam *param) {
+  //g_print("Edit directive called\n");
+  DenemoDirective *directive = select_stemdirective_directive();
+  //g_print("Got directive %p\n", directive);
+  if(directive==NULL)
+    return;
+  if(directive->tag == NULL)
+    directive->tag = g_string_new(UNKNOWN_TAG);
+  if(!edit_directive(directive, "stemdirective"))
+    delete_stemdirective_directive(directive->tag->str);
+  score_status (Denemo.gui, TRUE);
+}
+
 /**
  * callback for EditScoreDirective 
  */
@@ -2595,6 +2788,56 @@ GET_STR_FIELD_FUNC(timesig, prefix)
 GET_STR_FIELD_FUNC(timesig, postfix)
 GET_STR_FIELD_FUNC(timesig, display)
 
+PUT_INT_FIELD_FUNC(tuplet, x)
+PUT_INT_FIELD_FUNC(tuplet, y)
+PUT_INT_FIELD_FUNC(tuplet, tx)
+PUT_INT_FIELD_FUNC(tuplet, ty)
+PUT_INT_FIELD_FUNC(tuplet, gx)
+PUT_INT_FIELD_FUNC(tuplet, gy)
+PUT_INT_FIELD_FUNC(tuplet, override)
+GET_INT_FIELD_FUNC(tuplet, x)
+GET_INT_FIELD_FUNC(tuplet, y)
+GET_INT_FIELD_FUNC(tuplet, tx)
+GET_INT_FIELD_FUNC(tuplet, ty)
+GET_INT_FIELD_FUNC(tuplet, gx)
+GET_INT_FIELD_FUNC(tuplet, gy)
+GET_INT_FIELD_FUNC(tuplet, override)
+GET_INT_GRAPHIC_FIELD_FUNC(tuplet, width)
+GET_INT_GRAPHIC_FIELD_FUNC(tuplet, height)
+
+PUT_STR_FIELD_FUNC(tuplet, prefix)
+PUT_STR_FIELD_FUNC(tuplet, postfix)
+PUT_STR_FIELD_FUNC(tuplet, display)
+
+GET_STR_FIELD_FUNC(tuplet, prefix)
+GET_STR_FIELD_FUNC(tuplet, postfix)
+GET_STR_FIELD_FUNC(tuplet, display)
+
+PUT_INT_FIELD_FUNC(stemdirective, x)
+PUT_INT_FIELD_FUNC(stemdirective, y)
+PUT_INT_FIELD_FUNC(stemdirective, tx)
+PUT_INT_FIELD_FUNC(stemdirective, ty)
+PUT_INT_FIELD_FUNC(stemdirective, gx)
+PUT_INT_FIELD_FUNC(stemdirective, gy)
+PUT_INT_FIELD_FUNC(stemdirective, override)
+GET_INT_FIELD_FUNC(stemdirective, x)
+GET_INT_FIELD_FUNC(stemdirective, y)
+GET_INT_FIELD_FUNC(stemdirective, tx)
+GET_INT_FIELD_FUNC(stemdirective, ty)
+GET_INT_FIELD_FUNC(stemdirective, gx)
+GET_INT_FIELD_FUNC(stemdirective, gy)
+GET_INT_FIELD_FUNC(stemdirective, override)
+GET_INT_GRAPHIC_FIELD_FUNC(stemdirective, width)
+GET_INT_GRAPHIC_FIELD_FUNC(stemdirective, height)
+
+PUT_STR_FIELD_FUNC(stemdirective, prefix)
+PUT_STR_FIELD_FUNC(stemdirective, postfix)
+PUT_STR_FIELD_FUNC(stemdirective, display)
+
+GET_STR_FIELD_FUNC(stemdirective, prefix)
+GET_STR_FIELD_FUNC(stemdirective, postfix)
+GET_STR_FIELD_FUNC(stemdirective, display)
+
 
 
 
@@ -2722,6 +2965,8 @@ TEXT_EDIT_DIRECTIVE(voice);
 TEXT_EDIT_DIRECTIVE(score);
 TEXT_EDIT_DIRECTIVE(clef);
 TEXT_EDIT_DIRECTIVE(timesig);
+TEXT_EDIT_DIRECTIVE(tuplet);
+TEXT_EDIT_DIRECTIVE(stemdirective);
 TEXT_EDIT_DIRECTIVE(keysig);
 TEXT_EDIT_DIRECTIVE(scoreheader);
 TEXT_EDIT_DIRECTIVE(header);
