@@ -19,6 +19,16 @@ gchar *
 get_scoretitle(void);
 void 
 widget_for_directive(DenemoDirective *directive,  void fn());
+void 
+widget_for_staff_directive(DenemoDirective *directive, GtkMenu *menu);
+void 
+widget_for_voice_directive(DenemoDirective *directive, GtkMenu *menu);
+void
+widget_for_movementcontrol_directive(DenemoDirective *directive);
+void
+widget_for_header_directive(DenemoDirective *directive);
+void
+widget_for_layout_directive(DenemoDirective *directive);
 gboolean 
 text_edit_note_directive(gchar *tag);
 void 
