@@ -17,7 +17,7 @@ static gboolean lyric_keypress(GtkWidget *w, GdkEventKey *event) {
 DenemoGUI *gui = Denemo.gui;
  if(gtk_text_buffer_get_modified(gtk_text_view_get_buffer(GTK_TEXT_VIEW(w)))) {
    score_status(Denemo.gui, TRUE);
-   gtk_widget_queue_draw (gui->scorearea);
+   gtk_widget_queue_draw (Denemo.scorearea);
    gtk_text_buffer_set_modified(gtk_text_view_get_buffer(GTK_TEXT_VIEW(w)), FALSE);
  }
  return FALSE;
@@ -25,7 +25,7 @@ DenemoGUI *gui = Denemo.gui;
 gboolean lyric_change(GtkTextBuffer *buffer) {
   DenemoGUI *gui = Denemo.gui;
   score_status(gui, TRUE);
-  gtk_widget_queue_draw (gui->scorearea);
+  gtk_widget_queue_draw (Denemo.scorearea);
   return FALSE;
 }
 
@@ -45,7 +45,7 @@ newlyric (gint baseduration, gint numdots, gchar *lys) {
 
 static void
 switch_page (GtkNotebook *notebook, GtkNotebookPage *page,  guint pagenum, DenemoStaff *staff) {
-    gtk_widget_queue_draw (Denemo.gui->scorearea);
+    gtk_widget_queue_draw (Denemo.scorearea);
     staff->currentverse = g_list_nth(staff->verses, pagenum);
 }
 GtkWidget * add_verse_to_staff(DenemoScore *si, DenemoStaff *staff) {
@@ -55,8 +55,10 @@ GtkWidget * add_verse_to_staff(DenemoScore *si, DenemoStaff *staff) {
     gtk_widget_show(notebook);
     g_signal_connect (G_OBJECT(notebook), "switch_page", G_CALLBACK(switch_page), staff);
     if(si->lyricsbox==NULL)
-      install_lyrics_preview(si, gtk_widget_get_parent(gtk_widget_get_parent(Denemo.gui->scorearea)));//FIXME we need a proper way of getting to the top vbox, that will not break when scorearea is moved in the widget hierarchy.
+      install_lyrics_preview(si, gtk_widget_get_parent(gtk_widget_get_parent(Denemo.scorearea)));//FIXME we need a proper way of getting to the top vbox, that will not break when scorearea is moved in the widget hierarchy.
     gtk_box_pack_start(GTK_BOX(si->lyricsbox), notebook, TRUE, TRUE, 0);
+    if(si->measurewidth==DENEMO_INITIAL_MEASURE_WIDTH)
+      si->measurewidth = DENEMO_INITIAL_MEASURE_WIDTH*3;
   } else {
     GtkWidget *w = staff->verses->data;
     notebook = gtk_widget_get_parent(w);
@@ -66,7 +68,7 @@ GtkWidget * add_verse_to_staff(DenemoScore *si, DenemoStaff *staff) {
   //if(si->currentstaff && si->currentstaff->data == staff)
   //  gtk_widget_show(staff->currentverse->data);
   textview = new_lyric_editor();
-  gtk_text_view_set_wrap_mode (textview, GTK_WRAP_WORD_CHAR);
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(textview), GTK_WRAP_WORD_CHAR);
   gtk_widget_show(textview);
   staff->verses = g_list_append(staff->verses, textview);
   staff->currentverse = g_list_last(staff->verses);
@@ -102,7 +104,7 @@ DenemoScore *si = gui->si;
      gtk_widget_destroy(staff->currentverse->data);
      staff->currentverse = staff->verses;
      score_status(gui, TRUE);
-     gtk_widget_queue_draw (gui->scorearea);
+     gtk_widget_queue_draw (Denemo.scorearea);
    }
  }
 
@@ -201,3 +203,36 @@ DenemoScore *si = gui->si;
        gtk_widget_hide(gtk_widget_get_parent(((DenemoStaff *)current->data)->verses->data));
  }
 }
+
+gchar *get_lyrics_for_current_verse(DenemoStaff *thestaff) {
+  if(thestaff->currentverse && thestaff->currentverse->data)
+    return get_text_from_view(thestaff->currentverse->data);
+  else
+    return  NULL;
+}
+
+gboolean append_lyrics_for_current_verse(DenemoStaff *thestaff, gchar *text) {
+  if(thestaff->currentverse && thestaff->currentverse->data) {
+    GtkTextIter iter;
+    GtkTextBuffer *textbuffer = gtk_text_view_get_buffer (thestaff->currentverse->data);
+    gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(textbuffer), &iter);  
+    gtk_text_buffer_insert (textbuffer, &iter, text, -1);    
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
+gboolean put_lyrics_for_current_verse(DenemoStaff *thestaff, gchar *text) {
+  if(thestaff->currentverse && thestaff->currentverse->data) {
+    GtkTextBuffer *textbuffer = gtk_text_view_get_buffer (thestaff->currentverse->data);
+    GtkTextIter startiter, enditer; 
+    gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER(textbuffer), &startiter);
+    gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(textbuffer), &enditer);
+    gtk_text_buffer_delete (textbuffer, &startiter, &enditer);   
+    gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(textbuffer), &enditer);  
+    gtk_text_buffer_insert (textbuffer, &enditer, text, -1);    
+    return TRUE;
+  }
+  else
+    return FALSE;
+}
