@@ -415,9 +415,7 @@ void
 debug_handler (const gchar *log_domain, GLogLevelFlags log_level,
                const gchar *message, gpointer user_data)
 {
-#ifdef DEBUG
-  g_print (message);
-#endif
+  //g_debug ("%s",message);
 }
 
 
@@ -438,6 +436,7 @@ main (int argc, char *argv[])
 //#endif
   
    /* set the default handler for debug messages */
+  //FIXME this does not work
   g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, debug_handler, NULL);
 
 
@@ -490,6 +489,90 @@ main (int argc, char *argv[])
   add_font_directory(fontpath);
   fontpath = g_build_filename (prefix, "share", "fonts", "truetype","denemo", "denemo.ttf", NULL);
   add_font_directory(fontpath);
+#else
+
+
+#ifdef __APPLE__
+  //FIXME if this works, remove the duplication with windows case
+ {char apath[1024];
+       guint size = sizeof(apath);
+       _NSGetExecutablePath(apath, &size);
+       gchar * bindir = (gchar*)g_malloc(size);
+       if (_NSGetExecutablePath(bindir, &size) == 0)
+	 g_print("using bin path %s\n", bindir);
+       else
+	 g_critical("Cannot get bin dir\n");
+       gchar *prefix = g_build_filename (g_path_get_dirname(bindir), "..", NULL);      
+       gchar *guile = g_build_filename (prefix, "share", "guile", NULL);
+  gchar *guile_1_8 = g_build_filename (guile, "1.8", NULL);
+  gchar *lilypond_current_scm = g_build_filename (prefix, "share", "lilypond", "current", "scm", NULL);
+  if (g_file_test (guile, G_FILE_TEST_EXISTS))
+    {
+      gchar *guile_path = g_strconcat (guile, ":", guile_1_8, ":", lilypond_current_scm, NULL);
+      g_setenv ("GUILE_LOAD_PATH", guile_path, TRUE);//FIXME TRUE means we overwrite any installed version of lilyponds scm, FALSE risks not putting denemos scm in the path...
+      g_print ("Setting GUILE_LOAD_PATH=%s\n", guile_path);
+    }
+  else
+    warningdialog ("You may need to set GUILE_LOAD_PATH to the directory where you have ice9 installed\n");
+
+  gchar *rc_path = g_build_filename (prefix, "etc","pango", "pangorc", NULL);
+  g_setenv ("PANGO_RC_FILE", rc_path, TRUE);
+  g_setenv ("PANGO_PREFIX", prefix, TRUE);
+  g_setenv ("PANGO_MODULE_VERSION", "1.6.0", TRUE);
+  g_setenv ("PANGO_SO_EXTENSION", ".so", TRUE);
+  g_print ("Setting PANGO_PREFIX=%s\n", prefix);
+#if 1
+  g_setenv ("GTK_MODULE_VERSION", "2.10.0", TRUE);
+  g_setenv ("GTK_SO_EXTENSION", ".so", TRUE);
+  g_setenv ("GTK_PREFIX", prefix, TRUE);
+  g_print ("Setting GTK_PREFIX=%s\n", prefix);
+#endif
+  g_setenv ("GDK_PIXBUF_MODULE_FILE", g_build_filename (prefix, "etc", "gtk-2.0", "gdk-pixbuf.loaders", NULL), TRUE);
+  g_print("Set GDK_PIXBUF_MODULE_FILE to %s\n", g_build_filename (prefix, "etc", "gtk-2.0", "gdk-pixbuf.loaders", NULL));
+  gchar *fc_path = g_build_filename (prefix, "etc","fonts", NULL);
+  g_setenv ("FONTCONFIG_PATH", fc_path, TRUE);
+  g_print ("Setting FONTCONFIG_PATH=%s\n", fc_path);
+  gchar *fc_file = g_build_filename (fc_path, "fonts.conf", NULL);
+  g_setenv ("FONTCONFIG_FILE", fc_file, TRUE);
+  g_print ("Setting FONTCONFIG_FILE=%s\n", fc_file);
+
+
+  gchar *program_files =  g_getenv("PROGRAMFILES");
+  gchar *path = g_getenv ("PATH");
+  gchar *lilypond_path = g_build_filename(prefix, "bin", NULL);
+  gchar *lib_path = g_build_filename(prefix, "lib", NULL);
+  path = g_strconcat (path,":", lilypond_path, ":", lib_path, NULL);
+
+  g_setenv ("PATH", path, TRUE);
+  g_print("PATH set to %s\n", path);
+  gchar *lilypond_data_path = g_build_filename (prefix, "share", "lilypond", "current", NULL);
+  g_setenv ("LILYPOND_DATA_PATH", lilypond_data_path, FALSE);
+  g_print("LILYPOND_DATA_PATH will be %s if not already set", lilypond_data_path);
+  gchar *fontpath = g_build_filename (prefix, "share", "fonts", "truetype","denemo", "fetta.ttf", NULL);
+  g_setenv ("LILYPOND_VERBOSE", "1", FALSE);
+  add_font_directory(fontpath);
+  fontpath = g_build_filename (prefix, "share", "fonts", "truetype","denemo", "denemo.ttf", NULL);
+  add_font_directory(fontpath);
+
+
+ add_font_directory (DATAROOTDIR "/fonts");
+ add_font_directory(g_build_filename (prefix, "share", "fonts", "truetype","denemo", NULL));
+ add_font_directory(g_build_filename (prefix, "share", "fonts", NULL));
+ g_print("\n\nAdded %s to fonts search\n\n",  g_build_filename (prefix, "share", "fonts", NULL));
+
+      }
+#else
+  add_font_directory (DATAROOTDIR "/fonts");
+#endif
+
+
+#ifdef ENABLE_BINRELOC
+  g_debug("\n\n!!!!!!!!!!!!!!!!!\n\nBinreloc is %s\n\n the macro PKGDATADIR = %s, data dir is set to %s\n",  " Enabled", PKGDATADIR, get_data_dir());
+#else
+ g_debug("\n\n!!!!!!!!!!!!!!!!!\n\nBinreloc is %s\n\n the macro PKGDATADIR = %s, data dir is set to %s\n",  " Disabled", PKGDATADIR, get_data_dir());
+#endif
+
+
 #endif
   GError *error = NULL;
   /* gtk initialization */
@@ -506,57 +589,20 @@ main (int argc, char *argv[])
 
   register_stock_items ();
 
-#define choice1 "Simple\nQuick start users: use this until you have read the manual\n"
-#define choice2 "Arranger\nExperienced Users: transcribing music, playing music in, transposing etc"
-#define choice3 "Composer\nExperienced Users: entering and modifying music, working with selections WASD use etc"
-#define choice4 "Classic\nOld Denemo pc-keyboard interface."
-#define choice5 "LilyPond\nExperienced Users with LilyPond knowledge"
-#define choice6 "AllCommands\nUsers wanting to see the complete command set. No pre-defined shortcuts"
 
-  if(uses_default_commandset()) {
-    // infodialog("Nearly every menu item can be right-clicked, for help, setting keyboard shortcuts and more"); this  should always appear on top of the main window, but it is unresponsive to dismissal at first.
-    // get_option returns a pointer into the string passed in
-    gchar *choice = get_option(choice1"\0"choice2"\0"choice3"\0"choice4"\0"choice5"\0"choice6"\0", strlen(choice1)+1+strlen(choice2)+1+strlen(choice3)+1+strlen(choice4)+1+strlen(choice5)+1+strlen(choice6)+1);
-    if(choice==NULL)
-      choice = choice1;
-    if(strcmp(choice, choice1)) {
-      choice = g_strdup(choice);
-      gchar *c;
-      for(c=choice;*c;c++)
-	if(*c=='\n')
-	  *c='\0';
-      choice =  g_strconcat(choice, ".commands", NULL);
-      choice =  g_build_filename(get_data_dir(), "actions", choice, NULL);
-      //g_print("Choice is %s length %d\n", choice, strlen(choice));
-      Denemo.prefs.shortcut_filename = g_string_new(choice);
-    }
-  }
   //g_print("Calling scm boot guile with %d and %p\n", argc, argv);
   scm_boot_guile (argc, argv, inner_main, NULL);
   
   return 0;
 }
 
-
-static int
-openfile (gchar * name, gboolean template)
-{
-  GList *tmp = g_list_nth (Denemo.guis, 0);
-
-  DenemoGUI *gui = (DenemoGUI *) tmp->data;
-  gint result = open_for_real (name, gui, template, REPLACE_SCORE);
-  //gui->si->readonly = FALSE;
-  //si->readonly = TRUE;
-  return result;
-}
-
 gchar * process_command_line(int argc, char**argv) {
 
   gint opts;
   GDir *dir=NULL;
   gchar *filename;
   GError *error = NULL;
-  gchar *initschemefile=NULL,  *commandsetfile=NULL;
+  gchar *commandsetfile=NULL;
   /* parse command line and display help messages */
   gchar *helptext  = g_strconcat (_("\nGNU Denemo version "), VERSION, ".\n\n",
                                  _("\
@@ -572,9 +618,9 @@ Options:\n\
   -i pathtofile         process scheme commands in pathtofile on startup\n\
   -s filename           process scheme commands from system file\n\
   -v,--version          print version number and exit\n\n\n\
-Report bugs to bug-denemo@gnu.org\n"), NULL) ;
+Report bugs to http://www.denemo.org\n"), NULL) ;
 
-  gchar *copytext = _("(c) 1999-2005, 2009 Matthew Hiller, Adam Tee, and others\n\n\n"
+  gchar *copytext = _("(c) 1999-2005, 2009 Matthew Hiller, Adam Tee, and others, 2010-2011 Richard Shann, Jeremiah Benham, Nils Gey and others.\n\n\n"
 "This program is provided with absolutely NO WARRANTY; see\n"
 "the file COPYING for details.\n\n"
 "This software may be redistributed and modified under the\n"
@@ -585,7 +631,8 @@ Report bugs to bug-denemo@gnu.org\n"), NULL) ;
 #ifdef HAVE_GETOPT_H
   static struct option long_options[] = {
     {"help", no_argument, NULL, 'h'},
-    {"version", no_argument, NULL, 'v'}
+    {"version", no_argument, NULL, 'v'},
+    {NULL, NULL, NULL}
   };
 #endif
 
@@ -603,11 +650,11 @@ Report bugs to bug-denemo@gnu.org\n"), NULL) ;
         }
       else if (opts == 's')
         {
-	  initschemefile = g_build_filename(get_data_dir(), "actions",  optarg, NULL);
+	  Denemo.schemeinit = g_build_filename(get_data_dir(), "actions",  optarg, NULL);
         }
       else if (opts == 'i')
         {
-          initschemefile = g_strdup(optarg);
+          Denemo.schemeinit = g_strdup(optarg);
         }
 
       else if (opts == 'c')
@@ -627,41 +674,12 @@ Report bugs to bug-denemo@gnu.org\n"), NULL) ;
         }
     }
 
-  g_print (_("\nGNU Denemo, a gtk+ frontend for GNU Lilypond\n"));
+  g_print (_("\nGNU Denemo, a free and open music notation editor\n"));
   g_print ("%s", copytext);
 
   g_free (helptext);
 
 
-  denemo_scheme_init(initschemefile);
-
-#ifdef _HAVE_JACK_
-if (Denemo.prefs.midi_audio_output == Jack)
-  init_jack();
-#endif
-  /* audio initialization */
-  //ext_init (); 
-  /* external players (midi...) */
-#ifdef _HAVE_FLUIDSYNTH_
-if (Denemo.prefs.midi_audio_output == Fluidsynth)
-  fluidsynth_init(); 
-#endif
-#ifdef _HAVE_PORTAUDIO_
-if (Denemo.prefs.midi_audio_output == Portaudio){
-  /* Immediate Playback */
-  if(Denemo.prefs.immediateplayback) {
-    if( midi_init ()  )  {           /* Opens Denemo.prefs.sequencer, if this is set to an empty
-				 string then the open fails and direct audio out is used for 
-				immediate playback */
-      //g_print("Initializing audio out\n");
-      init_audio_out();
-    }
-  }
-}
-#endif    
-
-
-
   /* Set up the signal handler */
   signal (SIGSEGV, denemo_signal_handler);
 #ifdef HAVE_SIGCHLD
@@ -669,81 +687,6 @@ if (Denemo.prefs.midi_audio_output == Portaudio){
 #endif
 
 
-  if(locatedotdenemo ()) {
-    dir = g_dir_open (locatedotdenemo (), 0, &error);
-    if (error)
-      {
-	g_print ("Cannot read .denemo directory %s\n", error->message);
-	g_error_free (error);
-      }
-  }
-#if 0
-  while (dir && (filename = (gchar *) g_dir_read_name (dir)) != NULL)
-    {
-      if (0 == strcmp ("crashrecovery.denemo", filename))
-        {
-          GtkWidget *dialog = 
-            gtk_dialog_new_with_buttons (NULL,
-                                         NULL,
-                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_STOCK_YES,
-                                         GTK_RESPONSE_ACCEPT,
-                                         GTK_STOCK_SAVE_AS,
-                                         GTK_RESPONSE_CANCEL,
-                                         GTK_STOCK_DELETE,
-                                         GTK_RESPONSE_REJECT,
-                                         NULL);
-          GtkWidget *label =
-            gtk_label_new
-            ("\nDenemo crashed, The open file has been recovered\n"
-             "do you want to continue editing your work?\n");
-          gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-                             label);
-          gtk_widget_show_all (dialog);
-          gint result = gtk_dialog_run (GTK_DIALOG (dialog));
-          g_debug ("Dialog result is %d\n", result);
-          gchar *name = g_build_filename(locatedotdenemo (),
-                                         filename, NULL);
-          switch (result)
-            {
-            case GTK_RESPONSE_ACCEPT:
-              openfile (name, FALSE);
-              g_remove (name);
-              break;
-            case GTK_RESPONSE_CANCEL:
-              Denemo.window = NULL;
-              result = importXML (name, Denemo.gui, REPLACE_SCORE);
-              if (result != -1)
-                file_saveas (Denemo.gui, FALSE);
-              else
-                g_print ("Cannot open %s\n", name);
-              //g_free (gui);
-              g_remove (name);
-              break;
-            case GTK_RESPONSE_REJECT:
-              g_print ("Removing %s\n", name);
-              g_remove (name);
-              //g_free(name);
-              break;
-            }
-          gtk_widget_destroy (dialog);
-        }
-    }
-#endif
-
-  if (dir)
-    g_dir_close (dir);
-
-  if(commandsetfile==NULL)
-    if(Denemo.prefs.shortcut_filename!=NULL && (Denemo.prefs.shortcut_filename->len))
-      commandsetfile = Denemo.prefs.shortcut_filename->str;
-
-  if(commandsetfile && (0==load_xml_keymap(commandsetfile, TRUE)))
-    ;
-  else
-    load_default_keymap_file();
-
-
     if (optind < argc)
       return argv[optind];
     else
