@@ -41,15 +41,10 @@ addmeasures (DenemoScore * si, gint pos, guint nummeasures, gint all)
     {
       if (all)
 	{
-	  for (curstaff = si->thescore; curstaff; curstaff = curstaff->next)
+	  gint j;
+	  for (j=1, curstaff = si->thescore; curstaff; j++, curstaff = curstaff->next)
 	    {
-	      if (((DenemoStaff *) curstaff->data)->is_parasite)
-		{
-		  ((DenemoStaff *) curstaff->data)->measures =
-		    *(((DenemoStaff *) curstaff->data)->is_parasite);
-		  continue;
-		  /*assumes parasite staff always come before their hosts */
-		}
+	      store_for_undo_measure_insert(si, j, pos);
 	      ((DenemoStaff *) curstaff->data)->measures =
 		g_list_insert (firstmeasurenode (curstaff), barlinenode, pos);
 	      ((DenemoStaff *) curstaff->data)->nummeasures++;
@@ -57,11 +52,11 @@ addmeasures (DenemoScore * si, gint pos, guint nummeasures, gint all)
 	}
       else
 	{
+	  store_for_undo_measure_insert(si, si->currentstaffnum, pos);
 	  ((DenemoStaff *) si->currentstaff->data)->measures =
 	    g_list_insert (firstmeasurenode (si->currentstaff),
 			   barlinenode, pos);
 	  ((DenemoStaff *) si->currentstaff->data)->nummeasures++;
-
 	}
 
       gint maxmeasures=0;
@@ -133,6 +128,7 @@ void
 staffremovemeasures (staffnode * curstaff, guint pos)
 {
   //g_print ("In Staffremovemeasures\n");
+  take_snapshot();
   measurenode *firstmeasure;
   measurenode *delmeasure;
   if (((DenemoStaff *) curstaff->data)->is_parasite)
@@ -143,10 +139,10 @@ staffremovemeasures (staffnode * curstaff, guint pos)
   firstmeasure = firstmeasurenode (curstaff);
   delmeasure = g_list_nth (firstmeasure, pos);
   if(delmeasure) {
-#ifdef DEBUG
-    g_print ("Firstmeasure %x\t DelMeasure %x \t Position\n",
-	     firstmeasure, delmeasure, pos);
-#endif
+     
+    //  g_debug ("Firstmeasure %x\t DelMeasure %x \t Position\n",
+    //	     firstmeasure, delmeasure, pos);
+
     
     freeobjlist (delmeasure->data, NULL);
     ((DenemoStaff *) curstaff->data)->measures =
@@ -277,7 +273,7 @@ static gint is_end_grace(objnode *curobjnode) {
  * that's precisely where it is invoked 
  */
 static void
-settickvalsinmeasure (objnode * theobjs, gint ticksperbeat)
+settickvalsinmeasure (objnode * theobjs)
 {
   gint numerator = 1, denominator = 1;
   objnode *curobjnode;
@@ -379,7 +375,7 @@ setsdir (objnode * starter, objnode * ender, gint beamgroup_sum,
   gboolean is_stemup = TRUE;
   gint stemoffset;
   gint stemy;
-#ifdef DEBUG
+#if 0
   {static gint count = 0;
   count++;
   g_print("Call %d ++++++++++++++++++++++++++++++++\n\
@@ -485,7 +481,7 @@ calculatebeamsandstemdirs (objnode * theobjs, gint * pclef, gint * time1,
     }
 
   ticksperbeat = calcticksperbeat (*time1, *time2);
-  settickvalsinmeasure (theobjs, ticksperbeat);
+  settickvalsinmeasure (theobjs);
   beatendsat = ticksperbeat;
 
   for (curobjnode = theobjs; curobjnode; prevobj = theobj, curobjnode = curobjnode->next)
@@ -595,7 +591,7 @@ calculatebeamsandstemdirs (objnode * theobjs, gint * pclef, gint * time1,
 
 
 
-#define ACCS_TOO_CLOSE 5
+#define ACCS_TOO_CLOSE 10
 /**
  * This function offsets accidentals that are near to each
  * other on the same chord.  
@@ -611,6 +607,9 @@ set_accidental_positions (DenemoObject * the_chord)
   gint i;
   chord chordval = *(chord *) the_chord->object;
   gint baseduration = chordval.baseduration;
+  if(g_list_length(chordval.notes)>ACCS_TOO_CLOSE)
+    return;
+
   baseduration = MAX(baseduration, 0);
   gint additional_space
     = ((!chordval.is_stemup && chordval.is_reversealigned)
