@@ -37,6 +37,7 @@
 #include "audio.h"
 #include "jackmidi.h"
 #include "instrumentname.h"
+#include "view.h"
 
 static double
 midi2hz(int midinum)
@@ -174,7 +175,7 @@ playnote (note * tone, /*GList * chord, */int channel)
   key += ((note *) tone)->enshift;
   //voice = g_list_index ((GList *) chord, tone);
 
-  play_midikey(key, 0.3, 0.5/*Denemo.prefs.pcmvolume*/, channel);
+  play_midikey(key, 0.3, 1.0/*Denemo.prefs.pcmvolume*/, channel);
 
 }
 
@@ -198,24 +199,6 @@ playnotes (gboolean doit, chord *chord_to_play, int channel)
 #include <glib.h>
 static  GIOChannel* channel;/* a channel to access /dev/midi by */
 
-#define MAX_MIDI_NOTES (20)
-static gint midi_notes[MAX_MIDI_NOTES];
-static volatile int count;
-
-static void store_midi_note(gint pitch)
-{ 
-  if( ++count < MAX_MIDI_NOTES)
-    midi_notes[count] = pitch;
-}
-
-gint get_midi_note(void) {
-  gint ret=-1;
-  if(count) {
-    ret=midi_notes[count];
-    count = 0;
-  }
-  return ret;
-}
 
 void safely_add_track(smf_t *smf, smf_track_t *track) {
   if(track->smf==NULL)
@@ -227,11 +210,14 @@ void safely_track_remove_from_smf(smf_track_t *track) {
    smf_track_remove_from_smf(track);
 }
 /**
- * enter_midi_note_in_score
- * @mid_c_offset enters the midi note steps above/below mid-c
+ * action_note_into_score
+  enters ( or (if mode==INPUTEDIT and appending) edits the note at the cursor)
+ * the parameters specify which note
+ * @mid_c_offset
  * @enshift enharmonic adjustment -1 is one flat etc.. 
+ * @octave
  */
-static void enter_midi_note_in_score (DenemoGUI *gui, gint mid_c_offset, gint enshift, gint octave) {
+static void action_note_into_score (DenemoGUI *gui, gint mid_c_offset, gint enshift, gint octave) {
   gui->last_source = INPUTMIDI;
   gui->si->cursor_y = gui->si->staffletter_y = mid_c_offset;
   gui->si->cursor_y += 7*octave; 
@@ -239,11 +225,20 @@ static void enter_midi_note_in_score (DenemoGUI *gui, gint mid_c_offset, gint en
   setenshift(gui->si, enshift);
   displayhelper (gui);
 }
-
+static void add_note_to_chord (DenemoGUI *gui, gint mid_c_offset, gint enshift, gint octave) {
+  gui->last_source = INPUTMIDI;
+  gui->si->cursor_y = gui->si->staffletter_y = mid_c_offset;
+  gui->si->cursor_y += 7*octave; 
+  insert_chordnote(gui);
+  // shiftcursor(gui, mid_c_offset);
+  setenshift(gui->si, enshift);
+  displayhelper (gui);
+}
 typedef struct enharmonic
 {
   gint mid_c_offset;
   gint enshift;
+  gint octave;
 }enharmonic;
 
 
@@ -260,30 +255,50 @@ void record_midi(gchar *buf, gdouble time) {
   }
 }
 
-/* look for a new note played into midi input, if
-   present insert it into the score/edit the score following the mode */
-gint midientry(void) {
+static void 	      
+do_one_note(DenemoGUI *gui, gint mid_c_offset, gint enshift, gint notenum) {
   
+  if((Denemo.keyboard_state&ADDING_MASK) && (Denemo.keyboard_state&CHORD_MASK)) {
+    
+    add_note_to_chord(gui, mid_c_offset, enshift, notenum);
+  }
+  else {
+    action_note_into_score(gui, mid_c_offset, enshift, notenum);
+    if(Denemo.keyboard_state&ADDING_MASK)
+      Denemo.keyboard_state |= CHORD_MASK;
+    set_midi_in_status();
+  }
+}
+
+/*  take an action for the passed note. Enter/edit/check the score following the mode and keyboard state. */
+static gint midiaction(gint notenum) {
+
   DenemoGUI *gui = Denemo.gui;
   if(gui==NULL)
     return TRUE;
   if(gui->si==NULL)
+
     return TRUE;
-  gint notenum;
+  //gint notenum;
   DenemoStaff *curstaffstruct = (DenemoStaff *) gui->si->currentstaff->data;
-  notenum = get_midi_note();
-  if(notenum < 0) 
-    return TRUE;
+  // notenum = get_midi_note();
+  // if(notenum < 0) 
+  //  return TRUE;
   enharmonic enote;
-  notenum2enharmonic (notenum, &enote.mid_c_offset, &enote.enshift);
-  if (Denemo.prefs.midi_audio_output == Portaudio)
-    playpitch(midi2hz(notenum), 0.3, 0.5, 0);
-  if (Denemo.prefs.midi_audio_output == Jack)
-    jack_playpitch(notenum, 300 /*duration*/);
-  else if (Denemo.prefs.midi_audio_output == Fluidsynth)
-    fluid_playpitch(notenum, 300 /*duration*/,  curstaffstruct->midi_channel, 0);
- 
-  if(gui->mode & INPUTEDIT)
+  //g_print("Keyboard state %x, mask %x %x %x\n", Denemo.keyboard_state, CHECKING_MASK, GDK_CONTROL_MASK, GDK_MOD2_MASK);
+  notenum2enharmonic (notenum, &enote.mid_c_offset, &enote.enshift, &enote.octave);
+  if( !(Denemo.keyboard_state&CHECKING_MASK)) {
+    if(Denemo.prefs.immediateplayback) {
+      if (Denemo.prefs.midi_audio_output == Portaudio)
+	playpitch(midi2hz(notenum), 0.3, 0.5, 0);
+      if (Denemo.prefs.midi_audio_output == Jack)
+	jack_playpitch(notenum, 300 /*duration*/);
+      else if (Denemo.prefs.midi_audio_output == Fluidsynth)
+	fluid_playpitch(notenum, 300 /*duration*/,  curstaffstruct->midi_channel, 0);
+    }
+    stage_undo(gui->si, ACTION_STAGE_END);//undo is a queue so this is the end :)
+  }
+  if((gui->mode & INPUTEDIT) || (Denemo.keyboard_state&CHECKING_MASK))
     {
       static gboolean beep = FALSE;
       gboolean is_tied = FALSE;
@@ -295,36 +310,62 @@ gint midientry(void) {
 	    curObj = Denemo.gui->si->currentobject->data;
 	    chord *thechord = (chord *)  curObj->object;
 	    is_tied = thechord->is_tied;
-	    enter_midi_note_in_score(gui, enote.mid_c_offset, enote.enshift, notenum/12 - 5);
+	    
+#define check_midi_note(a,b,c,d) ((a->mid_c_offset==b)&&(a->enshift==c))?playnote(a,curstaffstruct->midi_channel):gdk_beep();
+
+	    //g_print("check %d %d %d %d %d\n", a->mid_c_offset, a->enshift, b, c, d);
+	    if( (Denemo.keyboard_state&CHECKING_MASK) && thechord->notes) {
+	      //later - find note nearest cursor and
+	      note *thenote = (note*)thechord->notes->data;
+	      check_midi_note(thenote, enote.mid_c_offset + 7 *(enote.octave), enote.enshift, enote.octave);
+	      if((!curObj->isinvisible)&&(thenote->mid_c_offset== (enote.mid_c_offset + 7 *( enote.octave)))&&(thenote->enshift==enote.enshift))
+		playnote(thenote,curstaffstruct->midi_channel);
+	      else {
+		gdk_beep();
+		break;//do not move on to next note
+	      }
+	    }
+	    else {
+
+	      do_one_note(gui, enote.mid_c_offset, enote.enshift, enote.octave);
+		
+	    }
 	    if(Denemo.gui->si->cursor_appending)
 	      break;
-	  } while(next_editable_note() && is_tied);
-	} else 
-	  gdk_beep();
+	  } while((!(Denemo.keyboard_state&ADDING_MASK)) && next_editable_note() && is_tied);
+	} else {
+	  if(gui->si->cursor_appending)
+	    do_one_note(gui, enote.mid_c_offset, enote.enshift, enote.octave);
+	  else
+	    gdk_beep();
+	}
 	if(gui->mode & INPUTRHYTHM) {
-	  if(measure != gui->si->currentmeasurenum)
+	  //g_print("measure was %d now %d with appending %d\n", measure, gui->si->currentmeasurenum, gui->si->cursor_appending);
+	  if(!beep && (measure != gui->si->currentmeasurenum) && !gui->si->cursor_appending)
 	    beep=TRUE;
 	  else if(beep) signal_measure_end(), beep=FALSE;
 	}
-      } else
-	enter_midi_note_in_score(gui, enote.mid_c_offset, enote.enshift, notenum/12 - 5);
-    } else
-    enter_midi_note_in_score(gui, enote.mid_c_offset, enote.enshift, notenum/12 - 5);
+      } else {// no current object
+	//	if((Denemo.keyboard_state&ADDING_MASK))
+	// add_note_to_chord(gui, enote.mid_c_offset, enote.enshift, notenum/12 - 5);
+	  //else
+	  //action_note_into_score(gui, enote.mid_c_offset, enote.enshift, notenum/12 - 5);
+	  do_one_note(gui, enote.mid_c_offset, enote.enshift, enote.octave);
+      }
+    } else {// not INPUTEDIT    
+      action_note_into_score(gui, enote.mid_c_offset, enote.enshift, enote.octave);
+  }
+  if( !(Denemo.keyboard_state&CHECKING_MASK)) {
+    stage_undo(gui->si, ACTION_STAGE_START);
+  }
+  gtk_widget_queue_draw (Denemo.scorearea);//just for advancing the cursor.
   return TRUE;
 }
 
-static guint midi_timer;// timer id
-#define DEFAULT_TIMER_RATE (5)
+
 
 void start_midi_input(void) {
-  
-  if(midi_timer)
-    g_source_remove(midi_timer);
 
-  midi_timer = g_timeout_add (DEFAULT_TIMER_RATE, (GSourceFunc)midientry, NULL);
-
-  if(midi_timer==0)
-    g_error("Timer id 0 - if valid the code needs re-writing (documentation not clear)");
 }
 
 
@@ -353,15 +394,29 @@ void process_midi_event(gchar *buf) {
     // this is only good for one endianness - FIXME
     *divert_midi_event = 0;//clear 4th byte
     memcpy(divert_midi_event, buf, 3);//midi events are up to three bytes long
-    gtk_main_quit();
-    return;// not reached
+    gtk_main_quit();    
+    return;//this *is* reached
   }
-
+#if 0
+  //already done upstream
   if(command==MIDI_NOTEON && velocity==0) {//Zero velocity NOTEON is used as NOTEOFF by some MIDI controllers
     buf[0]=MIDI_NOTEOFF;
     buf[2]=128;//FIXME 127
   }
-  
+#endif  
+   if (command==MIDI_CTL_CHANGE && (notenumber == 0x40)){
+	if (velocity == 0x7F)
+	  //PEDAL DOWN
+	  Denemo.keyboard_state |= ADDING_MASK;
+	else {
+	  Denemo.keyboard_state &= ~(CHORD_MASK|ADDING_MASK);
+	  next_editable_note();
+	}
+	set_midi_in_status();
+	displayhelper(Denemo.gui);
+      }
+
+
   if(midi_capture_on) {
     if(command!=MIDI_NOTEOFF) {
       gdk_beep();
@@ -369,7 +424,7 @@ void process_midi_event(gchar *buf) {
     }
   } else {
     if(command==MIDI_NOTEON)
-      store_midi_note(notenumber);
+      midiaction(notenumber);
     else if(command==MIDI_CTL_CHANGE) {
       gchar *command_name = get_midi_control_command(notenumber, velocity);
       if(command_name)
@@ -377,7 +432,7 @@ void process_midi_event(gchar *buf) {
 	execute_callback_from_name(Denemo.map, command_name);
 
     } else if(command==MIDI_PITCH_BEND) {
-      gchar *command_name = get_midi_pitch_bend_command((notenumber<<7) + velocity);
+      gchar *command_name = get_midi_pitch_bend_command((notenumber<<8) + velocity);
       if(command_name)
       //FIXME what sort of free? g_free(command_name);
 	execute_callback_from_name(Denemo.map, command_name);
@@ -524,36 +579,54 @@ return event->time_seconds;
 
 
 DenemoObject *get_obj_for_start_time(smf_t *smf, gdouble time) {
-  smf_event_t *event;
-  smf_event_t *initial = smf_peek_next_event(smf);
   if(time<0.0)
-    return NULL;
-  gdouble total = smf_get_length_seconds(smf);
-  time = (time>total?total:time);
-  smf_seek_to_seconds(smf, time);
-  do {
-    event = smf_get_next_event(smf);
-  } while(event && (!(event->midi_buffer[0] & MIDI_NOTEON) || !event->user_pointer));
-  if(initial)
-    smf_seek_to_event(smf, initial);
+      time=0.0;
+  static smf_event_t *event;
+  static guint smfsync;
+  static DenemoScore *last_si = NULL;
+  static gdouble last_time=-1.0;
+  if( fabs(time-last_time)>0.001 || (last_si!=Denemo.gui->si) || (smfsync!=Denemo.gui->si->smfsync)) {
+    smf_event_t *initial = smf_peek_next_event(smf);
+
+    gdouble total = smf_get_length_seconds(smf);
+    time = (time>total?total:time);
+    gint error = smf_seek_to_seconds(smf, time);
+    do {
+      event = smf_get_next_event(smf);
+    } while(event && (!(event->midi_buffer[0] & MIDI_NOTEON) || !event->user_pointer));
+    if(initial)
+      error = smf_seek_to_event(smf, initial);
+    last_si = Denemo.gui->si;
+    smfsync = Denemo.gui->si->smfsync;
+    last_time = time;
+  }
   if(event)
     return (DenemoObject *)(event->user_pointer);
   return NULL;
 }
 
 DenemoObject *get_obj_for_end_time(smf_t *smf, gdouble time) {
-  smf_event_t *event;
-  smf_event_t *initial = smf_peek_next_event(smf);
   if(time<0.0)
-    return NULL;
-  gdouble total = smf_get_length_seconds(smf);
-  time = (time>total?total:time);
-  smf_seek_to_seconds(smf, time);
-  do {
-    event = smf_get_next_event(smf);
-  } while(event && (!(event->midi_buffer[0] & MIDI_NOTEOFF) || !event->user_pointer));
-  if(initial)
-    smf_seek_to_event(smf, initial);
+      time=0.0;
+  static smf_event_t *event = NULL;
+  static guint smfsync;
+  static DenemoScore * last_si = NULL;
+  static gdouble last_time=-1.0;
+  if( fabs(time-last_time)>0.001 || (last_si!=Denemo.gui->si) || (smfsync!=Denemo.gui->si->smfsync)) {
+    smf_event_t *initial = smf_peek_next_event(smf);
+
+    gdouble total = smf_get_length_seconds(smf);
+    time = (time>total?total:time);
+    gint error = smf_seek_to_seconds(smf, time);
+    do {
+      event = smf_get_next_event(smf);
+    } while(event && (!(event->midi_buffer[0] & MIDI_NOTEOFF) || !event->user_pointer));
+    if(initial)
+      error = smf_seek_to_event(smf, initial);
+    last_si = Denemo.gui->si;
+    smfsync = Denemo.gui->si->smfsync;
+    last_time = time;
+  }
   if(event)
     return (DenemoObject *)(event->user_pointer);
   return NULL;
