@@ -14,7 +14,7 @@
 #include "lyric.h"
 #include "moveviewport.h"
 #include "mousing.h"
-
+#include "fluid.h"
 
 static gboolean lh_down;
 
@@ -355,7 +355,7 @@ static gboolean change_staff(DenemoScore *si, gint num, GList *staff) {
 static void
 transform_coords(double* x, double* y) {
   DenemoGUI *gui = Denemo.gui;
-  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+  gint line_height = Denemo.scorearea->allocation.height*gui->si->system_height;
   gint line_num = ((int)*y)/line_height;
   *y -= line_num * line_height;
   *x /= gui->si->zoom;
@@ -378,11 +378,12 @@ gint
 scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
 {
   DenemoGUI *gui = Denemo.gui;
-  if(gui==NULL)
+  if(gui==NULL || gui->si==NULL)
     return FALSE;
-  if(gui->scorearea==NULL)
+  if(Denemo.scorearea==NULL)
     return FALSE;
-  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+
+  gint line_height = Denemo.scorearea->allocation.height*gui->si->system_height;
   if (event->y < 0)
     event->y = 0.0;
   gint line_num = ((int)event->y)/line_height;
@@ -391,13 +392,13 @@ scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
 
 
   if(dragging_separator) {
-    gui->si->system_height =  event->y/gui->scorearea->allocation.height;
+    gui->si->system_height =  event->y/Denemo.scorearea->allocation.height;
     if(gui->si->system_height<DENEMO_MINIMUM_SYSTEM_HEIGHT)
       gui->si->system_height = DENEMO_MINIMUM_SYSTEM_HEIGHT;
     if(gui->si->system_height>1.0)
       gui->si->system_height = 1.0;
-    scorearea_configure_event(gui->scorearea, NULL);
-    gtk_widget_queue_draw (gui->scorearea);
+    scorearea_configure_event(Denemo.scorearea, NULL);
+    gtk_widget_queue_draw (Denemo.scorearea);
     return TRUE;
   }
 
@@ -425,11 +426,14 @@ scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
   }
 
     if (lh_down || (selecting && gui->si->markstaffnum)){
-      struct placement_info pi; 
+      struct placement_info pi;
+      pi.the_staff=NULL; 
       if (event->y < 0)
 	get_placement_from_coordinates (&pi, event->x, 0, gui->lefts[line_num],gui->rights[line_num],gui->scales[line_num]);
       else
 	get_placement_from_coordinates (&pi, event->x, event->y, gui->lefts[line_num],gui->rights[line_num],gui->scales[line_num]);
+      if(pi.the_staff==NULL)
+	return TRUE;//could not place the cursor
       if (pi.the_measure != NULL){ /*don't place cursor in a place that is not there*/
 	change_staff(gui->si, pi.staff_number, pi.the_staff);
 	gui->si->currentmeasurenum = pi.measure_number;
@@ -451,9 +455,14 @@ scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
 	   perform_command(event->state, GESTURE_MOVE, event->state&GDK_BUTTON1_MASK);
 
 	/* redraw to show new cursor position  */
-	gtk_widget_queue_draw (gui->scorearea);
+	gtk_widget_queue_draw (Denemo.scorearea);
       }
     }
+
+  if(Denemo.gui->midi_destination & MIDICONDUCT) {
+    advance_time(0.01);
+    return TRUE;
+  }
   return TRUE;
 }
 
@@ -466,8 +475,10 @@ gint
 scorearea_button_press (GtkWidget * widget, GdkEventButton * event)
 {
   DenemoGUI *gui = Denemo.gui;
+  if(gui==NULL || gui->si==NULL)
+    return FALSE;
   //if the cursor is at a system separator start dragging it
-  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+  gint line_height = Denemo.scorearea->allocation.height*gui->si->system_height;
   gint line_num = ((int)event->y)/line_height;
   //g_print("diff %d\n", line_height - ((int)event->y)%line_height);
   if(dragging_separator == FALSE)
@@ -480,27 +491,51 @@ scorearea_button_press (GtkWidget * widget, GdkEventButton * event)
   transform_coords(&event->x, &event->y);
   //g_print("after %f %f\n", event->x, event->y);
 
-  struct placement_info pi;
   gboolean left = (event->button != 3);
   gtk_widget_grab_focus(widget);
   gint key = gui->si->maxkeywidth;
   gint cmajor = key?0:5;//allow some area for keysig in C-major
   if(left && (gui->si->leftmeasurenum>1) && (event->x<KEY_MARGIN+SPACE_FOR_TIME+key)  && (event->x>LEFT_MARGIN)){
     set_currentmeasurenum (gui, gui->si->leftmeasurenum-1);
-    gtk_widget_queue_draw (gui->scorearea);
+    gtk_widget_queue_draw (Denemo.scorearea);
     return TRUE;
   } 
 
   if(gui->lefts[line_num] == 0)
     return TRUE;//On an empty system at the bottom where there is not enough room to draw another staff.
 
-
+  struct placement_info pi;
+  pi.the_staff=NULL;
   if (event->y < 0)
     get_placement_from_coordinates (&pi, event->x, 0, gui->lefts[line_num],gui->rights[line_num],gui->scales[line_num]);
   else
     get_placement_from_coordinates (&pi, event->x, event->y, gui->lefts[line_num],gui->rights[line_num],gui->scales[line_num]);
+  if(pi.the_staff==NULL)
+    return TRUE;//could not place the cursor
   change_staff(gui->si, pi.staff_number, pi.the_staff);
 
+
+
+
+  if (pi.the_measure != NULL){ /*don't place cursor in a place that is not there*/
+    //gui->si->currentstaffnum = pi.staff_number;
+    //gui->si->currentstaff = pi.the_staff;
+    gui->si->currentmeasurenum = pi.measure_number;
+    gui->si->currentmeasure = pi.the_measure;
+    gui->si->currentobject = pi.the_obj;
+    gui->si->cursor_x = pi.cursor_x;
+    gui->si->cursor_appending
+      =
+      (gui->si->cursor_x ==
+       (gint) (g_list_length ((objnode *) gui->si->currentmeasure->data)));
+    set_cursor_y_from_click (gui, event->y);
+    if(pi.nextmeasure)
+      movetomeasureright(NULL); 
+    write_status(gui);
+  }
+
+
+
   if(event->x<LEFT_MARGIN) {
     gint offset = (gint)get_click_height(gui, event->y);
     if(offset<STAFF_HEIGHT/2) {
@@ -525,26 +560,6 @@ scorearea_button_press (GtkWidget * widget, GdkEventButton * event)
   }
 
 
-  if (pi.the_measure != NULL){ /*don't place cursor in a place that is not there*/
-    //gui->si->currentstaffnum = pi.staff_number;
-    //gui->si->currentstaff = pi.the_staff;
-    gui->si->currentmeasurenum = pi.measure_number;
-    gui->si->currentmeasure = pi.the_measure;
-    gui->si->currentobject = pi.the_obj;
-    gui->si->cursor_x = pi.cursor_x;
-    gui->si->cursor_appending
-      =
-      (gui->si->cursor_x ==
-       (gint) (g_list_length ((objnode *) gui->si->currentmeasure->data)));
-    set_cursor_y_from_click (gui, event->y);
-    if(pi.nextmeasure)
-      movetomeasureright(NULL); //!!!!!!!!!!! measureleft - are these extending the selection??????
-
-    write_status(gui);
-
-
-  }
-
 
   if(left) {
     // if(!(GDK_CONTROL_MASK&event->state))
@@ -555,7 +570,7 @@ scorearea_button_press (GtkWidget * widget, GdkEventButton * event)
  
   }
   /* Redraw to show new cursor position, note a real draw is needed because of side effects on display*/
-  gtk_widget_draw (gui->scorearea, NULL);
+  gtk_widget_draw (Denemo.scorearea, NULL);
     
   set_cursor_for(event->state | (left?GDK_BUTTON1_MASK:GDK_BUTTON3_MASK));
   perform_command(event->state | (left?GDK_BUTTON1_MASK:GDK_BUTTON3_MASK), GESTURE_PRESS, left);
@@ -572,6 +587,8 @@ gint
 scorearea_button_release (GtkWidget * widget, GdkEventButton * event)
 {
 DenemoGUI *gui = Denemo.gui;
+ if(gui==NULL || gui->si==NULL)
+   return FALSE;
  gboolean left = (event->button != 3);
 
   if(dragging_separator) {
@@ -579,7 +596,7 @@ DenemoGUI *gui = Denemo.gui;
     return TRUE;
   }
 
- //g_signal_handlers_block_by_func(gui->scorearea, G_CALLBACK (scorearea_motion_notify), gui); 
+ //g_signal_handlers_block_by_func(Denemo.scorearea, G_CALLBACK (scorearea_motion_notify), gui); 
  if(left)
    lh_down = FALSE;
  selecting = FALSE;
@@ -592,12 +609,14 @@ DenemoGUI *gui = Denemo.gui;
 gint
 scorearea_scroll_event (GtkWidget *widget, GdkEventScroll *event) {
   DenemoGUI *gui = Denemo.gui;
+  if(gui==NULL || gui->si==NULL)
+    return FALSE;
   switch(event->direction) {
     DenemoScriptParam param;
   case GDK_SCROLL_UP:
     if(event->state&GDK_CONTROL_MASK) {
       Denemo.gui->si->zoom *= 1.1;
-      scorearea_configure_event(Denemo.gui->scorearea, NULL);
+      scorearea_configure_event(Denemo.scorearea, NULL);
       // displayhelper(gui);
       //update_vscrollbar (gui); these do not seem to work ...
       //update_hscrollbar (gui);
@@ -616,7 +635,7 @@ scorearea_scroll_event (GtkWidget *widget, GdkEventScroll *event) {
       Denemo.gui->si->zoom /= 1.1;
       if(Denemo.gui->si->zoom <0.01)
 	Denemo.gui->si->zoom = 0.01;
-      scorearea_configure_event(Denemo.gui->scorearea, NULL);
+      scorearea_configure_event(Denemo.scorearea, NULL);
       //displayhelper(gui);
     } else
     if(event->state&GDK_SHIFT_MASK) {
