@@ -18,6 +18,7 @@ initkeyaccs (gint * accs, gint number);
 /**
  * Free the given object
  * @param mudobj the DenemoObject to free
+FIXME is this failing to free the object field???
  */
 void
 freeobject (DenemoObject * mudobj)
@@ -33,14 +34,24 @@ freeobject (DenemoObject * mudobj)
       break;
     case CLEF:
       free_directives(((clef*)mudobj->object)->directives);
+      g_free (mudobj->object);
       g_free (mudobj);
-      break;
+      break; 
     case KEYSIG:
       free_directives(((keysig*)mudobj->object)->directives);
+      g_free (mudobj->object);
       g_free (mudobj);
-      break;
+      break; 
     case TIMESIG:
       free_directives(((timesig*)mudobj->object)->directives);
+      g_free (mudobj->object);
+      g_free (mudobj);
+      break; 
+
+    case TUPOPEN:
+    case TUPCLOSE:
+      free_directives(((tuplet*)mudobj->object)->directives);
+      g_free (mudobj->object);
       g_free (mudobj);
       break; 
     default:
@@ -258,7 +269,7 @@ free_directive_data(DenemoDirective *directive) {
     }
     if(directive->widget && G_IS_OBJECT(directive->widget)) {
       //g_print("We should destroy the widget now ");
-      GtkWidget *texteditor =  (GtkWidget*)g_object_get_data(directive->widget, DENEMO_TEXTEDITOR_TAG);
+      GtkWidget *texteditor =  (GtkWidget*)g_object_get_data(G_OBJECT(directive->widget), DENEMO_TEXTEDITOR_TAG);
       if(texteditor)
 	gtk_widget_destroy(texteditor);//FIXME we may need to destroy its parents
       gtk_widget_destroy((GtkWidget *)directive->widget);
@@ -297,10 +308,13 @@ dnm_clone_object (DenemoObject * orig)
 	      break;
 	    
 	    case TUPOPEN:
-	      ret  = (DenemoObject *)newtupopen(((tupopen*)orig->object)->numerator, ((tupopen*)orig->object)->denominator);	  
+	      ret  = (DenemoObject *)newtupopen(((tupopen*)orig->object)->numerator, ((tupopen*)orig->object)->denominator);	  	      ((tupopen *)ret->object)->directives = clone_directives(((tupopen *)orig->object)->directives);
+
 	      break;
 	    case TUPCLOSE:
 	      ret  = (DenemoObject *)newtupclose();
+	      ((tupopen *)ret->object)->directives = clone_directives(((tupopen *)orig->object)->directives);
+	      
 	      break;
 	    case CLEF:
 	      ret = dnm_newclefobj (((clef *)orig->object)->type);
@@ -319,6 +333,9 @@ dnm_clone_object (DenemoObject * orig)
 	      break;
 	    case STEMDIRECTIVE:
 	      ret = dnm_stem_directive_new  (((stemdirective *)orig->object)->type);
+
+	      ((stemdirective *)ret->object)->directives = clone_directives(((stemdirective *)orig->object)->directives);
+
 	      break;
 	    case MEASUREBREAK:
 	      ret = newmeasurebreakobject ();
@@ -478,7 +495,7 @@ dnm_newobj (DenemoObjType type)
 void
 dnm_setinitialkeysig (DenemoStaff * curstaff, gint tokey, gint type)
 {
-
+  take_snapshot();
   curstaff->keysig.number = tokey;
   curstaff->keysig.isminor = type;
 
