@@ -25,7 +25,6 @@
 struct callbackdata
 {
   DenemoPrefs *prefs;
-  GtkWidget *shortcut_filename;
   GtkWidget *lilypath;
   GtkWidget *midi_audio_output;
   GtkWidget *immediateplayback;
@@ -34,6 +33,7 @@ struct callbackdata
   GtkWidget *persistence;
   GtkWidget *startmidiin;
   GtkWidget *applytoselection;
+  GtkWidget *quickshortcuts;
   GtkWidget *saveparts;
   GtkWidget *autosave;
   GtkWidget *toolbar;
@@ -78,6 +78,7 @@ struct callbackdata
 #endif
   GtkWidget *display_refresh;
   GtkWidget *animation_steps;
+  GtkWidget *profile;
   GtkWidget *midiplayer;
   GtkWidget *audioplayer;
   GtkWidget *denemopath;
@@ -137,14 +138,13 @@ set_preferences (struct callbackdata *cbdata)
 #define ASSIGNCOMBO2(field) \
   prefs->field = get_midi_audio_pointer(\
     (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (cbdata->field)->entry)));
-  ASSIGNTEXT(shortcut_filename)
   ASSIGNTEXT(lilypath)
   ASSIGNTEXT(browser) 
   ASSIGNTEXT(pdfviewer)
   ASSIGNTEXT(imageviewer)
   ASSIGNTEXT(username)
   ASSIGNTEXT(password)
-    //ASSIGNTEXT(midiplayer)
+  ASSIGNTEXT(profile)
   ASSIGNTEXT(audioplayer)
   ASSIGNTEXT(denemopath)
     // ASSIGNTEXT(sequencer)
@@ -188,6 +188,7 @@ set_preferences (struct callbackdata *cbdata)
   ASSIGNBOOLEAN(cursor_highlight)
   ASSIGNBOOLEAN(startmidiin)
   ASSIGNBOOLEAN(applytoselection)
+  ASSIGNBOOLEAN(quickshortcuts)
   ASSIGNBOOLEAN(autosave)
   ASSIGNINT(autosave_timeout)
   ASSIGNBOOLEAN(articulation_palette)
@@ -412,11 +413,14 @@ preferences_change (GtkAction *action, gpointer param)
    * Preferences to do with commands
    */
   NEWPAGE("Command Behavior");
-
+  TEXTENTRY("Profile", profile)
   //  TEXTENTRY("Strict", strictshortcuts)
-  BOOLEANENTRY("Apply commands to selection if present", applytoselection);  
+  BOOLEANENTRY("Apply commands to selection if present", applytoselection); 
+  BOOLEANENTRY("Allow Quick Setting of Shortcuts", quickshortcuts); 
+
+ 
   BOOLEANENTRY("Strict Shortcuts", strictshortcuts);
-  TEXTENTRY("Shortcut File", shortcut_filename);
+
   /*
    * External (Helper) Programs 
    */
@@ -427,7 +431,7 @@ preferences_change (GtkAction *action, gpointer param)
   TEXTENTRY("File/Internet Browser", browser)
 
   TEXTENTRY("Image Viewer", imageviewer)
-    //TEXTENTRY("Midi Player", midiplayer)
+
   TEXTENTRY("Audio Player", audioplayer)
 
   TEXTENTRY("Default Save Path", denemopath)
@@ -477,7 +481,7 @@ preferences_change (GtkAction *action, gpointer param)
     gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
     label = gtk_label_new ("");	
     gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-    gtk_label_set_markup(GTK_LABEL (label), _("<span background=\"red\">Warning: changes only have effect after quitting and re-starting Denemo</span>"));
+    gtk_label_set_markup(GTK_LABEL (label), _("<span background=\"#FFA0A0\">Warning: changes only have effect after quitting and re-starting Denemo</span>"));
     gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   }
