@@ -81,7 +81,7 @@ initprefs ()
 
   /* Reasonable default values */
 
-  ret->mode = INPUTEDIT|INPUTRHYTHM;
+  ret->mode = INPUTEDIT|INPUTRHYTHM|INPUTNORMAL;
 
 
   ret->csoundcommand = g_string_new ("csound -dm6");
@@ -111,6 +111,7 @@ initprefs ()
   ret->lilypath = g_string_new ("lilypond.exe");//We don't assume the file assoc works - we are installing this anyway to a known place,the option  neither lilypond-windows.exe nor the -dgui option are used
   ret->pdfviewer = g_string_new ("");
   ret->imageviewer = g_string_new ("");
+
   ret->midiplayer = g_string_new("");
 #else /* !G_OS_WIN32 */
   ret->browser = g_string_new ("firefox");
@@ -126,8 +127,6 @@ initprefs ()
   ret->denemopath = g_string_new (g_get_home_dir());
   ret->lilyversion = g_string_new ("");//meaning use installed LilyPond version
   ret->temperament = g_string_new("Equal");
-  if(ret->shortcut_filename==NULL)
-    ret->shortcut_filename = g_string_new("");
   ret->strictshortcuts = FALSE;
   ret->resolution = 300;
   ret->display_refresh = 0.01;
@@ -135,6 +134,7 @@ initprefs ()
   ret->overlays = FALSE;
   ret->continuous = TRUE;
   ret->cursor_highlight = TRUE;
+
 #ifdef _HAVE_JACK_
   ret->immediateplayback = FALSE;
 #else
@@ -184,6 +184,7 @@ initprefs ()
   ret->zoom = 100;
   ret->system_height = 100;
   ret->applytoselection = TRUE;
+  ret->quickshortcuts = TRUE;
   
   /* Read values from systemwide preferences file */
 
@@ -201,6 +202,7 @@ initprefs ()
   g_free (systemwide);
   g_free (localrc);
 #undef ret
+
 }
 
 
@@ -246,7 +248,7 @@ parsePorts (xmlDocPtr doc, xmlNodePtr cur, gint i)
 	  xmlChar *tmp = xmlNodeListGetString (doc, child->xmlChildrenNode, 1);
 	  if(tmp)
 	    {
-	      g_print("storing port %s into client\n", tmp);
+	      //g_print("storing port %s into client\n", tmp);
 	      Denemo.prefs.midi_device[i].ports[j].midi_buffer = g_malloc0(DENEMO_BUFFER_MAX_INDEX*sizeof(MidiBuffer));
 	      Denemo.prefs.midi_device[i].ports[j].port_name = g_string_new(tmp);
 	      j++;
@@ -292,7 +294,7 @@ parseDevices (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
       if (xmlStrcmp (child->name, (const xmlChar *) "device") == 0)
 	{
 	  gchar *tmp = (gchar *) xmlGetProp (child, (xmlChar *) "client");
-g_print("storing client name %s\n", tmp);
+	  //g_print("storing client name %s\n", tmp);
 	  if(tmp)
 	    {
 	      if(prefs->midi_device[i].client_name)
@@ -303,7 +305,7 @@ g_print("storing client name %s\n", tmp);
 	    }
 	}
       xmlNodePtr ports = child->xmlChildrenNode;
-      g_print("for client ports is %p\n", ports);   
+      // g_print("for client ports is %p\n", ports);   
       if (ports && xmlStrcmp (ports->name, (const xmlChar *) "ports") == 0)
 	{
 	  parsePorts(doc, ports, i);
@@ -413,7 +415,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  parseDevices(doc, cur, &Denemo.prefs);
 
 	}
-      // see below - do not read READXMLENTRY(shortcut_filename)      
+
 	READXMLENTRY(midiplayer)      
 	READXMLENTRY(audioplayer)        
 	READXMLENTRY(browser)
@@ -454,6 +456,8 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
       
       READXMLENTRY(pdfviewer)
       READXMLENTRY(imageviewer)    
+      READXMLENTRY(profile) 
+	
       READXMLENTRY(username)    
       READXMLENTRY(password)           
       READXMLENTRY(denemopath)          
@@ -468,6 +472,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
       READBOOLXMLENTRY(persistence) 
       READBOOLXMLENTRY(cursor_highlight) 
       READBOOLXMLENTRY(applytoselection) 
+      READBOOLXMLENTRY(quickshortcuts) 
       READBOOLXMLENTRY(startmidiin) 
       READINTXMLENTRY(mode) 
   
@@ -507,6 +512,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
       
       cur = cur->next;
     }
+
   return;
 }
 
@@ -761,13 +767,14 @@ writeXMLPrefs (DenemoPrefs * prefs)
    
 
   WRITEXMLENTRY(lilypath)
-    // do not store WRITEXMLENTRY(shortcut_filename) it is for internal use. .denemo/Default.xxx is used for the customized shortcuts.
+
   WRITEXMLENTRY(midiplayer)
   WRITEXMLENTRY(audioplayer)
   WRITEXMLENTRY(csoundcommand)
   WRITEXMLENTRY(csoundorcfile)  
   WRITEXMLENTRY(pdfviewer)
   WRITEXMLENTRY(imageviewer)
+  WRITEXMLENTRY(profile)
   WRITEXMLENTRY(username)
   WRITEXMLENTRY(password)
   WRITEXMLENTRY(denemopath)
@@ -820,6 +827,7 @@ writeXMLPrefs (DenemoPrefs * prefs)
   WRITEBOOLXMLENTRY(persistence)
   WRITEBOOLXMLENTRY(cursor_highlight)
   WRITEBOOLXMLENTRY(applytoselection)
+  WRITEBOOLXMLENTRY(quickshortcuts)
   WRITEBOOLXMLENTRY(startmidiin)
   WRITEINTXMLENTRY(mode)
   WRITEBOOLXMLENTRY(strictshortcuts)
