@@ -13,17 +13,16 @@ void
 printexcerptpreview_cb(GtkAction *action, gpointer param);
 void
 printpart_cb(GtkAction *action, gpointer param);
-gchar *
-get_printfile_pathbasename(void);
-void
-run_lilypond_and_viewer(gchar *basename, DenemoGUI *gui);
-void export_pdf_action (GtkAction *action, gpointer param);
-
-void install_printpreview(DenemoGUI *gui, GtkWidget *vbox);
+void 
+install_printpreview(DenemoGUI *gui, GtkWidget *vbox);
 void
 refresh_print_view(gboolean interactive);
 gchar *
 get_lily_version_string (void);
+void 
+print_lily_cb (GtkWidget *item, DenemoGUI *gui);
 void
-run_lilypond(gchar *filename, DenemoGUI *gui);
+export_pdf (gchar *filename, DenemoGUI * gui);
+void
+export_png (gchar *filename, gboolean show_preview, DenemoGUI * gui);
 #endif /*PRINT_H*/	
