@@ -222,22 +222,30 @@ register_command(Denemo.map, gtk_action_group_get_action(action_group, "SharpenE
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "FlattenEnharmonicSet"), "FlattenEnharmonicSet", "Shift Accidentals Flatwise", "Shifts the set of accidentals one step flatter", set_flatter);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "New"), "New", "New File", "Start a new musical score", file_newwrapper);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "Open"), "Open", "Open", "Open a file containing a music score for editing", file_open_with_check);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ImportLilypond"), "ImportLilypond", "Import Lilypond", "Import a Lilypond file", file_import_lilypond_with_check);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ImportMidi"), "ImportMidi", "Import Midi", "Import a Midi file", file_import_midi_with_check);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ImportMusicXml"), "ImportMusicXml", "Import MusicXml", "Import a MusicXml file", file_import_musicxml_with_check);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "AddStaffs"), "AddStaffs", "Add Staffs", "Add staffs from a Denemo file", file_add_staffs);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "AddMovements"), "AddMovements", "Add Movement", "Add movements from a Denemo file", file_add_movements);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "MovementProps"), "MovementProps", "Change Properties", "Change properties of this movement", movement_props_dialog);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "OpenNewWindow"), "OpenNewWindow", "Open In New", "Open a file containing a music score for editing in a separate working area (tab", openinnew);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "Save"), "Save", "Save", "Save the score", file_savewrapper);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "SaveAs"), "SaveAs", "Save As", "Save the score under a new name", file_saveaswrapper);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "SaveCopy"), "SaveCopy", "Create Copy", "Save a copy of the score", file_copy_save);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "OpenTemplate"), "OpenTemplate", "Open Template", "Start a new score from a built-in template file", system_template_open_with_check);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "OpenExample"), "OpenExample", "Open Example", "Start a new score from a built-in example", system_example_open_with_check);
-register_command(Denemo.map, gtk_action_group_get_action(action_group, "OpenMyTemplate"), "OpenMyTemplate", "Open custom template", "Start a new score from one of your own template files", local_template_open_with_check);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "OpenMyTemplate"), "OpenMyTemplate", "Open Custom Template", "Start a new score from one of your own template files", local_template_open_with_check);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "SaveTemplate"), "SaveTemplate", "Save Template", "Save the score as a template for re-use as a starting point for new scores", template_save);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "NewWindow"), "NewWindow", "New Tab", "Create working area (tab with an empty score in it)", newview);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "InsertMovementBefore"), "InsertMovementBefore", "Insert Movement Before", "Insert a new movement before the current one", insert_movement_before);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "InsertMovementAfter"), "InsertMovementAfter", "Insert Movement After", "Insert a new movement after the current one", insert_movement_after);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "NewMovement"), "NewMovement", "New Movement", "Create a new movement, usign any default template", append_new_movement);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "SaveParts"), "SaveParts", "Save Parts", "Save Parts: each staff becomes a file in lilypond format", file_savepartswrapper);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ExportMUDELA"), "ExportMUDELA", "Export Lilypond", "Export the score as a lilypond file", export_mudela_action);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "ExportPDF"), "ExportPDF", "Export PDF", "Export the score as a PDF document file", export_pdf_action);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ExportPNG"), "ExportPNG", "Export PNG", "Export the score as a PNG image file", export_png_action);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ExportMIDI"), "ExportMIDI", "Export MIDI", "Export the score as a MIDI file", export_midi_action);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ExportCSOUND"), "ExportCSOUND", "Export Csound", "Export the score as a Csound Score file", export_csound_action);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "ConfigureScore"), "ConfigureScore", "Score Wizard", "Start up a wizard to create a new score. This allows you to set various properties of the score", scorewizard);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "PrintPreview"), "PrintPreview", "Print Preview", "Displays the final finished score in your pdf viewer", printpreview_cb);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "PrintSelection"), "PrintSelection", "Print Selection", "Displays selected music from score in your pdf viewer", printselection_cb);
@@ -287,9 +295,9 @@ register_command(Denemo.map, gtk_action_group_get_action(action_group, "DeleteVe
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "EditFiguredBass"), "EditFiguredBass", "Insert/Edit Figured Bass", "Add a bass figure to the current note. Use | sign to split the duration of a note so as to have multiple figures on one note. See Lilypond docs for other notation", figure_insert);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "DeleteFiguredBass"), "DeleteFiguredBass", "Delete Figures", "Delete the figured bass on the current staff", delete_figured_bass);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "HideFiguredBass"), "HideFiguredBass", "Hide Figures (Print)", "Hide the figured bass on the current staff on printing", hide_figured_bass);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "ShowFiguredBass"), "ShowFiguredBass", "Show Figures (Print)", "Show the figured bass on the current staff on printing", show_figured_bass);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "EditChords"), "EditChords", "Edit Chord Symbols", "Allows chord symbols to be added to the current note. E.G.cis:dim7 for c-sharp diminished 7th. See Lilypond docs for notation", fakechord_insert);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "InsertDynamic"), "InsertDynamic", "Insert Dynamics", "Inserts a dynamic marking at the cursor position", insert_dynamic);
-register_command(Denemo.map, gtk_action_group_get_action(action_group, "InsertLilyDirective"), "InsertLilyDirective", "Insert Lilypond", "Insert or edit a directive in the LilyPond music typesetting language. This can be used for extra spacing, transposing or almost anything. See LilyPond documentation for ideas.", standalone_directive);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "EditObject"), "EditObject", "Edit Object", "Edit the object at the cursor.", edit_object);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "EditDirective"), "EditDirective", "Edit Directives", "Edit any directives attached to chord/note at cursor.", edit_object_directive);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "EditStaffDirective"), "EditStaffDirective", "Edit Staff Directives", "Edit any directives attached to staff.", edit_staff_directive);
@@ -306,8 +314,8 @@ register_command(Denemo.map, gtk_action_group_get_action(action_group, "InsertBa
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "GoToMeasure"), "GoToMeasure", "Go to Measure", "Opens a dialog for going to a numbered measure", tomeasurenum);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "GoToBeginning"), "GoToBeginning", "Go to Beginning", "Cursor to start of staff/voice, extending selection if any", tohome);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "GoToEnd"), "GoToEnd", "Go to End", "Cursor to end of staff/voice, extending selection if any", toend);
-register_command(Denemo.map, gtk_action_group_get_action(action_group, "MoveToBeginning"), "MoveToBeginning", "Move to Beginning", "Cursor to start of staff/voice, without extending selection if any", movetostart);
-register_command(Denemo.map, gtk_action_group_get_action(action_group, "MoveToEnd"), "MoveToEnd", "Move to End", "Cursor to end of staff/voice, without extending selection if any", movetoend);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "MoveToBeginning"), "MoveToBeginning", "Move to Staff/Voice Beginning", "Cursor to start of staff/voice, without extending selection if any", movetostart);
+register_command(Denemo.map, gtk_action_group_get_action(action_group, "MoveToEnd"), "MoveToEnd", "Move to Staff/Voice End", "Cursor to end of staff/voice, without extending selection if any", movetoend);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "NextMovement"), "NextMovement", "Next Movement", "Go to the next movement", next_movement);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "PreviousMovement"), "PreviousMovement", "Previous Movement", "Go to the previous movement", prev_movement);
 register_command(Denemo.map, gtk_action_group_get_action(action_group, "DeleteMovement"), "DeleteMovement", "Delete Movement", "Delete the current movement", delete_movement);
