@@ -895,6 +895,18 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "New")), "scm", (gpointer)
 SCM scheme_Open(SCM optional);
 install_scm_function ("d-Open", scheme_Open);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "Open")), "scm", (gpointer)1);
+/*ImportLilypond file_import_lilypond_with_check*/
+SCM scheme_ImportLilypond(SCM optional);
+install_scm_function ("d-ImportLilypond", scheme_ImportLilypond);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ImportLilypond")), "scm", (gpointer)1);
+/*ImportMidi file_import_midi_with_check*/
+SCM scheme_ImportMidi(SCM optional);
+install_scm_function ("d-ImportMidi", scheme_ImportMidi);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ImportMidi")), "scm", (gpointer)1);
+/*ImportMusicXml file_import_musicxml_with_check*/
+SCM scheme_ImportMusicXml(SCM optional);
+install_scm_function ("d-ImportMusicXml", scheme_ImportMusicXml);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ImportMusicXml")), "scm", (gpointer)1);
 /*AddStaffs file_add_staffs*/
 SCM scheme_AddStaffs(SCM optional);
 install_scm_function ("d-AddStaffs", scheme_AddStaffs);
@@ -919,6 +931,10 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "Save")), "scm", (gpointer
 SCM scheme_SaveAs(SCM optional);
 install_scm_function ("d-SaveAs", scheme_SaveAs);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "SaveAs")), "scm", (gpointer)1);
+/*SaveCopy file_copy_save*/
+SCM scheme_SaveCopy(SCM optional);
+install_scm_function ("d-SaveCopy", scheme_SaveCopy);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "SaveCopy")), "scm", (gpointer)1);
 /*OpenTemplate system_template_open_with_check*/
 SCM scheme_OpenTemplate(SCM optional);
 install_scm_function ("d-OpenTemplate", scheme_OpenTemplate);
@@ -955,10 +971,26 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "NewMovement")), "scm", (g
 SCM scheme_SaveParts(SCM optional);
 install_scm_function ("d-SaveParts", scheme_SaveParts);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "SaveParts")), "scm", (gpointer)1);
+/*ExportMUDELA export_mudela_action*/
+SCM scheme_ExportMUDELA(SCM optional);
+install_scm_function ("d-ExportMUDELA", scheme_ExportMUDELA);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ExportMUDELA")), "scm", (gpointer)1);
 /*ExportPDF export_pdf_action*/
 SCM scheme_ExportPDF(SCM optional);
 install_scm_function ("d-ExportPDF", scheme_ExportPDF);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ExportPDF")), "scm", (gpointer)1);
+/*ExportPNG export_png_action*/
+SCM scheme_ExportPNG(SCM optional);
+install_scm_function ("d-ExportPNG", scheme_ExportPNG);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ExportPNG")), "scm", (gpointer)1);
+/*ExportMIDI export_midi_action*/
+SCM scheme_ExportMIDI(SCM optional);
+install_scm_function ("d-ExportMIDI", scheme_ExportMIDI);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ExportMIDI")), "scm", (gpointer)1);
+/*ExportCSOUND export_csound_action*/
+SCM scheme_ExportCSOUND(SCM optional);
+install_scm_function ("d-ExportCSOUND", scheme_ExportCSOUND);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ExportCSOUND")), "scm", (gpointer)1);
 /*ConfigureScore scorewizard*/
 SCM scheme_ConfigureScore(SCM optional);
 install_scm_function ("d-ConfigureScore", scheme_ConfigureScore);
@@ -1155,6 +1187,10 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "DeleteFiguredBass")), "sc
 SCM scheme_HideFiguredBass(SCM optional);
 install_scm_function ("d-HideFiguredBass", scheme_HideFiguredBass);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "HideFiguredBass")), "scm", (gpointer)1);
+/*ShowFiguredBass show_figured_bass*/
+SCM scheme_ShowFiguredBass(SCM optional);
+install_scm_function ("d-ShowFiguredBass", scheme_ShowFiguredBass);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ShowFiguredBass")), "scm", (gpointer)1);
 /*EditChords fakechord_insert*/
 SCM scheme_EditChords(SCM optional);
 install_scm_function ("d-EditChords", scheme_EditChords);
@@ -1163,10 +1199,6 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "EditChords")), "scm", (gp
 SCM scheme_InsertDynamic(SCM optional);
 install_scm_function ("d-InsertDynamic", scheme_InsertDynamic);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "InsertDynamic")), "scm", (gpointer)1);
-/*InsertLilyDirective standalone_directive*/
-SCM scheme_InsertLilyDirective(SCM optional);
-install_scm_function ("d-InsertLilyDirective", scheme_InsertLilyDirective);
-g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "InsertLilyDirective")), "scm", (gpointer)1);
 /*EditObject edit_object*/
 SCM scheme_EditObject(SCM optional);
 install_scm_function ("d-EditObject", scheme_EditObject);
