@@ -6,13 +6,14 @@
 
 #include <denemo/denemo.h>
 #include <stdio.h>
+#include <string.h>
 #include "config.h"
 #include "staffops.h"
 #include "scoreops.h"
 #include "prefops.h"
 #include "selectops.h"
-#define INITIAL_WHOLEWIDTH 160
-#define INITIAL_STAFFHEIGHT 100
+#include "objops.h"
+#include "lyric.h"
 
 static remove_all_staffs(DenemoScore * si);
 
@@ -40,6 +41,7 @@ point_to_new_movement (DenemoGUI *gui)
 {
   point_to_empty_movement(gui);
   newstaff (gui, INITIAL, DENEMO_NONE);
+  gui->si->undo_guard = Denemo.prefs.disable_undo;
 }
 
 static void
@@ -60,6 +62,8 @@ append_movement(GtkAction *action, gpointer param,  gboolean populate) {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *source_movement = gui->si;
   GList *g;
+  if(gui->si->lyricsbox)
+    gtk_widget_hide(gui->si->lyricsbox);
   point_to_empty_movement(gui);
   for(g=source_movement->thescore;g;g=g->next) {
     DenemoStaff *source_staff = g->data;
@@ -71,7 +75,7 @@ append_movement(GtkAction *action, gpointer param,  gboolean populate) {
     copy_staff(source_staff, dest_staff);
   }
   gui->movements = g_list_append(gui->movements, gui->si);
-
+  gui->si->undo_guard = Denemo.prefs.disable_undo;
   set_width_to_work_with(gui);
   //FIXME duplicate code
   set_rightmeasurenum (gui->si);
@@ -79,9 +83,9 @@ append_movement(GtkAction *action, gpointer param,  gboolean populate) {
   set_bottom_staff (gui);
   update_hscrollbar (gui);
   update_vscrollbar (gui);
-  gtk_widget_queue_draw (gui->scorearea);
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
+  gtk_widget_queue_draw (Denemo.scorearea);
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.vadjustment), "changed");
   displayhelper(gui);
   score_status(gui, TRUE); 
 }
@@ -145,39 +149,60 @@ delete_movement(GtkAction *action, gpointer param) {
   score_status(gui, TRUE);
 }
 
-/* go to the movement voice measure staff and object numbers (starting from 1) */
+/* go to the movement voice measure staff and object numbers (starting from 1)
+ movementnum<=0 means current movement
+possible_gui is really a flag interactive or not
+*/
 gboolean
-goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint measurenum, gint objnum)
+goto_movement_staff_obj (DenemoGUI * possible_gui, gint movementnum, gint staffnum, gint measurenum, gint objnum)
 {
-  GList *this = g_list_nth(gui->movements, movementnum-1);
-
-  if(this==NULL){
-    warningdialog(_("No such movement"));
-    return FALSE;
+  DenemoGUI *gui;
+  if(possible_gui==NULL)
+    gui = Denemo.gui;
+  else
+    gui = possible_gui;
+  if(movementnum>0) {
+    GList *this = g_list_nth(gui->movements, movementnum-1);  
+    if(this==NULL){
+      if(possible_gui)
+	warningdialog(_("No such movement"));
+      return FALSE;
+    }
+    gtk_widget_hide(gui->si->buttonbox);
+    gui->si = this->data;
+    gtk_widget_show(gui->si->buttonbox);
   }
-  gtk_widget_hide(gui->si->buttonbox);
-  gui->si = this->data;
-  gtk_widget_show(gui->si->buttonbox);
+
+
   if(!moveto_currentstaffnum (gui, staffnum))
     {
-      warningdialog(_("No such voice"));
+      if(possible_gui)
+	warningdialog(_("No such voice"));
       return FALSE;
     }
   if(!moveto_currentmeasurenum(gui, measurenum))
   {
-    warningdialog(_("No such measure"));
+    if(possible_gui)
+      warningdialog(_("No such measure"));
     return FALSE;
   }
+  //cursor_x is zero and we are on the first, if any, object
+  if(gui->si->currentobject==NULL && objnum)
+    return FALSE;//asking for an object in an empty measure
+
   while(--objnum>0  && gui->si->currentobject && gui->si->currentobject->next) {
     gui->si->currentobject = gui->si->currentobject->next;
     gui->si->cursor_x++;
   }
   // g_print("objnum %d\n",objnum);
-  if(objnum>0)
-  gui->si->cursor_x+=objnum;
-  if(!gui->si->currentobject)
-    return FALSE;
+  if(objnum>0) {
+    gui->si->cursor_x++;// objnum;
+    if(!gui->si->currentobject)
+      return FALSE;
+    gui->si->cursor_appending = TRUE;
+  }
   write_status(gui);
+  //g_print("%s and %d\n", gui->si->currentobject?"Obj":"noObj", gui->si->cursor_x);
 #if 0
   //something bad here re-sets the staff to 1
   updatescoreinfo (gui);
@@ -187,9 +212,9 @@ goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint
 #endif
   update_hscrollbar (gui);
   update_vscrollbar (gui);
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
-  gtk_widget_queue_draw (gui->scorearea);
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.vadjustment), "changed");
+  gtk_widget_queue_draw (Denemo.scorearea);
   return TRUE;
 }
 
@@ -202,7 +227,7 @@ void PopPosition (GtkAction * action, DenemoScriptParam *param) {
     param->status = FALSE;
     return;
   }
-  param->status = goto_movement_staff_obj(Denemo.gui, pos->movement, pos->staff, pos->measure, pos->object);
+  param->status = goto_movement_staff_obj(NULL, pos->movement, pos->staff, pos->measure, pos->object);
   if(param->status)
     {
       Denemo.gui->si->cursor_appending = pos->appending;
@@ -222,7 +247,7 @@ void PopPushPosition (GtkAction * action, DenemoScriptParam *param) {
     param = &dummy;
   if(pos) {
     push_position();
-    param->status = goto_movement_staff_obj(Denemo.gui, pos->movement, pos->staff, pos->measure, pos->object);
+    param->status = goto_movement_staff_obj(NULL, pos->movement, pos->staff, pos->measure, pos->object);
   if(param->status)
     {
       Denemo.gui->si->cursor_appending = pos->appending;
@@ -270,9 +295,9 @@ next_movement (GtkAction *action, DenemoScriptParam *param)
 
   update_hscrollbar (gui);
   update_vscrollbar (gui);
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
-  gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME see staffup/down
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.vadjustment), "changed");
+  gtk_widget_draw (Denemo.scorearea, NULL);//KLUDGE FIXME see staffup/down
 }
 
 /**
@@ -311,10 +336,10 @@ prev_movement (GtkAction *action, DenemoScriptParam *param)
 
   update_hscrollbar (gui);
   update_vscrollbar (gui);
-  gtk_widget_queue_draw (gui->scorearea);
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
-  gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
- gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME see staffup/down
+  gtk_widget_queue_draw (Denemo.scorearea);
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.vadjustment), "changed");
+ gtk_widget_draw (Denemo.scorearea, NULL);//KLUDGE FIXME see staffup/down
 }
 /**
  * Initialise scoreinfo structure 
@@ -329,9 +354,9 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   si->readonly = 0;
   si->leftmeasurenum = si->rightmeasurenum = 1;
   si->top_staff = si->bottom_staff = 1;
-  si->measurewidth = INITIAL_WHOLEWIDTH;
+  si->measurewidth = DENEMO_INITIAL_MEASURE_WIDTH;
   si->measurewidths = NULL;
-  si->staffspace = INITIAL_STAFFHEIGHT;
+  si->staffspace = DENEMO_INITIAL_STAFF_HEIGHT;
   si->thescore = NULL;
   si->currentstaffnum = 1;
   si->currentmeasurenum = 1;
@@ -341,6 +366,7 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   si->staffletter_y = 0;
   si->cursor_appending = TRUE;
   si->zoom = Denemo.prefs.zoom>0?Denemo.prefs.zoom/100.0:1.0;
+  si->preview_zoom = 1.0;
   si->system_height = Denemo.prefs.system_height>0?Denemo.prefs.system_height/100.0:1.0;
 
   si->cursoroffend = FALSE;
@@ -371,6 +397,8 @@ init_score (DenemoScore * si, DenemoGUI *gui)
     gui->autosavename = g_string_new (dir);
     gui->autosavename = g_string_append (gui->autosavename, "/autosave.denemo");
   }
+  if(gui->tabname==NULL)
+    gui->tabname = g_string_new ("");
   si->curlilynode = 0;	 /* the node of the lily parse tree on display in 
 			    textwindow */
   si->lily_file = 0;   /* root of lily file parse, see lilyparser.y etc  */
@@ -381,7 +409,7 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   /* Undo/redo initialisation */
   si->undodata = g_queue_new ();
   si->redodata = g_queue_new ();
-  si->undo_redo_mode = UNDO;
+  si->undo_guard = 1;//do not collect undo information until file is loaded
   
   si->buttonbox = gtk_hbox_new(FALSE, 1);
   gtk_box_pack_end (GTK_BOX (gui->buttonboxes), si->buttonbox, FALSE, TRUE,
@@ -402,6 +430,18 @@ static delete_all_staffs(DenemoGUI * gui) {
   return TRUE;
 }
 
+
+static
+GList *extract_verses(GList *verses){
+  //g_warning("extract_verses not tested!!!!!!!");
+  GList *ret = NULL;
+  GList *g;
+  for(g=verses;g;g=g->next) {
+    GtkTextView *srcVerse = g->data;
+    ret = g_list_append(ret, get_text_from_view(GTK_WIDGET(srcVerse)));
+  }
+  return ret;
+}
 /**
  * frees the data in the passed scoreinfo stucture 
  *
@@ -427,3 +467,196 @@ free_score (DenemoGUI * gui)
   g_queue_free (gui->si->redodata);
 }
 
+DenemoScore * clone_movement(DenemoScore *si) {
+  DenemoScore *newscore = (DenemoScore *) g_malloc0 (sizeof (DenemoScore));
+  memcpy(newscore, si, sizeof(DenemoScore));
+
+  GList *g;
+  newscore->measurewidths = NULL;
+  for(g=si->measurewidths;g;g=g->next)
+    newscore->measurewidths = g_list_append(newscore->measurewidths, g->data);
+  newscore->playingnow = NULL;
+
+  for(newscore->thescore=NULL, g=si->thescore;g;g=g->next) {
+    DenemoStaff *thestaff = (DenemoStaff*)g_malloc(sizeof(DenemoStaff));
+    DenemoStaff *srcStaff = (DenemoStaff*)g->data;
+    // copy_staff(srcStaff, thestaff);!!!!!! does not copy e.g. no of lines ... need proper clone code.
+    memcpy(thestaff, srcStaff, sizeof(DenemoStaff));
+    
+    thestaff->denemo_name = g_string_new(srcStaff->denemo_name->str);
+    thestaff->lily_name = g_string_new(srcStaff->lily_name->str);
+    thestaff->midi_instrument = g_string_new(srcStaff->midi_instrument->str);
+
+    thestaff->clef.directives = clone_directives(srcStaff->clef.directives);
+    thestaff->keysig.directives = clone_directives(srcStaff->keysig.directives);
+    thestaff->timesig.directives = clone_directives(srcStaff->timesig.directives);
+
+    if(srcStaff->leftmost_clefcontext==&srcStaff->clef)
+       thestaff->leftmost_clefcontext = &thestaff->clef;
+     else
+       // has to be fixed up after the measures are done..., so do the whole thing after???
+       //							  likewise keysig timesig
+       g_warning("Not doing clef context yet..."), thestaff->leftmost_clefcontext = &thestaff->clef;
+
+    if(srcStaff->leftmost_timesig==&srcStaff->timesig)
+       thestaff->leftmost_timesig = &thestaff->timesig;
+     else
+       // has to be fixed up after the measures are done..., so do the whole thing after???
+       //							  likewise keysig timesig
+       g_warning("Not doing timesig context yet..."), thestaff->leftmost_timesig = &thestaff->timesig;
+
+    if(srcStaff->leftmost_keysig==&srcStaff->keysig)
+       thestaff->leftmost_keysig = &thestaff->keysig;
+     else
+       // has to be fixed up after the measures are done..., so do the whole thing after???
+       //							  likewise keysig timesig
+       g_warning("Not doing keysig context yet..."), thestaff->leftmost_keysig = &thestaff->keysig;
+
+
+
+    thestaff->denemo_name = g_string_new(srcStaff->denemo_name->str);
+    thestaff->lily_name = g_string_new(srcStaff->lily_name->str);
+    thestaff->midi_events = NULL;//cached data
+   
+    thestaff->staff_directives = clone_directives(srcStaff->staff_directives);
+    {GList *direc;
+      for(direc=thestaff->staff_directives;direc;direc=direc->next) {
+	DenemoDirective *directive = direc->data;
+	directive->widget = NULL;
+	//	widget_for_staff_directive(directive);
+      }
+    }
+    {GList *direc;
+      for(direc=thestaff->voice_directives;direc;direc=direc->next) {
+	DenemoDirective *directive = direc->data;
+	directive->widget = NULL;
+	//	widget_for_voice_directive(directive);
+      }
+    }
+  
+    newscore->lyricsbox = NULL;
+    thestaff->verses = extract_verses( srcStaff->verses);
+
+    if(srcStaff->currentverse)
+      thestaff->currentverse = g_list_nth(thestaff->verses, g_list_position(srcStaff->verses, srcStaff->currentverse));
+    
+    newscore->thescore = g_list_append(newscore->thescore, thestaff);
+    if(g==si->currentprimarystaff)
+      newscore->currentprimarystaff = newscore->thescore;
+    if(g==si->currentstaff)
+      newscore->currentstaff = newscore->thescore;
+    newscore->currentmeasure = newscore->currentobject = thestaff->measures = NULL;
+    GList *h;
+    for(h=srcStaff->measures;h;h=h->next) {
+      objnode *theobjs = h->data;     
+      GList *newmeasure=NULL;
+      GList *i;
+      for(i=theobjs;i;i=i->next) {
+	DenemoObject *theobj = (DenemoObject *)i->data;
+	DenemoObject *newobj = dnm_clone_object (theobj);
+	newmeasure = g_list_append(newmeasure, newobj);
+	if(i==si->currentobject)
+	  /*g_print("current object %x\n", g_list_last(newmeasure)), */newscore->currentobject = g_list_last(newmeasure);//???
+      }
+      thestaff->measures = g_list_append(thestaff->measures, newmeasure);
+      if(h==si->currentmeasure)
+	/*g_print("current measure %x\n", g_list_last(thestaff->measures)),*/newscore->currentmeasure = g_list_last(thestaff->measures);//???
+    }
+  }
+
+
+
+  newscore->movementcontrol.directives = clone_directives(si->movementcontrol.directives);
+  newscore->layout.directives = clone_directives(si->layout.directives);
+  newscore->header.directives = clone_directives(si->header.directives);
+
+
+
+  /*
+ 
+    
+    smfsync
+    savebuffer
+    bookmarks
+    Instruments
+   
+   
+  */
+  newscore->markstaffnum=0;//Do not clone the selection
+
+
+  return newscore;
+}
+
+
+
+
+/**
+ * Recalculates the stored information about a movement
+ * either gui->si or if that does exist yet, gui->movements->data, the first movement.(FIXME)
+ *
+ * @param gui pointer to the gui structure
+ */
+void
+updatescoreinfo (DenemoGUI * gui)
+{
+  staffnode *curstaff;
+  DenemoScore *si;
+  GList *g = gui->movements;
+  if(g)
+    si = g->data;
+  else
+    si = gui->si;
+  do {
+  for (curstaff = si->thescore; curstaff; curstaff = curstaff->next)
+    {
+      beamsandstemdirswholestaff ((DenemoStaff *) curstaff->data);
+      showwhichaccidentalswholestaff ((DenemoStaff *) curstaff->data);
+    }
+  find_xes_in_all_measures (si);
+  find_leftmost_allcontexts (si);
+
+  si->currentstaff = si->thescore;
+  si->currentmeasure = firstmeasurenode (si->currentstaff);
+  si->currentobject = firstobjnode (si->currentmeasure);
+  if (!si->currentobject)
+    si->cursor_appending = TRUE;
+  else
+    si->cursor_appending = FALSE;
+  si->leftmeasurenum = si->currentstaffnum = si->currentmeasurenum = 1;
+  } while(g && (g=g->next) && (si=g->data));
+  score_status(gui, FALSE);
+}
+
+/**
+ * Delete the movements of the given score and create a new one
+ * with one movement and empty music data, no title
+ * This is the action for the d-New command
+ */
+void
+deletescore (GtkWidget * widget, DenemoGUI * gui)
+{
+  free_movements(gui);
+  score_status(gui, FALSE);
+  if(gui->filename) {
+    g_string_free(gui->filename, TRUE);
+    g_string_free(gui->autosavename, TRUE);
+    gui->filename = NULL;
+    set_title_bar (gui);
+  }
+  if(gui->tabname){
+    g_string_free(gui->tabname, TRUE);
+    gui->tabname = NULL;
+  }
+
+  point_to_new_movement(gui);
+  gui->movements = g_list_append(gui->movements, gui->si);
+  set_width_to_work_with(gui);
+  set_rightmeasurenum (gui->si);
+  update_hscrollbar (gui);
+  update_vscrollbar (gui);
+  gtk_widget_queue_draw (Denemo.scorearea);
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (Denemo.vadjustment), "changed");
+  force_lily_refresh(gui);  
+}
