@@ -80,7 +80,7 @@ static void create_staff_instruments(wizarddata *wdata, gchar *instrument_name,
   //DenemoStaff *curstaffstruct  = g_list_last(wdata->gui->si->currentstaff);
 
   //g_printf("clef # in create_staff_instrument = %i\n", inst->clef);
-  gtk_widget_queue_draw (wdata->gui->scorearea);
+  gtk_widget_queue_draw (Denemo.scorearea);
   /* printf("\nlocation before switch currentstaffnum = %i\n",  */
 /* 	 wdata->gui->si->currentstaffnum); */
   switch((gint) inst->numstaffs)
@@ -256,13 +256,11 @@ static instproperties * lookup_instrument_properties(gchar *name, wizarddata *wd
               instr->clef = (gint *) ((InstrumentConfig *)inst->data)->sclef;
               instr->transposition = (gint *) ((InstrumentConfig *)inst->data)->transposition;
 	      instr->numstaffs = ((InstrumentConfig *)inst->data)->numstaffs;
-#ifdef DEBUG
-	      g_print("instrument name inst->data)->name->str = %s\n", ((InstrumentConfig *)inst->data)->name->str);
-              g_print("number of staves for instrument = %i\n", (gint) ((InstrumentConfig *)inst->data)->numstaffs);
-	      g_print("instrument staff inst->data)->sclef = %i\n", instr->clef);
-	      g_print("transposition = %i\n", (gint) instr->transposition);
-#endif
-	       break;
+	      g_debug("instrument name inst->data)->name->str = %s\n", ((InstrumentConfig *)inst->data)->name->str);
+              g_debug("number of staves for instrument = %i\n", (gint) ((InstrumentConfig *)inst->data)->numstaffs);
+	      g_debug("instrument staff inst->data)->sclef = %i\n", (gint) instr->clef);
+	      g_debug("transposition = %i\n", (gint) instr->transposition);
+	      break;
             }
           inst = g_list_next(inst);
         }
@@ -940,7 +938,7 @@ void scorewizard(GtkAction *action, gpointer param)
 			   G_CALLBACK (gtk_widget_destroy), G_OBJECT(dialog));
 
  
-  gtk_widget_queue_draw (gui->scorearea);
+  gtk_widget_queue_draw (Denemo.scorearea);
   gtk_widget_show_all(dialog);
      
   g_list_free(wdata->instruments);
