@@ -18,9 +18,11 @@
 #include "selectops.h"
 #include "staffops.h"
 #include "prefops.h"
+#include "lyric.h"
+#include "lilydirectives.h"
+#include "scoreops.h"
 /*For save selection function*/
 #include "utils.h"
-#define DEBUG 1
 /**
  * The copy buffer is a GList of objnode *s -- at first, I was going
  * to use staffnode *s, measurenode *s, and then objnode *s, but I
@@ -120,6 +122,9 @@ free_clipboard(GList *clipboard) {
   }
 }
 
+gint get_staffs_in_clipboard(void) {
+  return staffsinbuffer;
+}
 void
 insert_clipboard(GList *clipboard) {
   if(clipboard) {
@@ -251,14 +256,17 @@ cuttobuffer (DenemoScore * si, gboolean copyfirst)
     max;
   if (!si->markstaffnum)
     return;
+  take_snapshot();
   if(copyfirst)
     copytobuffer (si);
   gint staffs_removed_measures = 0;// a count of removed measures in the case where multiple staffs are involved
   gint lmeasurebreaksinbuffer = si->lastmeasuremarked - si->firstmeasuremarked;
   gint lstaffsinbuffer = si->laststaffmarked - si->firststaffmarked + 1;
   if(copyfirst) {
-    g_assert( lmeasurebreaksinbuffer == measurebreaksinbuffer);
-    g_assert(lstaffsinbuffer == staffsinbuffer);
+    if(!( lmeasurebreaksinbuffer == measurebreaksinbuffer))
+      g_warning("logic of copy to buffer seems wrong about measure breaks");
+    if(!(lstaffsinbuffer == staffsinbuffer))
+      g_warning("logic of copy to buffer seems wrong about staff breaks");
   }
   if (lstaffsinbuffer == 1)
     {
@@ -456,15 +464,45 @@ void insert_object(DenemoObject *clonedobj) {
   staffnode *curstaff = si->currentstaff;
   clonedobj->starttick = (si->currentobject?
 			  ((DenemoObject *)si->currentobject->data)->starttickofnextnote: 0);
+  /* update undo information */
+  DenemoUndoData *undo;
+  if (!si->undo_guard)
+    {
+      undo = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));
+      //undo->object = clonedobj;
+      //do position after inserting, so we can go back to it to delete
+    }
+
   Denemo.gui->si->currentmeasure->data =
     g_list_insert ((objnode *)si->currentmeasure->data,
 		   clonedobj, si->cursor_x);
+
+
+ if (!si->undo_guard)
+    {
+      get_position(si, &undo->position);
+      undo->position.appending = 0;
+      undo->action = ACTION_INSERT;
+      update_undo_info (si, undo);
+    }
+
+
+
   si->cursor_x++;
   if (si->cursor_appending)
     si->currentobject = g_list_last ((objnode *) si->currentmeasure->data);
   else
     si->currentobject = g_list_nth ((objnode *) si->currentmeasure->data,
 				    si->cursor_x);
+
+  if(si->currentobject==NULL) {
+    g_warning("problematic parameters on insert %d out of %d objects", si->cursor_x+1, g_list_length((objnode *) si->currentmeasure->data));
+    si->cursor_x--;
+    si->currentobject = g_list_nth ((objnode *) si->currentmeasure->data,
+				    si->cursor_x);
+  }
+
+
   beamsandstemdirswholestaff ((DenemoStaff *) curstaff->data);
   find_xes_in_all_measures (si);
 }
@@ -486,8 +524,17 @@ gboolean insert_clip_obj(gint m, gint n) {
   DenemoObject *curobj = (DenemoObject*)curbufferobj->data; 
   clonedobj = dnm_clone_object (curobj);
   insert_object(clonedobj);
+#if 0
   octave_up_key(Denemo.gui);//FIXME up and down to fix clef change bug !!!!!!!!
   octave_down_key(Denemo.gui);//FIXME up and down to fix clef change bug !!!!!!!!
+#endif
+  //reset_cursor_stats (si);
+  fixnoteheights ((DenemoStaff *) si->currentstaff->data);
+  beamsandstemdirswholestaff ((DenemoStaff *) si->currentstaff->data);
+  find_xes_in_all_measures (si);
+  showwhichaccidentals ((objnode *) si->currentmeasure->data,
+			    si->curmeasurekey, si->curmeasureaccs);
+
   return TRUE;
 }
 /**
@@ -549,10 +596,10 @@ pastefrombuffer (void)
        curstaff? curstaff = si->currentstaff:0, curbuffernode = curbuffernode->next)
     {
 
-      g_debug ("Current staff %x, Current Staff Next %x,"
+      /*g_debug ("Current staff %x, Current Staff Next %x,"
 	       " CurBuf %x, CurBuf Next %x\n",
 	       curstaff, curstaff->next, curbuffernode, curbuffernode->next);
-
+      */
       //gint prevailing_clef = find_prevailing_clef(si);
 
       curmeasure = g_list_nth (firstmeasurenode (curstaff),
@@ -908,15 +955,18 @@ DenemoPosition *pop_position(void) {
   return NULL;
 }
 
-void push_position(void) {
-  DenemoScore *si = Denemo.gui->si;
-  DenemoPosition *pos = ( DenemoPosition *)g_malloc(sizeof(DenemoPosition));
+void get_position(DenemoScore *si, DenemoPosition *pos) {
   pos->movement =  g_list_index(Denemo.gui->movements, si)+1;
   pos->staff =  si->currentstaffnum;
   pos->measure = si->currentmeasurenum;
   pos->object =  si->currentobject?si->cursor_x+1:0;
   pos->appending = si->cursor_appending;
   pos->offend = si->cursoroffend;
+}
+void push_position(void) {
+  DenemoScore *si = Denemo.gui->si;
+  DenemoPosition *pos = ( DenemoPosition *)g_malloc(sizeof(DenemoPosition));
+  get_position(si, pos);
   if(pos->movement)
      positions = g_slist_prepend(positions, pos);
   else
@@ -924,7 +974,11 @@ void push_position(void) {
   //g_print("%d %d %d %d \n", pos->movement, pos->staff, pos->measure, pos->object);
 }
 
-
+static void push_given_position(DenemoPosition *pos) {
+  DenemoPosition *position = (DenemoPosition*)g_malloc(sizeof(DenemoPosition));
+  memcpy(position, pos, sizeof(DenemoPosition));
+  positions = g_slist_prepend(positions, position);
+}
 /**
  *  copywrapper
  *  Wrapper function for the copy command
@@ -971,7 +1025,12 @@ delete_selection(void) {
 void
 pastewrapper (GtkAction *action, DenemoScriptParam *param)
 {  
-  call_out_to_guile("(DenemoPaste)");
+  gboolean success;
+  stage_undo(Denemo.gui->si, ACTION_STAGE_END);//undo is a queue (ie stack) so we push the end first
+  success = !call_out_to_guile("(DenemoPaste)");
+  //FIXME if not success a ACTION_SCRIPT_ERROR will have been put in the undo queue...
+  stage_undo(Denemo.gui->si, ACTION_STAGE_START);
+
   score_status(Denemo.gui, TRUE);
 }
 
@@ -1059,126 +1118,509 @@ redowrapper (GtkAction *action, gpointer param)
   displayhelper (gui);
 }
 
+/* store the passed object as ACTION_CHANGE undo information */
+/* potentially we could optimize the storage of undo information by telescoping changes to the same object when the undo is staged, it would mean keeping a global note of whether the undo is currently staged. We would peek at the head of the queue and if it was an ACTION_CHANGE at the same position we could free the stored object and replace it with the clone created here */
+void store_for_undo_change (DenemoScore *si, DenemoObject *curobj) {
+  if (!si->undo_guard)
+    {
+      DenemoUndoData *data = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));
+      data->object = dnm_clone_object (curobj);
+      get_position(si, &data->position);
+      data->action = ACTION_CHANGE;
+      update_undo_info (si, data);
+    }
+}
+void store_for_undo_measure_insert(DenemoScore *si, gint staffnum, gint measurenum) {
+  if (!si->undo_guard)
+    {
+      DenemoUndoData *data = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));     
+      data->position.staff = staffnum;
+      data->position.measure = measurenum+1;
+      data->action = ACTION_MEASURE_CREATE;
+      update_undo_info (si, data);
+    }
+}
+
+static free_chunk(DenemoUndoData *chunk) {
+g_print("free %d\n", chunk->action);
+  switch (chunk->action) {
+  case ACTION_STAGE_START:
+  case ACTION_STAGE_END:
+  case ACTION_SCRIPT_ERROR:
+    return;//statically allocated
+   
+  case ACTION_INSERT:
+  case ACTION_DELETE:
+  case ACTION_CHANGE:
+    freeobject(chunk->object);
+    g_free(chunk);
+    break;
+    
+  case ACTION_MEASURE_CREATE:
+  case ACTION_MEASURE_REMOVE:
+    g_free(chunk);
+    break;
+  case ACTION_SNAPSHOT:
+    g_warning("Snapshot free is not implemented");
+    g_free(chunk);
+    break;
+  default:
+    g_warning("Uknown type of undo data %d", chunk->action);
+  }
+}
+
+
+
+static DenemoUndoData ActionStageStart={ACTION_STAGE_START};
+static DenemoUndoData  ActionStageEnd = {ACTION_STAGE_END};
+static DenemoUndoData  ActionScriptError = {ACTION_SCRIPT_ERROR};
+void stage_undo(DenemoScore *si, action_type type) {
+  switch(type) {
+  case ACTION_STAGE_START: {
+    if(g_queue_is_empty(si->undodata))
+      return;
+    DenemoUndoData *chunk = g_queue_peek_head(si->undodata);
+    if(chunk->action==ACTION_STAGE_END) {
+      chunk = g_queue_pop_head(si->undodata);
+      // free_chunk(chunk); not needed, is static anyway
+      // g_print("Script did not need undoing");
+    } else
+      update_undo_info (si, &ActionStageStart);
+  }
+    break;
+  case ACTION_STAGE_END:
+    update_undo_info (si, &ActionStageEnd);
+    break;
+  case ACTION_SCRIPT_ERROR:
+    update_undo_info (si, &ActionScriptError);
+    break;
+  default:
+    g_warning("Unknown undo action %d will not be stored\n", type);
+  }
+}
+
+// snapshot the current movement for undo
+gboolean take_snapshot(void) {
+  if (!Denemo.gui->si->undo_guard)
+    {
+      DenemoUndoData *chunk;
+      chunk = (DenemoUndoData *) g_malloc (sizeof (DenemoUndoData));
+      chunk->object = (DenemoObject*)clone_movement(Denemo.gui->si);
+      //fix up somethings...
+      get_position(Denemo.gui->si,&chunk->position);
+      chunk->position.appending = 0;
+      chunk->action = ACTION_SNAPSHOT;
+      update_undo_info (Denemo.gui->si, chunk);
+      
+      return TRUE;
+    } else
+    return FALSE;
+}
+
+static print_queue(gchar *msg, GQueue *q) {
+  GList*g;
+  g_debug("%s",msg);
+  for(g=q->head;g;g=g->next) {
+    DenemoUndoData *chunk = g->data;
+    switch(chunk->action) {
+    case ACTION_STAGE_START:
+      g_print("[");
+      break;
+    case ACTION_STAGE_END:
+      g_print("]\n");
+      break;
+    case ACTION_SNAPSHOT:
+      g_print("Snapshot; ");
+      break;
+    case ACTION_INSERT:
+      g_print("Ins; ");
+      break;
+    case ACTION_DELETE:
+      g_print("Del %s; ", DenemoObjTypeNames[((DenemoObject*) chunk->object)->type]);
+      break;
+    case ACTION_CHANGE:
+      g_print("Chn %s; ", DenemoObjTypeNames[((DenemoObject*) chunk->object)->type]);
+      break;
+    case ACTION_MEASURE_CREATE:
+      g_print("Create; ");
+      break;
+    case ACTION_MEASURE_REMOVE:
+      g_print("Remove; ");
+      break;
+    case ACTION_NOOP:
+      g_print("No-op; ");
+      break;
+    default:
+      g_print("Unknown action %d\n", chunk->action);
+
+    }
+  }
+  g_print("\nEnd queue\n");
+}
+
+
+static gboolean position_for_chunk(DenemoGUI * gui, DenemoUndoData *chunk) {
+  DenemoScriptParam param;
+  param.status=TRUE;
+  //g_print("undo guard before %d level is %d\n undo action is %d\n",  gui->si->undo_guard, gui->undo_level, chunk->action);
+
+  switch(chunk->action) {
+  case ACTION_CHANGE:
+    if(chunk->position.object==0)
+      return FALSE;//Cannot undo a change in an empty measure=>undo queue is corrupt
+    //FALL THRU
+  case ACTION_INSERT:
+  case ACTION_DELETE:
+  case ACTION_MEASURE_CREATE://this creates an (blank)measure
+  case ACTION_MEASURE_REMOVE://this is the action that removes a blank measure at pos
+    { 	
+      push_given_position(&chunk->position);	
+      PopPosition(NULL, &param);
+    }
+    break;
+  case ACTION_NOOP:
+    break;
+  default:
+    break;
+  }
+  return param.status;
+}
+
+//Takes the action needed for one chunk of undo/redo data
+
+static void	action_chunk(DenemoGUI * gui, DenemoUndoData **pchunk) {
+  DenemoUndoData *chunk = *pchunk;
+	switch(chunk->action) {
+	case ACTION_MEASURE_CREATE: {
+	  //delete the empty measure in the chunk->position.staff at measure number chunk->position->object
+	  dnm_deletemeasure(gui->si);
+	  chunk->action = ACTION_MEASURE_REMOVE;
+	  if(chunk->position.measure>1)
+	    chunk->position.measure--;
+	  else
+	    chunk->action = ACTION_NOOP;
+
+	  if(!gui->si->currentmeasure) {
+	    g_warning("position after undo insert Bug in selectops.c");
+	    position_for_chunk(gui, chunk);
+	      //movetoend(NULL, NULL);
+	  }
+	}
+	  break;
+
+	case ACTION_MEASURE_REMOVE: {
+	  //create empty measure in the chunk->position.staff at measure number chunk->position->object
+	  insertmeasureafter(gui);
+	  chunk->action = ACTION_MEASURE_CREATE;
+	  chunk->position.measure++;
+	  if(!gui->si->currentmeasure) {
+	    g_warning("position after undo insert Bug in selectops.c");
+	    position_for_chunk(gui, chunk);//????
+	      //movetoend(NULL, NULL);
+	  }
+	}
+	  break;
+
+
+
+	case ACTION_STAGE_START:
+	  gui->undo_level++;
+
+	  *pchunk = &ActionStageEnd;
+	  break;
+	case ACTION_STAGE_END:
+	  gui->undo_level--;
+	  *pchunk = &ActionStageStart;
+
+	  break;
+	case ACTION_SCRIPT_ERROR:
+	  //chunk = &ActionScriptError;
+	  gui->undo_level = 0;
+	  break;
+
+	case ACTION_INSERT:
+	  {
+	    chunk->object = dnm_clone_object(gui->si->currentobject->data);
+	    dnm_deleteobject (gui->si);	    
+	    chunk->action = ACTION_DELETE;
+	  }
+	  break;
+	case  ACTION_DELETE:
+	  {	    
+	    object_insert (gui, chunk->object);
+	    chunk->action = ACTION_INSERT;
+	    chunk->object = NULL;	    
+	  }
+	  break;
+	case ACTION_CHANGE:
+	  {
+	    //FIXME guard against a corrupt undo queue here by checking  if(gui->si->currentobject) {
+	    DenemoObject *temp = gui->si->currentobject->data;
+	    gui->si->currentobject->data =  chunk->object;
+	    chunk->object = temp;
+	  }
+	  break;
+	case ACTION_SNAPSHOT:
+	  {
+
+	    DenemoScore *si = (DenemoScore*)chunk->object;
+	    gint initial_guard = gui->si->undo_guard;
+	    gint initial_changecount = gui->si->changecount;
+	    gboolean initial_redo_invalid = gui->si->redo_invalid;
+	    gpointer initial_smf = gui->si->smf;
+	    // replace gui->si in gui->movements with si
+	    GList *find = g_list_find(gui->movements, gui->si);
+	    if(find) {
+	      find->data = si;
+	      GList *g, *gorig, *curstaff;
+	      for(curstaff = gui->si->thescore;curstaff;curstaff=curstaff->next) {
+		DenemoStaff *thestaff = curstaff->data;
+		gorig =	g = thestaff->verses;
+		thestaff->verses = NULL;
+		for(;g;g=g->next) {
+		  gchar *text = get_text_from_view (g->data);
+		  gtk_widget_destroy(g->data);//what about its parent??? FIXME
+		  thestaff->verses = g_list_append(thestaff->verses, text);
+		  if(thestaff->currentverse==g)
+		    thestaff->currentverse = g_list_last(thestaff->verses);
+		}
+
+		{GList *direc;
+		  for(direc=thestaff->staff_directives;direc;direc=direc->next) {
+		    DenemoDirective *directive = direc->data;
+		    if(directive->widget) {
+		    gtk_widget_destroy(directive->widget);
+		    directive->widget = NULL;
+		    }
+		    //widget_for_staff_directive(directive);
+		  }
+		}
+		{GList *direc;
+		  for(direc=thestaff->voice_directives;direc;direc=direc->next) {
+		    DenemoDirective *directive = direc->data;
+		    if(directive->widget) {		    
+		    gtk_widget_destroy(directive->widget);
+		    directive->widget = NULL;
+		    }
+		    //widget_for_voice_directive(directive);
+		  }
+		}
+	      }
+
+
+	      {GList *direc;
+	      for(direc=gui->si->movementcontrol.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		gtk_widget_destroy(directive->widget);
+		directive->widget = NULL;
+
+	      }
+	    } 
+	    {GList *direc;
+	      for(direc=gui->si->header.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		gtk_widget_destroy(directive->widget);
+		directive->widget = NULL;
+	      }
+	    }
+	    {GList *direc;
+	      for(direc=gui->si->layout.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		gtk_widget_destroy(directive->widget);
+		directive->widget = NULL;
+	      }
+	    }
+
+	    g_list_free(gorig);
+	    chunk->object = (DenemoObject*)gui->si;
+	    //FIXME fix up other values in stored object si?????? voice/staff directive widgets
+	    gui->si = si;
+	    for(curstaff = si->thescore;curstaff;curstaff=curstaff->next) {
+	      DenemoStaff *thestaff = curstaff->data;
+	      gorig = g = thestaff->verses;
+	      gint curversenum = g_list_position(g, thestaff->currentverse);
+	      thestaff->verses = NULL;
+		
+	      for(;g;g=g->next) {
+		add_verse_to_staff(si, thestaff);
+		gtk_text_buffer_set_text (gtk_text_view_get_buffer ((GtkTextView *) thestaff->currentverse->data), g->data, -1);
+		gtk_widget_show(thestaff->currentverse->data);
+		g_signal_connect (G_OBJECT (gtk_text_view_get_buffer (thestaff->currentverse->data)), "changed", G_CALLBACK (lyric_change), NULL);
+	      }
+	      thestaff->currentverse = g_list_nth(thestaff->verses, curversenum);
+
+
+	      {GList *direc;
+		for(direc=thestaff->staff_directives;direc;direc=direc->next) {
+		  DenemoDirective *directive = direc->data;	    
+		  directive->widget = NULL;   
+		  widget_for_staff_directive(directive, thestaff->staffmenu);
+		}
+	      }
+	      {GList *direc;
+		for(direc=thestaff->voice_directives;direc;direc=direc->next) {
+		  DenemoDirective *directive = direc->data;
+		  directive->widget = NULL;
+		  widget_for_voice_directive(directive, thestaff->voicemenu);
+		}
+	      }
+	      g_list_free(gorig);
+	    }
+	    
+
+	    {GList *direc;
+	      for(direc=gui->si->movementcontrol.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		directive->widget = NULL;
+		widget_for_movementcontrol_directive(directive);
+	      }
+	    } 
+	    {GList *direc;
+	      for(direc=gui->si->header.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		directive->widget = NULL;
+		widget_for_header_directive(directive);
+	      }
+	    }
+	    {GList *direc;
+	      for(direc=gui->si->layout.directives;direc;direc=direc->next) {
+		DenemoDirective *directive = direc->data;
+		directive->widget = NULL;
+		widget_for_layout_directive(directive);
+	      }
+	    }
+	    gui->si->smf = initial_smf;
+	    gui->si->smfsync = -1;//force recalculation of midi
+	    gui->si->redo_invalid = initial_redo_invalid;
+	    gui->si->undo_guard = initial_guard;//we keep all the guards we had on entry which will be removed when
+	    gui->si->changecount = initial_changecount;
+	    position_for_chunk(gui, chunk);//FIXME check return val
+	    if(!gui->si->currentmeasure) {
+	      g_warning("positioning after snapshot Bug in selectops.c");
+	      movetoend(NULL, NULL);
+	    }	    
+	    gui->si->currentstaffnum = 1+g_list_position(gui->si->thescore, gui->si->currentstaff);	    
+	    }
+	    else {
+	      g_critical("Movement does not exist in list of movements");
+	    }
+	  }
+	  break;
+	case ACTION_NOOP:
+	  break;
+
+	default:
+	  g_warning("Unexpected undo case ");
+	}
+}
+
+
+
+
+static void position_warning(DenemoUndoData *chunk) {
+  g_warning("Could not find position for undotype %d  movement %d staff %d measure %d object %d appending %d offend %d\n",
+	    chunk->action,
+	    chunk->position.movement,
+	    chunk->position.staff,
+	    chunk->position.measure,
+	    chunk->position.object,
+	    chunk->position.appending,
+	    chunk->position.offend);
+  print_queue("The undo queue was:", Denemo.gui->si->undodata);
+  print_queue("The redo queue was:", Denemo.gui->si->redodata);
+}
+static void
+warn_no_more_undo(DenemoGUI *gui) 
+{
+  g_warning("No more undo information at level %d guard %d ... resetting", gui->undo_level, gui->si->undo_guard );
+  gui->undo_level = 0;
+  gui->si->undo_guard = Denemo.prefs.disable_undo;
+}
+
+
+
+static free_queue( GQueue *queue) {
+  DenemoUndoData *chunk;
+  //g_print("before redo queue %p is %d empty\n", queue, g_queue_is_empty(queue));
+  while((chunk = (DenemoUndoData *) g_queue_pop_head (queue)))
+    free_chunk(chunk);
+  //g_print("after redo queue %p is %d empty\n", queue, g_queue_is_empty(queue));
+}
 
 /**
  * undo
- * Self explantory - undo's the previous command
+ * Undoes an insert, delete change of a DenemoObject, transferring the undo object to the redo queue and switching it between delete/insert
+ * Undoes other changes to movement by returning to a snapshot.
  *
- * Input
- * scoreinfo - score data
+ * PARAM gui  the score (why??? this is per movement undo FIXME)
  */
 static void
 undo (DenemoGUI * gui)
 {
-  unre_data *undo=NULL;
-  unre_data *redo = (unre_data *) g_malloc (sizeof (unre_data));//FIXME memory leak
 
-  if (gui->notsaved /* consider has the changed been to this movement?? */&& g_queue_get_length (gui->si->undodata) > 0)
-    {
-      gui->si->undo_redo_mode = UNDO;
-      undo = (unre_data *) g_queue_pop_head (gui->si->undodata);
-      memcpy (redo, undo, sizeof (*undo));
-
-      gui->si->currentstaffnum = undo->staffnum;
-      gui->si->currentmeasurenum = undo->measurenum;
-      setcurrents (gui->si);
-calcmarkboundaries (gui->si);
-      gui->si->cursor_x = undo->position;
-      if (undo->action == ACTION_INSERT)
-	{
-	  g_debug ("Undo Action Insert:  Remove Object from score\n");
-	  g_debug ("staffnum %d, measurenum %d, position %d\n",
-		   undo->staffnum, undo->measurenum, undo->position);
-
-	  setcurrentobject (gui->si, gui->si->cursor_x);
-
-	  g_debug ("Position after set_currents %d\n", gui->si->cursor_x);
-
-	  dnm_deleteobject (gui->si);
-	  /*  redo->action = ACTION_DELETE;*/
-	}
-      else if (undo->action == ACTION_DELETE)
-	{
-
-	  object_insert (gui, undo->object);
-
-	  g_debug ("UNDO Delete Object %d\n",
-		   ((DenemoObject *) undo->object)->type);
-	  g_debug ("Cursor position before UNDO %d\n", gui->si->cursor_x);
-	  g_debug ("Cursor Position %d\n", gui->si->cursor_x);
-
-	  setcurrents (gui->si);
-calcmarkboundaries (gui->si);
-	}
-      else if (undo->action == ACTION_CHANGE)
-	{
-	  displayhelper (gui);
-	}
-    }
-
-  /* update_redo_info (gui->si, redo);*/
-
-  if (undo)
-    g_free (undo);
+  DenemoUndoData *chunk = (DenemoUndoData *) g_queue_pop_head (gui->si->undodata);
+  if (chunk)
+    {      
+      gui->si->undo_guard++;
+      //g_print("undo %d\n", chunk->action);
+      if(position_for_chunk(gui, chunk)) {
+	action_chunk(gui, &chunk);
+      } else {
+	position_warning(chunk);
+	free_queue(gui->si->redodata);
+	free_queue(gui->si->undodata);
+	warn_no_more_undo(gui);//returns guard to user preference and sets level 0
+	return;
+      }
+      //g_print("actioned undo now pushing %d\n", chunk->action);
+      update_redo_info (gui->si, chunk);	  
+      gui->si->undo_guard--;
+      //g_print("***undo guard after undo %d\n",  gui->si->undo_guard);
+      if(gui->undo_level>0)
+	undo(gui);
+      score_status(gui, TRUE);
+      //print_queue("Undo, queue: ", gui->si->undodata);
+    } else
+    warn_no_more_undo(gui);
 }
 
+
+
 /**
  * redo
- * Self explanitary - redoes the previous undo command
+ * Takes objects from the redo queue and actions them, staged by ACTION_STAGE_START/END
+ * Once actioned they are transferred back to the undo queue, with inverse transformation
  *
  * Input
  * scoreinfo - score data
  */
 void
 redo (DenemoGUI * gui)
-{
-  unre_data *undo = NULL;
-  unre_data *redo = NULL;
-  
-  DenemoScore *si = gui->si;
-  if (gui->notsaved && g_queue_get_length (si->redodata) > 0)
+{  
+  DenemoUndoData *chunk = (DenemoUndoData *) g_queue_pop_head (gui->si->redodata);
+  if (chunk)
     {
-      si->undo_redo_mode = REDO;
-      redo = (unre_data *) g_queue_pop_head (si->redodata);
+      //g_print("Before %s and %d\n", gui->si->currentobject?"Obj":"noObj", gui->si->cursor_x);
+     gui->si->undo_guard++;
+      if(position_for_chunk(gui, chunk)) {
+	action_chunk(gui, &chunk);
+      } else {
+	position_warning(chunk);
 
-      g_debug ("List length %d\n", g_queue_get_length (si->undodata));
-      g_debug ("ACtion %d\n", redo->action);
-
-      if (redo->action == ACTION_INSERT)
-	{
-	  si->currentstaffnum = redo->staffnum;
-	  si->currentmeasurenum = redo->measurenum;
-	  si->cursor_x = redo->position;
-	  setcurrents (si);
-calcmarkboundaries (si);
-
-	  g_debug ("Position after set_currents %d\n", gui->si->cursor_x);
-
-	  dnm_deleteobject (gui->si);
-	}
-      else if (redo->action == ACTION_DELETE)
-	{
-
-	  g_debug ("Redo Action Insert:  Remove Object from score\n");
-	  g_debug ("staffnum %d, measurenum %d, position %d\n",
-		   redo->staffnum, redo->measurenum, redo->position);
-
-	  si->currentstaffnum = redo->staffnum;
-	  si->currentmeasurenum = redo->measurenum;
-	  si->cursor_x = redo->position;
-	  setcurrents (si);
-calcmarkboundaries (si);
-	  object_insert (gui, (DenemoObject *) redo->object);
-
-	  redo->action = ACTION_DELETE;
-	}
-      else if (redo->action == ACTION_CHANGE)
-	{
-	  g_debug ("Do something useful\n");
-	}
-    }
+      }
+      update_undo_info (gui->si, chunk);	  
+      gui->si->undo_guard--;
+      //g_print("After %s and %d\n", gui->si->currentobject?"Obj":"noObj!!", gui->si->cursor_x);
+      if(gui->undo_level>0)
+	redo(gui);
+      score_status(gui, TRUE);
+    } else
+    warn_no_more_undo(gui);
 }
 
+
+
+
 /**
  *  update_undo_info
  *  
@@ -1187,20 +1629,22 @@ calcmarkboundaries (si);
  *
  */
 void
-update_undo_info (DenemoScore * si, unre_data * undo)
+update_undo_info (DenemoScore * si, DenemoUndoData * undo)
 {
-  unre_data *tmp = NULL;
+  DenemoUndoData *tmp = NULL;
 
-  g_debug ("Undo structure: Action %d, Position %d,  Staff %d, Measure %d\n",
-	   undo->action, undo->position, undo->staffnum, undo->measurenum); 
 
-  if (g_queue_get_length (si->undodata) == MAX_UNDOS)
-    {
-      tmp = g_queue_pop_tail (si->undodata);
-    }
+  //g_print ("Adding: Action %d at pos %d appending %d\n",  undo->action, undo->position.object, undo->position.appending); 
+
+  //  if (g_queue_get_length (si->undodata) == MAX_UNDOS)
+  //    {
+  //      tmp = g_queue_pop_tail (si->undodata);//FIXME freeing undo info, especially the object
+  //      g_warning("Lost undo of %p %p\n", tmp, tmp->object);
+  //   }
 
   g_queue_push_head (si->undodata, undo);
-  
+  si->redo_invalid = TRUE;
+  // print_queue("\nUpdate Undo, queue:", si->undodata);
 }
 
 
@@ -1211,22 +1655,21 @@ update_undo_info (DenemoScore * si, unre_data * undo)
  *  Is passed score structure and redo_data structure
  *  @param si pointer to the DenemoScore structure
  *  @param redo redo data structure to prepend to the queue
- */
+g */
 
 void
-update_redo_info (DenemoScore * si, unre_data * redo)
+update_redo_info (DenemoScore * si, DenemoUndoData * redo)
 {
-  unre_data *tmp = NULL;
-
-  g_debug ("Redo structure: Action %d, Position %d,  Staff %d, Measure %d\n",
-	   redo->action, redo->position, redo->staffnum, redo->measurenum);
+  DenemoUndoData *tmp = NULL;
+  //print_queue("Update redo ******************\nUndo queue:\n", si->undodata);
+  //print_queue("Update redo ******************\nredo queue:\n", si->redodata);
 
 
-  if (g_queue_get_length (si->redodata) == MAX_UNDOS)
+  if(si->redo_invalid)
     {
-      tmp = g_queue_pop_tail (si->redodata);
-      if (tmp)
-	g_free (tmp);
+      free_queue(si->redodata);
+      si->redo_invalid = FALSE;
+      //g_print("queue = %p\n", si->redodata);
     }
   g_queue_push_head (si->redodata, redo);
 }
