@@ -1,8 +1,8 @@
 /* selectops.h
- * selecting, cutting, copying, and pasting music
+ * Undoing, selecting, cutting, copying, and pasting music
  *
  * for Denemo, a gtk+ frontend to GNU Lilypond
- * (c) 1999-2005 Matthew Hiller */
+ * (c) 1999-2005 Matthew Hiller 2010 Richard Shann */
 
 #include <denemo/denemo.h>
 
@@ -65,14 +65,18 @@ void
 saveselwrapper(GtkAction *action, DenemoScriptParam *param);
 void undowrapper(GtkAction *action, gpointer param);
 void redowrapper(GtkAction *action, gpointer param);
-void update_undo_info(DenemoScore *si, unre_data *undo);
-void update_redo_info(DenemoScore *si, unre_data *redo);
+void update_undo_info(DenemoScore *si, DenemoUndoData *undo);
+void update_redo_info(DenemoScore *si, DenemoUndoData *redo);
+void store_for_undo_change (DenemoScore *si, DenemoObject *obj);
+gboolean take_snapshot(void);
+void stage_undo(DenemoScore *si, action_type type);
+
 void goto_mark (GtkAction *action,  DenemoScriptParam *param);
 void goto_selection_start (GtkAction *action,  DenemoScriptParam *param);
 
 DenemoPosition *pop_position(void);
 void push_position(void);
-
+void get_position(DenemoScore *si, DenemoPosition *pos);
 
 DenemoObject *get_mark_object(void);
 DenemoObject *get_point_object(void);
@@ -88,5 +92,5 @@ GList *pop_off_clipboard(void);
 gboolean pop_clipboard(void);
 
 void push_clipboard (void);
-
+gint get_staffs_in_clipboard(void);
 #endif
