@@ -1,4 +1,4 @@
-/* staffpropdialog.cpp
+/* staffpropdialog.c
  * callback that creates a "Staff Properties" dialog box asking
  * the user to change the properties of the current staff
  
@@ -69,7 +69,7 @@ static gchar *instruments[] = {
   "contrabass",
   "tremolo strings",
   "pizzicato strings",
-  "orchestral strings",
+  "orchestral harp",
   "timpani",
   "string ensemble 1",
   "string ensemble 2",
@@ -207,36 +207,10 @@ struct callbackdata
   GtkWidget *volume;
   GtkWidget *mute_volume;
  
-  GtkWidget *context;
-  GtkWidget *staff_prolog_insert;
-  GtkWidget *voice_prolog_insert;
+  
+
 };
 
-/**
- * Convert Context string to denemocontext 
- * 
- * @param string the context string
- * @return denemocontext of the string or -1 if not passed string is not set to a context
- */
-static gint
-setcontext (const gchar * string)
-{
-  if (!strcmp (NONE_STRING, string))
-    return DENEMO_NONE;
-  if (!strcmp (PIANO_START_STRING, string))
-    return DENEMO_PIANO_START;
-  if (!strcmp (PIANO_END_STRING, string))
-    return DENEMO_PIANO_END;
-  if (!strcmp (CHOIR_START_STRING, string))
-    return DENEMO_CHOIR_START;
-  if (!strcmp (CHOIR_END_STRING, string))
-    return DENEMO_CHOIR_END;
-  if (!strcmp (GROUP_START_STRING, string))
-    return DENEMO_GROUP_START;
-  if (!strcmp (GROUP_END_STRING, string))
-    return DENEMO_GROUP_END;
-  return -1;
-}
 
 /**
  * Set the staffs properties 
@@ -308,17 +282,6 @@ set_properties (struct callbackdata *cbdata)
     ASSIGNNUMBER_1(midi_prognum);
     ASSIGNNUMBER_1(midi_channel);
   }
-  g_print("chan prog %d %d\n", staffstruct->midi_channel, staffstruct->midi_prognum); 
-  /* staff context */
-  DenemoContext old = staffstruct->context;
-  staffstruct->context =
-    setcontext (gtk_entry_get_text (GTK_ENTRY (cbdata->context)));
-  if(staffstruct->context==-1)
-    staffstruct->context=old;
-  else
-    if(staffstruct->context != DENEMO_NONE)
-      staffstruct->context |= old;//Allow more than one context to start/end, reset using DENEMO_NONE (0)
-  
 
 #ifdef DEBUG
   g_printf("Staff Transposition %d\n", staffstruct->transposition);
@@ -372,11 +335,11 @@ staff_properties_change (void)
       gui = Denemo.gui;
       si = gui->si;
       staffstruct = (DenemoStaff *) si->currentstaff->data;
-      if(staffstruct->staff_prolog && staffstruct->staff_prolog->len) {
-	warningdialog("This staff has a custom prolog for the staff.\n"
-		      "You will need to make your edits in the LilyPond window\n"
-		      "to see them in the print-out.");
-      }
+     /*  if(staffstruct->staff_prolog && staffstruct->staff_prolog->len) { */
+/* 	warningdialog("This staff has a custom prolog for the staff.\n" */
+/* 		      "You will need to make your edits in the LilyPond window\n" */
+/* 		      "to see them in the print-out."); */
+/*       } */
     }
  
 
@@ -473,14 +436,13 @@ staff_properties_change (void)
 
   
   /*print appearance tab */
-  NEWPAGE("Printout Appearance");
-  //TEXTENTRY("Staff prolog:", staff_prolog_insert); 
-  //TEXTENTRY("Voice prolog:", voice_prolog_insert);
+  // NEWPAGE("Printout Appearance");
+
   
-  GString *s = context_string(staffstruct->context);
-  g_print("\ncontext string = %s\n",s->str);
-  COMBOBOXENTRY("Context:", context, context_list, s);
-  g_string_free(s, TRUE); 
+  //GString *s = context_string(staffstruct->context);
+  // g_print("\ncontext string = %s\n",s->str);
+  //  COMBOBOXENTRY("Context:", context, context_list, s);
+  // g_string_free(s, TRUE); 
  
   /* MIDI tab */
   NEWPAGE("MIDI");
@@ -549,6 +511,8 @@ void staff_properties_change_cb (GtkAction *action, DenemoScriptParam * param) {
    }
    return;
  }
+ take_snapshot();
+
  if(denemo_name) {
    g_string_assign(staff->denemo_name, denemo_name);
     param->status = TRUE;
