@@ -8,6 +8,7 @@
 
 #include <stdio.h>
 #include <string.h> /*for SIGTERM */
+#include <fontconfig/fontconfig.h>
 #include <gtk/gtk.h>
 #include "accwidths.h"
 #include <denemo/denemo.h>
@@ -24,12 +25,15 @@
 #endif
 #include "pitchentry.h"
 
+#ifdef __APPLE__
+#include <mach-o/dyld.h>
+#endif
 
 void add_font_directory(gchar *fontpath) {
 #ifdef G_OS_WIN32
   AddFontResource(fontpath);
 #else
-  ;
+  FcConfigAppFontAddDir(NULL, fontpath);
 #endif
 }
 
@@ -87,6 +91,7 @@ warningdialog (gchar * msg)
 				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE, "%s", msg);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
   gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
   gtk_widget_destroy (dialog);
 }
 
@@ -106,11 +111,104 @@ infodialog (gchar * msg)
   g_signal_connect_swapped (dialog, "response",
 			    G_CALLBACK (gtk_widget_destroy),
 			    dialog);
-  gtk_window_set_keep_above(dialog, TRUE);
+  gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
   gtk_widget_show_all(dialog);
 }
 
+/* data stucture to contain Progressbar 
+ * data
+ */
+typedef struct _ProgressData {
+  GtkWidget *window;
+  GtkWidget *pbar;
+  int timer;
+} ProgressData;
 
+/* Clean up allocated memory */
+static void destroy_progress(ProgressData *pdata)
+{
+  pdata->timer = 0;
+  pdata->window = NULL;
+  g_free (pdata);
+}
+
+static volatile gboolean progressing = TRUE;
+/* Update the value of the progress bar so that we get
+ * some movement */
+static gboolean progress_timeout(gpointer data)
+{
+  ProgressData *pdata = (ProgressData *) data;
+  gdouble new_val;
+
+  /* Calculate the value of the progress bar using the
+   * value range set in the adjustment object */
+#if 0
+  new_val = gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (pdata->pbar)) + 0.01;
+  
+  if (new_val >= 1.0){
+    gtk_widget_destroy (pdata->window);
+    destroy_progress(pdata);
+    return FALSE;
+  }
+
+  /* Set the new value */
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pdata->pbar), new_val);
+#endif
+  if (progressing)
+    gtk_progress_bar_pulse (GTK_PROGRESS_BAR (pdata->pbar));
+  else {
+    gtk_widget_destroy (pdata->window);
+    destroy_progress(pdata);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+/**
+ * Displays progress bar
+ * window should close upon completion
+ * @param msg message to display
+ * @return none
+ */
+void
+progressbar (gchar *msg)
+{
+  ProgressData *pdata;
+  GtkWidget *vbox;
+  
+  /* If this is false the progress bar will stop */
+  progressing = TRUE;
+  /* Allocate memory for the data that is passed to the callbacks */
+  pdata = g_malloc (sizeof (ProgressData));
+
+  /* Replace GTK_WINDOW_TOPLEVEL with GTK_WINDOW_POPUP
+   * to have it witout window decoration. 
+   */
+  pdata->window = gtk_window_new (GTK_WINDOW_POPUP); 
+  gtk_window_set_title (GTK_WINDOW (pdata->window), _("Progress")); 
+ 
+  vbox = gtk_vbox_new (FALSE, 5);
+  gtk_container_add (GTK_CONTAINER (pdata->window), vbox);
+  gtk_widget_show (vbox);
+
+  pdata->pbar = gtk_progress_bar_new();
+  gtk_container_add (GTK_CONTAINER (vbox), pdata->pbar);
+  gtk_widget_show(pdata->pbar);
+  
+  /* set text inside progress bar */
+  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (pdata->pbar), msg);
+ 
+  pdata->timer = g_timeout_add (100, progress_timeout, pdata);  
+		   
+  gtk_window_set_keep_above(GTK_WINDOW (pdata->window), TRUE);
+  gtk_widget_show(pdata->window);
+}
+
+void
+progressbar_stop(){
+  progressing = FALSE;
+}
 /**
  *  Draws the given bitmap mask on to the pixmap using the given 
  *  grahpics context.
@@ -193,12 +291,12 @@ void drawtext_cr (cairo_t *cr, const char *text, double x, double y, double size
 }
 void drawnormaltext_cr (cairo_t *cr, const char *text, double x, double y)
 {
-  drawtext_cr(cr, text, x, y, 9.0);
+  drawtext_cr(cr, text, x, y, 14.0);
 
 }
 void drawlargetext_cr (cairo_t *cr, const char *text, double x, double y)
 {
- drawtext_cr(cr, text, x, y, 18.0);
+ drawtext_cr(cr, text, x, y, 24.0);
 }
 void
 setcairocolor (cairo_t * cr, GdkGC * gc)
@@ -210,6 +308,32 @@ setcairocolor (cairo_t * cr, GdkGC * gc)
   gdk_cairo_set_source_color( cr, &col );
 }
 
+/* draw display text and or graphics for directives
+ return the widest graphic width*/
+gint   draw_for_directives(cairo_t * cr, GList *directives, gint x, gint y) {
+ gint count=10;
+ gint maxwidth=0;
+  for(;directives;directives=directives->next, count+=10) {
+    DenemoDirective *directive = (DenemoDirective *)directives->data;
+    if(directive->graphic) {
+      gint gwidth, gheight;
+      gwidth = directive->graphic->width;
+      gheight = directive->graphic->height;
+
+      maxwidth = MAX(gwidth, maxwidth);
+      //g_print("drawing a graphic at %d %d\n", xx+directive->gx+count-gwidth/2,  y+height+directive->gy-gheight/2);
+      drawbitmapinverse_cr ( cr, directive->graphic,
+			       x+directive->gx+count-gwidth/2,  y+directive->gy-gheight/2);
+      
+    }
+    if(directive->display) {
+      drawnormaltext_cr (cr, directive->display->str, x+directive->tx+count, y+directive->ty ); 
+    }
+  }
+
+  return maxwidth;
+}
+
 /**
  * Utility function to set the number of ticks used by the given object
  * if it is within a given tuplet
@@ -737,6 +861,7 @@ initdir ()
   GError *error=NULL;
   if(!gbr_init (&error)&& (error != (GError *)GBR_INIT_ERROR_DISABLED))
     {
+#ifdef DEBUG
       g_print ("BinReloc failed to initialize:\n");
       g_print ("Domain: %d (%s)\n",
                (int) error->domain,
@@ -745,6 +870,7 @@ initdir ()
       g_print ("Message: %s\n", error->message);
       g_error_free (error);
       g_print ("----------------\n");
+#endif
     }
 #endif /* not G_OS_WIN32 */
 }
@@ -764,7 +890,23 @@ get_data_dir ()
     g_print ("datadir=%s\n", datadir);
     g_free (rootdir);
 #else /* not G_OS_WIN32 */
+
+#ifdef __APPLE__
+     
+      {char path[1024];
+       guint size = sizeof(path);
+       _NSGetExecutablePath(path, &size);
+       gchar * bindir = (gchar*)g_malloc(size);
+       if (_NSGetExecutablePath(bindir, &size) == 0)
+	 g_print("using bin path %s\n", bindir);
+       else
+	 g_critical("Cannot get bin dir\n");
+       datadir = g_build_filename (g_path_get_dirname(bindir), "..", "share", "denemo", NULL);
+       g_print("OSX set data dir to %s\n", datadir);
+      }
+#else
     datadir = gbr_find_pkg_data_dir (PKGDATADIR, PKGNAME);
+#endif
 #endif /* not G_OS_WIN32 */
   }
   return datadir;
@@ -781,7 +923,22 @@ get_conf_dir ()
   confdir = g_build_filename (rootdir, "etc", "denemo", NULL);
   g_free (rootdir);
 #else /* not G_OS_WIN32 */
+#ifdef __APPLE__
+     
+      {char path[1024];
+       guint size = sizeof(path);
+       _NSGetExecutablePath(path, &size);
+       gchar * bindir = (gchar*)g_malloc(size);
+       if (_NSGetExecutablePath(bindir, &size) == 0)
+	 g_print("using bin path %s\n", bindir);
+       else
+	 g_critical("Cannot get bin dir\n");
+       confdir = g_build_filename (g_path_get_dirname(bindir), "..", "etc", "denemo", NULL);
+       g_print("OSX set conf dir to %s\n", confdir);
+      }
+#else
   confdir = gbr_find_etc_dir(SYSCONFDIR);
+#endif
 #endif /* not G_OS_WIN32 */
   }
   return confdir;
@@ -798,6 +955,20 @@ get_locale_dir ()
     localedir = g_build_filename (rootdir, "share", "locale", NULL);
     g_free (rootdir);
 #else /* not G_OS_WIN32 */
+#ifdef __APPLE__
+     
+      {char path[1024];
+       guint size = sizeof(path);
+       _NSGetExecutablePath(path, &size);
+       gchar * bindir = (gchar*)g_malloc(size);
+       if (_NSGetExecutablePath(bindir, &size) == 0)
+	 g_print("using bin path %s\n", bindir);
+       else
+	 g_critical("Cannot get bin dir\n");
+       localedir = g_build_filename (g_path_get_dirname(bindir), "..", "share", "locale", NULL);
+       g_print("OSX set locale dir to %s\n", localedir);
+      }
+#else
 # ifndef ENABLE_BINRELOC
     /* it seems to be the standard way (no binreloc)
      * to set the path of translations this way:
@@ -811,6 +982,7 @@ get_locale_dir ()
      */
     localedir = gbr_find_locale_dir (LOCALEDIR);
 # endif /* ENABLE_BINRELOC */
+#endif
 #endif /* not G_OS_WIN32 */
   }
   return localedir;
@@ -836,14 +1008,20 @@ gchar * music_font(gchar *str) {
 
 void  set_title_bar(DenemoGUI *gui) {
   gchar *title;
-  if(gui->filename && gui->filename->len)
-    title = gui->filename->str;
+  if(gui->tabname && gui->tabname->len)
+    title = gui->tabname->str;
   else
     title = "(Untitled)";
-  title = g_strdup_printf("Denemo - %s%c", title, gui->notsaved?'*':' ');
+  title = g_strdup_printf("%s%c", title, gui->notsaved?'*':' ');
   gtk_window_set_title (GTK_WINDOW (Denemo.window), title); 
   gchar *base = g_path_get_basename (title);
-  gtk_notebook_set_tab_label_text (GTK_NOTEBOOK(Denemo.notebook), gui->page, base); 
+  gint index = g_list_index(Denemo.guis, gui);
+  GtkWidget *page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(Denemo.notebook), index);
+  gtk_notebook_set_tab_label_text (GTK_NOTEBOOK(Denemo.notebook), page, base); 
+
+  gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(Denemo.notebook),page, base);
+  
+                                                       
   g_free(title);
   g_free(base);
 }
@@ -1000,7 +1178,7 @@ void write_status(DenemoGUI *gui) {
   if(gui->si->smf && (gui->si->smfsync == gui->si->changecount) && Denemo.prefs.playback_controls)
     g_string_append_printf(status, "%d min %.2f sec %.2f %.2f", minutes, seconds, early, late);
   else
-    g_string_append_printf(status, " Staff %d Measure %d Object %d %s", gui->si->currentstaffnum, gui->si->currentmeasurenum, gui->si->cursor_x, gui->si->cursor_appending?"Appending":"Not Appending"/*not understood this one... , gui->si->cursoroffend?"Off End":"Not Off End" */);
+    g_string_append_printf(status, " Staff %d Measure %d Position %d %s", gui->si->currentstaffnum, gui->si->currentmeasurenum, gui->si->cursor_x+1, gui->si->cursor_appending?"Appending":"Not Appending"/*not understood this one... , gui->si->cursoroffend?"Off End":"Not Off End" */);
   
   if(Denemo.prefs.midi_in_controls) {
       gchar *thesharp = sharpest();
@@ -1020,27 +1198,7 @@ void write_status(DenemoGUI *gui) {
 void
 write_input_status(void) {
   DenemoGUI *gui = Denemo.gui;
-  GString *text;
-  switch (gui->input_source) { 
-  case INPUTKEYBOARD:
-    gtk_widget_hide(Denemo.input_source);
-    return;
-  case INPUTAUDIO:
-    text = g_string_new("Audio Input: ");
-    break;
-  case INPUTMIDI:
-    text = g_string_new("MIDI Input: ");
-    break;
-  default: g_warning("Unknown input source");
-    break;
-  }
-  if(Denemo.input_filters)
-    g_string_append(text, Denemo.input_filters->str);
-  else
-    g_string_append(text, "No filtering");
-  gtk_label_set_text(GTK_LABEL(Denemo.input_source), text->str);
-  gtk_widget_show(Denemo.input_source);
-  g_string_free(text, TRUE);
+  gtk_label_set_text(GTK_LABEL(Denemo.input_source),  Denemo.input_filters->str);  
 }
 
 /**
@@ -1129,7 +1287,8 @@ string_dialog_entry_with_widget (DenemoGUI *gui, gchar *wlabel, gchar *direction
         gtk_widget_grab_focus (entry);
   	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-	gtk_widget_show_all (dialog);
+	gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
+        gtk_widget_show_all (dialog);
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){ 
 		entry_string = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
@@ -1167,18 +1326,19 @@ gchar * get_option(gchar *str, gint length) {
 		      TRUE, TRUE, 0);
   gchar *opt;
   gint i;
-  GtkWidget *widget1, *widget2, *widget;
+  GtkWidget *widget1, *widget;
   for(opt = str;(opt-str)<length;opt += strlen(opt)+1) {
     if(opt==str) {
-      widget = widget1 =   gtk_radio_button_new_with_label(NULL, opt);
+      widget = widget1 = gtk_radio_button_new_with_label(NULL, opt);
       response = opt;
     } else {
-      widget =  widget2  =   gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (widget1), opt);
+      widget = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (widget1), opt);
     }	
     g_object_set_data(G_OBJECT(widget), "choice", (gpointer)opt);
     g_signal_connect(G_OBJECT(widget), "toggled", G_CALLBACK(option_choice), &response);
     gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
   }
+  gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
   gtk_widget_show_all (dialog);
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_REJECT){ 
     response = NULL;
