@@ -69,6 +69,8 @@ drawlargetext_cr (cairo_t *cr, const char *text, double x, double y);
 void 
 drawtext_cr (cairo_t *cr, const char *text, double x, double y, double size);
 
+gint draw_for_directives(cairo_t *cr, GList *directives, gint x, gint y);
+
 /* Gives space after a note or rest */
 
 gint
@@ -122,7 +124,10 @@ warningdialog(gchar *msg);
 void
 infodialog(gchar *msg);
 
-
+void
+progressbar(gchar *msg);
+void
+progressbar_stop(void);
 
 void 
 headerfields(GtkWidget *dialog, GtkListStore *list_store, 
