@@ -63,9 +63,8 @@ void	  highlight_rest(DenemoGUI *gui, gint dur);
 
 gboolean loadGraphicItem(gchar *name, DenemoGraphic **pgraphic);
 
-void denemo_scheme_init(gchar *initscheme);
 
-void activate_action(gchar *path);
+GtkAction *activate_action(gchar *path);
 
 GError *execute_script_file(gchar *filename);
 
@@ -87,7 +86,7 @@ void
 execute_init_scripts(gchar *menupath);
 
 void
-load_local_scheme_init(void);
+denemo_scheme_init(void);
 
 void 
 execute_scheme(GtkAction *action, DenemoScriptParam *param);
@@ -116,5 +115,6 @@ void ToggleReduceToDrawingArea (GtkAction * action, DenemoScriptParam *param);
 
 gchar *get_midi_control_command(guchar type, guchar value);
 gchar *get_midi_pitch_bend_command(gint value);
-
+gint hide_printarea_on_delete(void);
+void set_midi_in_status(void);
 #endif
